﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DiplomovaPrace
{
    public partial class DialogZdroje : Form
    {
        private Graf graf;
        public DialogZdroje(Graf graf)
        {
            this.graf = graf;
            InitializeComponent();
        }

        private void Vypis()
        {
            zdrojeListView.BeginUpdate();
            try
            {
                zdrojeListView.Items.Clear();
                foreach (Zdroj polozka in graf.zdroje)
                {
                    ListViewItem item = new ListViewItem(polozka.Nazev);
                    item.SubItems.Add(polozka.PuvodniKapacita.ToString());
                    item.SubItems.Add(polozka.AktualniKapacita.ToString());
                    if (polozka.Obnovitelnost == true) item.SubItems.Add("ANO");
                    else item.SubItems.Add("NE");
                    item.SubItems.Add(polozka.Cena.ToString());
                    zdrojeListView.Items.Add(item);
                }
            }
            finally
            {
                zdrojeListView.EndUpdate();
            }
        }

        private void DialogZdroje_Load(object sender, EventArgs e)
        {
            Vypis();
        }

        private void pridatButton_Click(object sender, EventArgs e)
        {
            using (ZdrojDialog dialog = new ZdrojDialog())
            {
                dialog.Zdroj = new Zdroj();
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    if (graf.VlozZdroj(dialog.Zdroj) == 1)
                    {
                        MessageBox.Show("Zdroj nebyl přidán - takový zdroj již existuje.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    ListViewItem item = new ListViewItem(dialog.Zdroj.Nazev);
                    item.SubItems.Add(dialog.Zdroj.PuvodniKapacita.ToString());
                    item.SubItems.Add(dialog.Zdroj.AktualniKapacita.ToString());
                    if (dialog.Zdroj.Obnovitelnost == true) item.SubItems.Add("ANO");
                    else item.SubItems.Add("NE");
                    item.SubItems.Add(dialog.Zdroj.Cena.ToString());
                    zdrojeListView.Items.Add(item);
                }
            }
        }

        private void upravitButton_Click(object sender, EventArgs e)
        {
            if (zdrojeListView.SelectedIndices.Count != 1)
            {
                MessageBox.Show("Není vybrána pouze jedna položka", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            using (ZdrojDialog dialog = new ZdrojDialog())
            {
                Zdroj zdroj = new Zdroj();
                zdroj.Nazev = zdrojeListView.SelectedItems[0].SubItems[0].Text;
                int pom = graf.zdroje.Najdi(zdroj, 0);
                if (pom <= -1)
                {
                    MessageBox.Show("Zdroj nenalezen", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                dialog.Zdroj = (Zdroj)graf.zdroje[pom];
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    zdrojeListView.SelectedItems[0].SubItems[0].Text = dialog.Zdroj.Nazev;
                    zdrojeListView.SelectedItems[0].SubItems[1].Text = dialog.Zdroj.PuvodniKapacita.ToString();
                    zdrojeListView.SelectedItems[0].SubItems[2].Text = dialog.Zdroj.AktualniKapacita.ToString();
                    if (dialog.Zdroj.Obnovitelnost == true) zdrojeListView.SelectedItems[0].SubItems[3].Text = "ANO";
                    else zdrojeListView.SelectedItems[0].SubItems[3].Text = "NE";
                }
            }
            Vypis();
        }

        private void odebratButton_Click(object sender, EventArgs e)
        {
            if (zdrojeListView.SelectedIndices.Count != 1)
            {
                MessageBox.Show("Není vybrána pouze jedna položka", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            Zdroj zdroj = new Zdroj();
            zdroj.Nazev=zdrojeListView.SelectedItems[0].Text;
            if (graf.OdeberZdroj(zdroj) == 1)
                MessageBox.Show("Zdroj nebyl smazán", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else Vypis();
        }

        private void zdrojeListView_DoubleClick(object sender, EventArgs e)
        {
            if (zdrojeListView.SelectedIndices.Count != 1)
            {
                MessageBox.Show("Není vybrána pouze jedna položka", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            using (ZdrojDialog dialog = new ZdrojDialog())
            {
                Zdroj zdroj = new Zdroj();
                zdroj.Nazev = zdrojeListView.SelectedItems[0].SubItems[0].Text;
                int pom = graf.zdroje.Najdi(zdroj, 0);
                if (pom <= -1)
                {
                    MessageBox.Show("Zdroj nenalezen", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                dialog.Zdroj = (Zdroj)graf.zdroje[pom];
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    zdrojeListView.SelectedItems[0].SubItems[0].Text = dialog.Zdroj.Nazev;
                    zdrojeListView.SelectedItems[0].SubItems[1].Text = dialog.Zdroj.PuvodniKapacita.ToString();
                    zdrojeListView.SelectedItems[0].SubItems[2].Text = dialog.Zdroj.AktualniKapacita.ToString();
                    if (dialog.Zdroj.Obnovitelnost == true) zdrojeListView.SelectedItems[0].SubItems[3].Text = "ANO";
                    else zdrojeListView.SelectedItems[0].SubItems[3].Text = "NE";
                }
            }
            Vypis();
        }
    }

    
}
