﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;

namespace DiplomovaPrace
{
    public partial class hlavniOkno : Form
    {
        private DialogVysledky dialogVysledky = new DialogVysledky();
        private DialogNapoveda dialogNapoveda = new DialogNapoveda();
        private DialogVypis dialogVypis = new DialogVypis();
        private int puvodniH = 17;
        private int puvodniV = 17;
        private ParametryBaB parametry = null;
        private NastaveniVzhled nastaveni = new NastaveniVzhled();
        bool zobrazPrastrom = false;
        private Vrchol posledniPosunVrchol = null;
        private Vrchol vrchol1 = null;
        private Point aktualniKurzor = default(Point);
        private Point kurzor = default(Point);
        private String typEditace = null;
        private String predchozitypEditace = null;
        private bool posunVrcholu = false;
        private bool posunHrany = false;
        private Point stareSouradnice = default(Point);
        private Graf graf = new Graf();
        private Graf prastromReseni = new Graf();
        private Graphics g, g3; //zakladní panel, souřadnice, prubeh algoritmu
        public int posledniKlic = new int();
        private string zobrazit = "sitovy graf";
        private Graf puvodniZobrazeni = null;
        private Graf ganttuvDiagram = new Graf();
        private VypocetPostup postup = new VypocetPostup();
        //private Graf[] ganttovyDiagramy = null;
        #region nastaveni vzhledu
        //panel informací o hraně / uzlu
        //bool informaceNad = false;
        private PoleVrcholu optimalniReseni = new PoleVrcholu();
        private Vrchol vrcholInfoListBox = null;
        private Hrana hranaInfoListBox = null;
        private Font infoPolozkaListBoxFont = new Font("Arial", 10);
        private Color infoPolozkaListBoxColor = Color.Black;

        #endregion

        /// <summary>
        /// Umožňuje zobrazení položek v menu a inicializuje grafiku panelů.
        /// </summary>
        public hlavniOkno()
        {
            InitializeComponent();
            g = doubleBufPanel1.CreateGraphics();
            g3 = progressPanel.CreateGraphics();
            ZobrazEditacniMenu();
            doubleBufPanel1.Width += 1;
            doubleBufPanel1.Height += 1;
            graf.ZacatekProjektu = DateTime.Now;
            graf.Nazev = "Nový projekt";
            infoPolozkaListBox.Visible = false;
            progressPanel.Hide();
        }

        /// <summary>
        /// Zobrazí menu pro editaci projektu.
        /// </summary>
        private void ZobrazEditacniMenu()
        {
            editaceGrafuToolStripMenuItem.Visible = true;
            vypoctyToolStripMenuItem.Visible = true;
        }

        /// <summary>
        /// Změní velikost plátna podle souřadnic uzlu.
        /// </summary>
        /// <param name="vrchol">Uzel s nejvyšší hodnotou souřadnice x nebo y</param>
        private void ZmenRozmerPlatna(Vrchol vrchol)
        {
            if (vrchol.SouradniceX + nastaveni.Velikost * 5 * nastaveni.Meritko > doubleBufPanel1.Width) doubleBufPanel1.Width += nastaveni.Velikost * 10 * nastaveni.Meritko;
            if (vrchol.SouradniceY + nastaveni.Velikost * 5 * nastaveni.Meritko > doubleBufPanel1.Height) doubleBufPanel1.Height += nastaveni.Velikost * 10 * nastaveni.Meritko;

        }

        /// <summary>
        /// Inicializuje nastavení projektu. Vymaže kompletně celý projekt z paměti.
        /// </summary>
        private void VymazNastaveni()
        {
            graf = null;
            puvodniZobrazeni = null;
            prastromReseni = null;
            graf = new Graf();
            prastromReseni = new Graf();
            puvodniH = 17;
            puvodniV = 17;
            zobrazPrastrom = false;
            posledniPosunVrchol = null;
            vrchol1 = null;
            aktualniKurzor = default(Point);
            kurzor = default(Point);
            typEditace = null;
            posunVrcholu = false;
            posunHrany = false;
            stareSouradnice = default(Point);
            posledniKlic = new int();
            doubleBufPanel1.Width = panel1.Width + 1;
            doubleBufPanel1.Height = panel1.Height + 1;
            clearButton();
        }

        private void novyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ZobrazEditacniMenu();
            ZmenaZobrazeni("sitovy graf");
            VymazNastaveni();
            if (cpmTypToolStripMenuItem.Checked == true) graf.TypOhodnoceniHran = "CPM"; else graf.TypOhodnoceniHran = "PERT";
            doubleBufPanel1.Invalidate();
        }

        private void grafickáEditaceToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void tabulkováEditaceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (puvodniZobrazeni!=null)
                using (TabulkaDialog dialog = new TabulkaDialog(puvodniZobrazeni, posledniKlic))
                {
                    dialog.PosledniKlic = posledniKlic;
                    if (dialog.ShowDialog() == DialogResult.OK)
                    {
                    }
                }
            else
            using (TabulkaDialog dialog = new TabulkaDialog(graf, posledniKlic))
            {
                dialog.PosledniKlic = posledniKlic;
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                }
            }
        }

        private void otevritToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ZmenaZobrazeni("sitovy graf");
            if (cpmTypToolStripMenuItem.Checked == true) graf.TypOhodnoceniHran = "CPM"; else graf.TypOhodnoceniHran = "PERT";
            try
            {
                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    VymazNastaveni();
                    switch (openFileDialog.FilterIndex)
                    {
                        case 1:
                            using (StreamReader sr = new StreamReader(File.OpenRead(openFileDialog.FileName)))
                            {
                                graf.NactiZeSouboru(sr);
                            }
                            break;
                        case 2:
                            using (BinaryReader br = new BinaryReader(File.OpenRead(openFileDialog.FileName)))
                            {
                                graf.NactiZeSouboru(br);
                            }
                            break;
                    }
                    int maximumX = 0;
                    int maximumY = 0;
                    foreach (Vrchol polozka in graf.vrcholy)
                    {
                        if (polozka.SouradniceX > maximumX) maximumX = polozka.SouradniceX;
                        if (polozka.SouradniceY > maximumY) maximumY = polozka.SouradniceY;
                    }
                    if ((maximumX + nastaveni.Velikost * nastaveni.Meritko * 10) > doubleBufPanel1.Width)
                    {
                        puvodniH = 17 + (maximumX + nastaveni.Velikost * nastaveni.Meritko * 10) - doubleBufPanel1.Width;
                        doubleBufPanel1.Width = maximumX + nastaveni.Velikost * nastaveni.Meritko * 10;
                    }
                    if ((maximumY + nastaveni.Velikost * nastaveni.Meritko * 10) > doubleBufPanel1.Height)
                    {
                        puvodniV = 17 + (maximumY + nastaveni.Velikost * nastaveni.Meritko * 10) - doubleBufPanel1.Height;
                        doubleBufPanel1.Height = maximumY + nastaveni.Velikost * nastaveni.Meritko * 10;
                    }
                    ZobrazEditacniMenu();
                    nazevProjektToolStripTextBox.Text = graf.Nazev;
                    posledniKlic = graf.vrcholy.Count;
                    doubleBufPanel1.Invalidate();
                }
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message, "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                VymazNastaveni();
                //throw;
            }
            if (graf.TypOhodnoceniHran == "PERT")
            {
                pertTypToolStripMenuItem.Checked = true;
                cpmTypToolStripMenuItem.Checked = false;
            }
        }

        private void ulozitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (puvodniZobrazeni != null) graf = puvodniZobrazeni;
            saveFileDialog.FileName = graf.Nazev;
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                switch (saveFileDialog.FilterIndex)
                {
                    case 1:
                        using (StreamWriter sw = new StreamWriter(File.Create(saveFileDialog.FileName)))
                        {
                            graf.UlozDoSouboru(sw);
                        }
                        break;
                    case 2:
                        using (BinaryWriter bw = new BinaryWriter(File.Create(saveFileDialog.FileName)))
                        {
                            graf.UlozDoSouboru(bw);
                        }
                        break;
                }
                editaceGrafuToolStripMenuItem.Visible = true;
            }
        }

        private void doubleBufPanel1_MouseWheel(object sender, MouseEventArgs e)
        {
            MessageBox.Show("Kolečko myši reaguje, zvlastni ze to neudelalo drive");
            int cisloOtoceni = e.Delta * SystemInformation.MouseWheelScrollLines / 120;
            nastaveni.ZoomTest += cisloOtoceni;
            doubleBufPanel1.Invalidate();
        }

        private void doubleBufPanel1_MouseDown(object sender, MouseEventArgs e)
        {
            graf.NajdiNejblizsiUzel(e.X, e.Y, nastaveni);
            graf.NajdiNejblizsiHranu(e.X, e.Y, nastaveni);
            if (typEditace == "vrchol" && e.Button == MouseButtons.Left && graf.NejblizsiUzel == null)
            {
                Vrchol vrchol = new Vrchol();
                String klic;
                int pom;
                do
                {
                    klic = "V" + posledniKlic.ToString("D4");
                    vrchol.Klic.SetKlic(klic);
                    vrchol.SouradniceX = (int)(e.X / nastaveni.ZoomTest);
                    vrchol.SouradniceY = (int)(e.Y / nastaveni.ZoomTest);
                    vrchol.Nazev = klic;
                    pom = graf.VlozVrchol(vrchol);
                    ZmenRozmerPlatna(vrchol);
                    posledniKlic++;
                } while (pom >= 1);
                graf.kritickaCesta.Clear();
                prastromReseni.Init();
                doubleBufPanel1.Invalidate();
            }
            else
                if (typEditace == "vrchol" && e.Button == MouseButtons.Left && graf.NejblizsiUzel != null)
                {
                    posunVrcholu = !posunVrcholu;
                    posledniPosunVrchol = graf.NejblizsiUzel;
                    if (posunVrcholu == false) graf.XorNuluj();
                }
            if (typEditace == "hrana" && e.Button == MouseButtons.Right && vrchol1 != null)
            {
                vrchol1 = null;
                return;
            }
            if (typEditace == "hrana" && e.Button == MouseButtons.Left && graf.NejblizsiUzel != null)
            {
                if (vrchol1 == null) vrchol1 = graf.NejblizsiUzel;
                else
                {
                    Hrana hrana = new Hrana();
                    hrana.Vrchol1.SetKlic(vrchol1.Klic.GetKlic());
                    hrana.Vrchol2.SetKlic(graf.NejblizsiUzel.Klic.GetKlic());
                    hrana.Data.Ohodnoceni = 0;
                    hrana.Data.Nazev = vrchol1.Klic.GetKlic() + graf.NejblizsiUzel.Klic.GetKlic();
                    hrana.Data.Zkratka = "H" + graf.vrcholy.Count.ToString() + graf.naslednici.Count.ToString();
                    int chyba = 0;
                    if ((chyba = graf.VlozHranu(hrana)) >= 1)
                    {
                        switch (chyba)
                        {
                            case 1: MessageBox.Show("Chyba, nenalezeny vrcholy hrany.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                break;
                            case 2: MessageBox.Show("Nelze vkládat násobné hrany");
                                break;
                            case 3: MessageBox.Show("Nelze vkládat smyčky. Graf musí být acyklický.");
                                break;
                        }
                        vrchol1 = null;
                        return;
                    }
                    graf.kritickaCesta.Clear();
                    prastromReseni.Init();
                    doubleBufPanel1.Invalidate();
                    vrchol1 = null;
                }
            }
            else if (typEditace == "hrana" && e.Button == MouseButtons.Left && graf.NejblizsiUzel == null && graf.NejblizsiHrana != null)
            {
                posunHrany = !posunHrany;
                stareSouradnice.X = e.X;
                stareSouradnice.Y = e.Y;
            }
        }

        private void doubleBufPanel1_Paint(object sender, PaintEventArgs e)
        {
            if (zobrazPrastrom == false)
            {
                e.Graphics.ScaleTransform(nastaveni.ZoomTest, nastaveni.ZoomTest);
                graf.Kresli(e.Graphics, nastaveni);
                if (typEditace == "hrana" && vrchol1 != null)
                {
                    int x1 = (int)(vrchol1.SouradniceX);
                    int y1 = (int)(vrchol1.SouradniceY);
                    int x2 = (int)(aktualniKurzor.X / nastaveni.ZoomTest);
                    int y2 = (int)(aktualniKurzor.Y / nastaveni.ZoomTest);
                    e.Graphics.DrawLine(new Pen(Color.Black, 2), new Point(x1, y1), new Point(x2, y2));
                }
            }
            else
            {
                graf.kreslitBaB = true;
                prastromReseni.Kresli(e.Graphics, nastaveni);
                graf.kreslitBaB = false;
            }
            if (zobrazit == "ganttuv diagram" && nastaveni.ZobrazitGrid == true)
            {
                graf = puvodniZobrazeni;
                double minimalniOhodnoceni = double.MaxValue;
                foreach (Hrana polozka in graf.naslednici)
                {
                    if (polozka.Data.Ohodnoceni < minimalniOhodnoceni && polozka.Data.Ohodnoceni > 0) minimalniOhodnoceni = polozka.Data.Ohodnoceni;
                }
                graf = ganttuvDiagram;
                Pen paintPero = new Pen(new SolidBrush(Color.Black));
                paintPero.DashPattern = new float[] { 2.0F, 3.0F, 2.0F, 3.0F };
                for (int i = 0; ((i * nastaveni.Meritko * 50) / minimalniOhodnoceni) < doubleBufPanel1.Width; i += 10)
                {
                    int x1 = 100 + (int)((i * nastaveni.Meritko * 50) / minimalniOhodnoceni);
                    e.Graphics.DrawLine(paintPero, new Point(x1, 0), new Point(x1, doubleBufPanel1.Height * nastaveni.Velikost));
                    e.Graphics.DrawString(((double)i / 2).ToString(), nastaveni.PismoFont, new SolidBrush(Color.Black), new Point(x1, 0));
                }
            }
        }

        private void doubleBufPanel1_MouseMove(object sender, MouseEventArgs e)
        {
            kurzor.X = e.X;
            kurzor.Y = e.Y;
            if (posunVrcholu == false)
                graf.NajdiNejblizsiUzel(e.X, e.Y, nastaveni);
            graf.NajdiNejblizsiHranu(e.X, e.Y, nastaveni);
            if (typEditace == "vrchol" && (graf.NejblizsiUzel != null || posunVrcholu == true)
                && (e.Button == MouseButtons.Left || posunVrcholu == true))
            {
                if (graf.NejblizsiUzel != null) graf.NastavIncidenceVrcholuXOR(graf.NejblizsiUzel);
                else graf.NejblizsiUzel = posledniPosunVrchol;
                graf.NejblizsiUzel.SouradniceX = (int)(e.X / nastaveni.ZoomTest);
                graf.NejblizsiUzel.SouradniceY = (int)(e.Y / nastaveni.ZoomTest);
                ZmenRozmerPlatna(graf.NejblizsiUzel);
            }
            else posunVrcholu = false;
            if (typEditace == "hrana" && posunHrany == false)
            {
                aktualniKurzor.X = e.X;
                aktualniKurzor.Y = e.Y;
            }
            else
                if (typEditace == "hrana" && posunHrany == true && graf.NejblizsiHrana != null)
                {
                    int dx = e.X - stareSouradnice.X;
                    int dy = e.Y - stareSouradnice.Y;
                    Vrchol pomVrchol = new Vrchol();
                    Vrchol pomVrchol1;
                    pomVrchol.Klic.SetKlic(graf.NejblizsiHrana.Vrchol1.GetKlic());
                    pomVrchol1 = (Vrchol)graf.vrcholy[graf.vrcholy.Najdi(pomVrchol, 0)];
                    pomVrchol1.SouradniceX += dx;
                    pomVrchol1.SouradniceY += dy;
                    pomVrchol.Klic.SetKlic(graf.NejblizsiHrana.Vrchol2.GetKlic());
                    pomVrchol1 = (Vrchol)graf.vrcholy[graf.vrcholy.Najdi(pomVrchol, 0)];
                    pomVrchol1.SouradniceX += dx;
                    pomVrchol1.SouradniceY += dy;
                    stareSouradnice.X = e.X;
                    stareSouradnice.Y = e.Y;
                }
                else posunHrany = false;
            doubleBufPanel1.Invalidate();
            if (((typEditace == "vrchol" || typEditace == "PERT") && graf.NejblizsiUzel != null) || (typEditace == "hrana" && graf.NejblizsiHrana != null))
            {
                try
                {
                    if (hranaInfoListBox == graf.NejblizsiHrana && typEditace == "hrana") return;
                    if (vrcholInfoListBox == graf.NejblizsiUzel && typEditace == "vrchol") return;
                    vrcholInfoListBox = graf.NejblizsiUzel;
                    hranaInfoListBox = graf.NejblizsiHrana;
                    infoPolozkaListBox.Items.Clear();
                    infoPolozkaListBox.Visible = true;
                    infoPolozkaListBox.Font = infoPolozkaListBoxFont;
                    infoPolozkaListBox.ForeColor = infoPolozkaListBoxColor;
                    if (nastaveni.FixedInfo == false)
                        infoPolozkaListBox.Location = (new Point(e.X - panel1.HorizontalScroll.Value, e.Y - panel1.VerticalScroll.Value));
                    else infoPolozkaListBox.Location = (new Point(panel1.Size.Width + panel1.HorizontalScroll.Value - infoPolozkaListBox.Width - 10, 0));
                    if (graf.NejblizsiUzel != null && (typEditace == "vrchol" || typEditace == "PERT"))
                    {
                        infoPolozkaListBox.Height = 140;
                        infoPolozkaListBox.Items.Add(graf.NejblizsiUzel.Nazev);
                        infoPolozkaListBox.Items.Add("TE: " + graf.NejblizsiUzel.MinimalniCas.ToString());
                        infoPolozkaListBox.Items.Add("TL: " + graf.NejblizsiUzel.MaximalniCas.ToString());
                        string retezec = "";
                        int pocCinnosti = 0;
                        bool triTecky = false;
                        infoPolozkaListBox.Items.Add("Předchůdci: " + graf.NejblizsiUzel.PocetPredchudcu.ToString());
                        foreach (Hrana predHrana in graf.IncidencePredchudci(graf.NejblizsiUzel))
                        {
                            pocCinnosti++;
                            retezec += predHrana.Data.Zkratka + ",";
                            if (pocCinnosti == 5)
                            {
                                infoPolozkaListBox.Items.Add(retezec);
                                retezec = "";
                            }
                            if (pocCinnosti >= 10)
                            {
                                triTecky = true;
                                break;
                            }
                        }
                        if (retezec.Length > 0)
                            if (triTecky == true)
                                retezec = retezec.Insert(retezec.Length, "...");
                            else retezec = retezec.Remove(retezec.Length - 1, 1);
                        infoPolozkaListBox.Items.Add(retezec);
                        retezec = "";
                        pocCinnosti = 0;
                        triTecky = false;
                        infoPolozkaListBox.Items.Add("Následnici: " + graf.NejblizsiUzel.PocetNasledniku.ToString());
                        foreach (Hrana naslHrana in graf.IncidenceNaslednici(graf.NejblizsiUzel))
                        {
                            pocCinnosti++;
                            retezec += naslHrana.Data.Zkratka + ",";
                            if (pocCinnosti == 5)
                            {
                                infoPolozkaListBox.Items.Add(retezec);
                                retezec = "";
                            }
                            if (pocCinnosti >= 10)
                            {
                                triTecky = true;
                                break;
                            }
                        }
                        if (retezec.Length > 0)
                            if (triTecky == true)
                                retezec = retezec.Insert(retezec.Length, "...");
                            else retezec = retezec.Remove(retezec.Length - 1, 1);
                        infoPolozkaListBox.Items.Add(retezec);
                    }
                    if (graf.NejblizsiHrana != null && typEditace == "hrana")
                    {
                        infoPolozkaListBox.Height = 220;
                        infoPolozkaListBox.Items.Add("Klíč: " + graf.NejblizsiHrana.Vrchol1.GetKlic() + graf.NejblizsiHrana.Vrchol2.GetKlic());
                        infoPolozkaListBox.Items.Add(graf.NejblizsiHrana.Data.Nazev);
                        infoPolozkaListBox.Items.Add("Zkratka: " + graf.NejblizsiHrana.Data.Zkratka);
                        if (graf.TypOhodnoceniHran == "PERT") infoPolozkaListBox.Items.Add("Ohodnocení: " + graf.NejblizsiHrana.Data.A.ToString()+"-"+graf.NejblizsiHrana.Data.A.ToString()+"-"+graf.NejblizsiHrana.Data.B.ToString());
                        else infoPolozkaListBox.Items.Add("Ohodnocení: " + graf.NejblizsiHrana.Data.Ohodnoceni.ToString());
                        infoPolozkaListBox.Items.Add("Celková rezerva: " + String.Format("{0:F2}", graf.NejblizsiHrana.Data.RC));
                        infoPolozkaListBox.Items.Add("Volná rezerva: " + String.Format("{0:F2}",graf.NejblizsiHrana.Data.RV));
                        infoPolozkaListBox.Items.Add("Nezávislá rezerva: " + String.Format("{0:F2}",graf.NejblizsiHrana.Data.RN));
                        string retezec = "";
                        int pocCinnosti = 0;
                        bool triTecky = false;
                        Vrchol pomNejVrchol = new Vrchol(graf.NejblizsiHrana.Vrchol1.GetKlic());
                        pomNejVrchol = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(pomNejVrchol, 0)]);
                        infoPolozkaListBox.Items.Add("Předchůdci: " + pomNejVrchol.PocetPredchudcu.ToString());
                        foreach (Hrana predHrana in graf.IncidencePredchudci(pomNejVrchol))
                        {
                            pocCinnosti++;
                            retezec += predHrana.Data.Zkratka + ",";
                            if (pocCinnosti == 5)
                            {
                                infoPolozkaListBox.Items.Add(retezec);
                                retezec = "";
                            }
                            if (pocCinnosti > 5)
                            {
                                triTecky = true;
                                break;
                            }
                        }
                        if (retezec.Length > 0)
                            if (triTecky == true)
                                retezec = retezec.Insert(retezec.Length, "...");
                            else retezec = retezec.Remove(retezec.Length - 1, 1);
                        infoPolozkaListBox.Items.Add(retezec);
                        retezec = "";
                        pocCinnosti = 0;
                        triTecky = false;
                        infoPolozkaListBox.Items.Add("Následnici: " + pomNejVrchol.PocetNasledniku.ToString());
                        foreach (Hrana naslHrana in graf.IncidenceNaslednici(pomNejVrchol))
                        {
                            pocCinnosti++;
                            retezec += naslHrana.Data.Zkratka + ",";
                            if (pocCinnosti == 5)
                            {
                                infoPolozkaListBox.Items.Add(retezec);
                                retezec = "";
                            }
                            if (pocCinnosti >= 10)
                            {
                                triTecky = true;
                                break;
                            }
                        }
                        if (retezec.Length > 0)
                            if (triTecky == true)
                                retezec = retezec.Insert(retezec.Length, "...");
                            else retezec = retezec.Remove(retezec.Length - 1, 1);
                        infoPolozkaListBox.Items.Add(retezec);
                    }
                }
                catch (Exception) {  }
            }
            else
            {
                infoPolozkaListBox.Visible = false;
                hranaInfoListBox = null;
                vrcholInfoListBox = null;
            }
        }

        /// <summary>
        /// Odškrtne tlačítka na panelu nástorojů.
        /// </summary>
        private void clearButton()
        {
            posunVrcholu = false;
            vrcholToolStripButton.Checked = false;
            hranaToolStripButton.Checked = false;
            clearToolStripButton.Checked = false;
            platnoToolStripButton.Checked = false;
            panel1.Cursor = System.Windows.Forms.Cursors.Default;
        }

        private void vrcholToolStripButton_Click(object sender, EventArgs e)
        {
            typEditace = "vrchol";
            clearButton();
            vrcholToolStripButton.Checked = true;
        }

        private void hranaToolStripButton_Click(object sender, EventArgs e)
        {
            typEditace = "hrana";
            clearButton();
            hranaToolStripButton.Checked = true;
        }

        private void doubleBufPanel1_MouseUp(object sender, MouseEventArgs e)
        {
        }

        private void doubleBufPanel1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (typEditace == "vrchol" && graf.NejblizsiUzel != null && e.Button == MouseButtons.Left)
            {

                using (VrcholDialog dialog = new VrcholDialog())
                {
                    dialog.Vrchol = graf.NejblizsiUzel;
                    if (dialog.ShowDialog() == DialogResult.OK)
                    {
                        graf.kritickaCesta.Clear();
                        prastromReseni.Init();
                    }
                }
            }
            if (typEditace == "hrana" && graf.NejblizsiHrana != null && e.Button == MouseButtons.Left)
            {
                using (HranaDialog dialog = new HranaDialog(graf))
                {
                    dialog.Hrana = graf.NejblizsiHrana;
                    if (dialog.ShowDialog() == DialogResult.OK)
                    {
                        graf.kritickaCesta.Clear();
                        prastromReseni.Init();
                    }
                }
            }
        }

        private void doubleBufPanel1_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right && typEditace == "vrchol" && graf.NejblizsiUzel != null)
            {
                if (graf.OdeberVrchol(graf.NejblizsiUzel) == 1) MessageBox.Show("Vrchol se nepodařilo odebrat.", "Fatální chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                graf.kritickaCesta.Clear();
                prastromReseni.Init();
                doubleBufPanel1.Invalidate();
            }
            if (e.Button == MouseButtons.Right && typEditace == "hrana" && graf.NejblizsiHrana != null)
            {
                graf.kritickaCesta.Clear();
                prastromReseni.Init();
                if (graf.OdeberHranu(graf.NejblizsiHrana) == 1) MessageBox.Show("Hranu se nepodařilo odebrat.", "Fatální chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                doubleBufPanel1.Invalidate();
            }
            if (e.Button == MouseButtons.Left && typEditace == "PERT" && graf.NejblizsiUzel != null && clearToolStripButton.Checked==true)
            {
                using (DialogTerminRezerva dialog = new DialogTerminRezerva())
                {
                    dialog.Uzel = graf.NejblizsiUzel;
                    if (dialog.ShowDialog() == DialogResult.OK)
                    {

                    }
                }
                doubleBufPanel1.Cursor = Cursors.Default;
                clearButton();
                doubleBufPanel1.Invalidate();
            }
        }

        private void cPMToolStripMenuItem_Click(object sender, EventArgs e)
        {    
            ZmenaZobrazeni("sitovy graf");
            postup.Vycisti();
            postup.Vloz("Začátek projektu: " + graf.ZacatekProjektu.ToLongDateString());
            Vysledky vysledky = new Vysledky();
            vysledky.NazevProjektu = graf.Nazev;
            vysledky.ZacatekProjektu = graf.ZacatekProjektu;
            double hodnotaCPM = 0; ;
            int chybaReseni = 0;
            if (nastaveni.VyukovyMod == true)
            {
                hodnotaCPM = graf.AlgoritmusTopologickehoUsporadani(true, postup);
                graf.AlgoritmusTopologickehoUsporadani2(postup);
            }
            else
            {
                hodnotaCPM = graf.AlgoritmusTopologickehoUsporadani(true);
                graf.AlgoritmusTopologickehoUsporadani2();
            }
            chybaReseni = (int)hodnotaCPM;
            switch (chybaReseni)
            {
                case -1: MessageBox.Show("V síťovém grafu je více než jeden počátečních uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -2: MessageBox.Show("V síťovém grafu je více než jeden koncových uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -3: MessageBox.Show("V síťovém grafu není počáteční uzel.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -4: MessageBox.Show("V síťovém grafu není koncový uzel", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -5: MessageBox.Show("Síťový graf není acyklický.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
            }
            if (chybaReseni < 0) return;
            postup.Vloz(" ");
            postup.Vloz("Délka projektu: " + hodnotaCPM.ToString() + " (ve dnech)");
            postup.Vloz("Konec projektu: " + (graf.ZacatekProjektu.AddDays(hodnotaCPM)).ToLongDateString());
            vysledky.DelkaProjektu = hodnotaCPM;
            vysledky.KonecProjektu = graf.ZacatekProjektu.AddDays(hodnotaCPM);
            vysledky.Typ = 1;
            postup.Vloz("Kritické činnosti:");
            String retezec = "";
            VypocetPostup jednoReseni = new VypocetPostup();
            foreach (Hrana polozka in graf.kritickaCesta)
            {
                if (nastaveni.PopisCinnosti == 1) retezec += polozka.Data.Zkratka + ",";
                else retezec += polozka.Data.Nazev + ",";
                jednoReseni.VlozNaZacatek(polozka.Data.Nazev + " - ohodnocení: " + polozka.Data.Ohodnoceni);
            }
            vysledky.Reseni = new System.Collections.ArrayList(1);
            vysledky.Reseni.Add(jednoReseni);
            if (retezec.Length >= 1) retezec = retezec.Remove(retezec.Length - 1, 1);
            postup.Vloz(retezec);
            if (nastaveni.VyukovyMod == true) graf.VypocetRezerv(postup, nastaveni);
            dialogVysledky.NazevProjektu = graf.Nazev;
            dialogVysledky.Vysledky = vysledky;
            dialogVysledky.Zobraz();
            dialogVysledky.Show();
            if (nastaveni.VyukovyMod == false) VypisAlgoritmu();
            else
            {
                dialogVypis.Postup = postup;
                dialogVypis.NazevProjektu = graf.Nazev;
                dialogVypis.Zobraz();
                dialogVypis.Show();

            }
            doubleBufPanel1.Invalidate();
        }

        private void VypisAlgoritmu()
        {
        }

        private void definiceZdrojůToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (puvodniZobrazeni!=null)
                using (DialogZdroje dialog = new DialogZdroje(puvodniZobrazeni))
                {
                    if (dialog.ShowDialog() == DialogResult.OK)
                    {
                    }
                }
            else
            using (DialogZdroje dialog = new DialogZdroje(graf))
            {
                if (dialog.ShowDialog() == DialogResult.OK)
                {

                }
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {

        }

        private void toolStripComboBox1_Click(object sender, EventArgs e)
        {

        }

        private void MenuZobrazeniInit()
        {
            sítovýGrafToolStripMenuItem.Checked = false;
            prastromŘešeníToolStripMenuItem.Checked = false;
            ganntůvDiagramToolStripMenuItem.Checked = false;
        }

        /// <summary>
        /// Změní typ grafického zobrazení dle parametru.
        /// Volby:
        /// "sitovy graf" ... zobrazí síťový graf
        /// "ganttuv diagram" ... zobrazí Ganttův diagram
        /// "prastrom reseni" ... zobrazí prastrom řešení
        /// </summary>
        /// <param name="typ">Parametr, který udává změnu zobrazení.</param>
        private void ZmenaZobrazeni(string typ)
        {
            zobrazit = typ;
            reseniToolStripComboBox.Visible = false;
            if (zobrazit == "sitovy graf" && sítovýGrafToolStripMenuItem.Checked == false)
            {
                nastaveni.ZobrazeniInformaci = 0;
                if (puvodniZobrazeni != null) graf = puvodniZobrazeni;
                puvodniZobrazeni = null;
                MenuZobrazeniInit();
                sítovýGrafToolStripMenuItem.Checked = true;
                int maxWidth = 0, maxHeight = 0;
                foreach (Vrchol uzelItem in graf)
                {
                    if (uzelItem.SouradniceX > maxWidth) maxWidth = uzelItem.SouradniceX;
                    if (uzelItem.SouradniceY > maxHeight) maxHeight = uzelItem.SouradniceY;
                }
                doubleBufPanel1.Width = maxWidth + 50;
                doubleBufPanel1.Height = maxHeight + 50;
                doubleBufPanel1.Invalidate();
                return;
            }
            if (zobrazit == "prastrom reseni" && prastromŘešeníToolStripMenuItem.Checked == false)
            {
                nastaveni.ZobrazeniInformaci = 0;
                MenuZobrazeniInit();
                if (puvodniZobrazeni == null) puvodniZobrazeni = graf;
                graf = prastromReseni;
                prastromŘešeníToolStripMenuItem.Checked = true;
                int maxWidth = 0, maxHeight = 0;
                foreach (Vrchol uzelItem in prastromReseni)
                {
                    if (uzelItem.SouradniceX > maxWidth) maxWidth = uzelItem.SouradniceX;
                    if (uzelItem.SouradniceY > maxHeight) maxHeight = uzelItem.SouradniceY;
                }
                doubleBufPanel1.Width = maxWidth + 50;
                doubleBufPanel1.Height = maxHeight + 50;
                doubleBufPanel1.Invalidate();
                return;
            }
            if (zobrazit == "ganttuv diagram" && ganntůvDiagramToolStripMenuItem.Checked == false)
            {
                if (prastromReseni != null) reseniToolStripComboBox.Visible = true;
                optimalniReseni.Clear();
                double minimum = double.MaxValue;
                foreach (Vrchol polozka in prastromReseni.vrcholy)
                {
                    if (polozka.MaximalniCas == polozka.MinimalniCas && polozka.MaximalniCas < minimum) minimum = polozka.MaximalniCas;
                }
                foreach (Vrchol polozka in prastromReseni.vrcholy)
                {
                    if (polozka.MaximalniCas == polozka.MinimalniCas && polozka.MaximalniCas == minimum) optimalniReseni.Add(polozka);
                }
                reseniToolStripComboBox.Items.Clear();
                for (int p = 0; p < optimalniReseni.Count; p++)
                {
                    reseniToolStripComboBox.Items.Add("řešení " + (p + 1).ToString());
                }

                if (VytvorGanttuvDiagram(0) == 0)
                {
                    if (puvodniZobrazeni == null) puvodniZobrazeni = graf;
                    graf = ganttuvDiagram;
                    MenuZobrazeniInit();
                    ganntůvDiagramToolStripMenuItem.Checked = true;
                    doubleBufPanel1.Invalidate();
                    return;
                }
                ZmenaZobrazeni("sitovy graf");
                return;
            }
        }
        /// <summary>
        /// Změní typ grafického zobrazení dle parametru.
        /// Volby:
        /// "ganttuv diagram" ... zobrazí Ganttův diagram
        /// </summary>
        /// <param name="typ">Parametr, který udává změnu zobrazení. Druhý parametr udává pořadové číslo
        /// řešení prastromu.</param>
        private void ZmenaZobrazeni(string typ, int indexZobrazeni)
        {
            zobrazit = typ;
            //reseniToolStripComboBox.Visible = false;
            if (zobrazit == "ganttuv diagram")
            {
                graf = puvodniZobrazeni;
                if (VytvorGanttuvDiagram(indexZobrazeni) == 0)
                {
                    graf = ganttuvDiagram;
                    MenuZobrazeniInit();
                    ganntůvDiagramToolStripMenuItem.Checked = true;
                    doubleBufPanel1.Invalidate();
                    return;
                }
                ZmenaZobrazeni("sitovy graf");
                return;
            }
        }
        /// <summary>
        /// Změní velikost plátna.
        /// </summary>
        private void ResizePlatno()
        {
            if (zobrazit == "sitovy graf")
            {
                int maxX = 0;
                int maxY = 0;
                foreach (Vrchol polozka in graf.vrcholy)
                {
                    if (polozka.SouradniceX > maxX) maxX = polozka.SouradniceX;
                    if (polozka.SouradniceY > maxY) maxY = polozka.SouradniceY;
                }
                doubleBufPanel1.Width = maxX + nastaveni.Velikost * nastaveni.Meritko * 20;
                doubleBufPanel1.Height = maxY + nastaveni.Velikost * nastaveni.Meritko * 20;
                doubleBufPanel1.Invalidate();
                return;
            }
            if (zobrazit == "prastrom reseni")
            {
                int maxX = 0;
                int maxY = 0;
                foreach (Vrchol polozka in prastromReseni.vrcholy)
                {
                    if (polozka.SouradniceX > maxX) maxX = polozka.SouradniceX;
                    if (polozka.SouradniceY > maxY) maxY = polozka.SouradniceY;
                }
                doubleBufPanel1.Width = maxX + nastaveni.Velikost * nastaveni.Meritko * 20;
                doubleBufPanel1.Height = maxY + nastaveni.Velikost * nastaveni.Meritko * 20;
                doubleBufPanel1.Invalidate();
                return;
            }
            if (zobrazit == "ganttuv diagram")
            {
                int maxX = 0;
                int maxY = 0;
                foreach (Vrchol polozka in ganttuvDiagram.vrcholy)
                {
                    if (polozka.SouradniceX > maxX) maxX = polozka.SouradniceX;
                    if (polozka.SouradniceY > maxY) maxY = polozka.SouradniceY;
                }
                doubleBufPanel1.Width = maxX + nastaveni.Velikost * nastaveni.Meritko * 20;
                doubleBufPanel1.Height = maxY + nastaveni.Velikost * nastaveni.Meritko * 20;
                doubleBufPanel1.Invalidate();
                return;
            }
        }
        /// <summary>
        /// Vytvoří Ganttův diagram podle zobrazení. Pokud je vypočítán prastrom řešení, pak zobrazí
        /// Ganttův diagram příslušného řešení metody.
        /// </summary>
        /// <param name="indexReseni">Parametrem metody je vybrané pořadové číslo řešení.</param>
        /// <returns>Vrací chybové hlášení typu <c>int</c>.</returns>
        private int VytvorGanttuvDiagram(int indexReseni)
        {
            if (nastaveni.Velikost == 0) nastaveni.Velikost = 1;
            nastaveni.ZobrazeniInformaci = 1;
            //ganttovyDiagramy = null;
            ganttuvDiagram.kritickaCesta.Clear();
            ganttuvDiagram.Init();
            double minimalniOhodnoceni = double.MaxValue;
            foreach (Hrana polozka in graf.naslednici)
            {
                if (polozka.Data.Ohodnoceni < minimalniOhodnoceni && polozka.Data.Ohodnoceni > 0) minimalniOhodnoceni = polozka.Data.Ohodnoceni;
            }

            Graf zaloha = prastromReseni;
            if (prastromReseni.vrcholy.Count > 0)
            {
                if (prastromReseni.naslednici.Count == 0) return 1;
                reseniToolStripComboBox.Visible = true;
                //graf = prastromReseni;
                int pocetReseni = -1;
                while (pocetReseni < 0)
                {
                    pocetReseni++;
                    Vrchol item = (Vrchol)(optimalniReseni[indexReseni]);
                    //ganttuvDiagram = ganttovyDiagramy[pocetReseni];
                    SeznamHran reseni = new SeznamHran();
                    {
                        Vrchol pomItem = new Vrchol();
                        item.Kopie(pomItem);
                        while (pomItem.PocetPredchudcu > 0)
                        {
                            Hrana novaHrana = (Hrana)(prastromReseni.predchudci[pomItem.IndexPrvnihoPredchudce]);
                            Hrana vkladanaHrana = new Hrana(novaHrana.Vrchol1, novaHrana.Vrchol2);
                            novaHrana.Kopie(vkladanaHrana);
                            reseni.Insert(0, vkladanaHrana);
                            pomItem = new Vrchol(novaHrana.Vrchol1.GetKlic());
                            pomItem = (Vrchol)(prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(pomItem, 0)]);
                        }
                    }
                    int novaSouradniceX = 100;
                    int novaSouradniceY = 70;
                    int posledniKlicPR = 0;
                    string klicPR = "";
                    int pomVlozen = 0;
                    Hrana hranaPrastromu = (Hrana)(reseni[0]);
                    for (int i = 0; i < hranaPrastromu.Data.PoleHran.Length; i++)
                    {
                        Hrana skutecnaHrana = new Hrana(hranaPrastromu.Data.PoleHran[i].Vrchol1, hranaPrastromu.Data.PoleHran[i].Vrchol2);
                        skutecnaHrana = (Hrana)(graf.naslednici[graf.naslednici.Najdi(skutecnaHrana, 0)]);
                        Hrana ganttuvDiagramHrana = new Hrana();
                        skutecnaHrana.Kopie(ganttuvDiagramHrana);
                        Vrchol vrcholReal1 = new Vrchol(skutecnaHrana.Vrchol1.GetKlic());
                        Vrchol vrcholReal2 = new Vrchol(skutecnaHrana.Vrchol2.GetKlic());
                        vrcholReal1 = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(vrcholReal1, 0)]);
                        vrcholReal2 = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(vrcholReal2, 0)]);
                        Vrchol predVrcholGD1 = new Vrchol();
                        Vrchol predVrcholGD2 = new Vrchol();
                        Vrchol vrcholPrastromu1 = new Vrchol(hranaPrastromu.Vrchol1.GetKlic());
                        Vrchol vrcholPrastromu2 = new Vrchol(hranaPrastromu.Vrchol2.GetKlic());
                        vrcholPrastromu1 = (Vrchol)(prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(vrcholPrastromu1, 0)]);
                        vrcholPrastromu2 = (Vrchol)(prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(vrcholPrastromu2, 0)]);


                        do
                        {
                            klicPR = "V" + posledniKlicPR.ToString("D3");
                            predVrcholGD1.Klic.SetKlic(klicPR);
                            predVrcholGD1.Nazev = vrcholReal1.Nazev;
                            predVrcholGD1.MinimalniCas = vrcholPrastromu1.MinimalniCas;
                            predVrcholGD1.MaximalniCas = vrcholPrastromu2.MinimalniCas;
                            predVrcholGD1.SouradniceX = novaSouradniceX;
                            predVrcholGD1.SouradniceY = novaSouradniceY;
                            pomVlozen = ganttuvDiagram.VlozVrchol(predVrcholGD1);
                            posledniKlicPR++;
                        } while (pomVlozen >= 1);
                        do
                        {
                            klicPR = "V" + posledniKlicPR.ToString("D3");
                            predVrcholGD2.Klic.SetKlic(klicPR);
                            predVrcholGD2.Nazev = vrcholReal2.Nazev;
                            predVrcholGD2.MinimalniCas = vrcholPrastromu2.MinimalniCas;
                            predVrcholGD2.MaximalniCas = vrcholPrastromu2.MinimalniCas;
                            predVrcholGD2.SouradniceX = predVrcholGD1.SouradniceX + (int)(nastaveni.Meritko * 100 * (hranaPrastromu.Data.Vykonano / minimalniOhodnoceni));
                            predVrcholGD2.SouradniceY = novaSouradniceY;
                            pomVlozen = ganttuvDiagram.VlozVrchol(predVrcholGD2);
                            posledniKlicPR++;
                        } while (pomVlozen >= 1);
                        ganttuvDiagramHrana.Vrchol1.SetKlic(predVrcholGD1.Klic.GetKlic());
                        ganttuvDiagramHrana.Vrchol2.SetKlic(predVrcholGD2.Klic.GetKlic());
                        ganttuvDiagramHrana.Data.poleHran = new ADTHrana[1];
                        ADTHrana puvodniHrana = new ADTHrana(vrcholReal1.Klic.GetKlic(), vrcholReal2.Klic.GetKlic());
                        ganttuvDiagramHrana.Data.poleHran[0] = puvodniHrana;
                        if (ganttuvDiagram.VlozHranu(ganttuvDiagramHrana) != 0) MessageBox.Show("Gantt - nevlozena hrana 01");
                        novaSouradniceY += (int)(nastaveni.Meritko * nastaveni.Velikost + nastaveni.Tloustka * 10 + 10);
                    }
                    reseni.RemoveAt(0);
                    while (reseni.Count > 0)
                    {
                        hranaPrastromu = (Hrana)(reseni[0]);
                        for (int i = 0; i < hranaPrastromu.Data.PoleHran.Length; i++)
                        {
                            Hrana skutecnaHrana = new Hrana(hranaPrastromu.Data.PoleHran[i].Vrchol1, hranaPrastromu.Data.PoleHran[i].Vrchol2);
                            skutecnaHrana = (Hrana)(graf.naslednici[graf.naslednici.Najdi(skutecnaHrana, 0)]);
                            Hrana ganttuvDiagramHrana = new Hrana();
                            skutecnaHrana.Kopie(ganttuvDiagramHrana);
                            Vrchol vrcholReal1 = new Vrchol(skutecnaHrana.Vrchol1.GetKlic());
                            Vrchol vrcholReal2 = new Vrchol(skutecnaHrana.Vrchol2.GetKlic());
                            vrcholReal1 = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(vrcholReal1, 0)]);
                            vrcholReal2 = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(vrcholReal2, 0)]);
                            Vrchol predVrcholGD1 = new Vrchol();
                            Vrchol predVrcholGD2 = new Vrchol();
                            Vrchol vrcholPrastromu1 = new Vrchol(hranaPrastromu.Vrchol1.GetKlic());
                            Vrchol vrcholPrastromu2 = new Vrchol(hranaPrastromu.Vrchol2.GetKlic());
                            vrcholPrastromu1 = (Vrchol)(prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(vrcholPrastromu1, 0)]);
                            vrcholPrastromu2 = (Vrchol)(prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(vrcholPrastromu2, 0)]);
                            int zalohaY;
                            zalohaY = novaSouradniceY;
                            foreach (Hrana hranaPolozka in ganttuvDiagram.naslednici)
                            {
                                Hrana stejnaHrana = new Hrana(hranaPolozka.Data.poleHran[0].Vrchol1, hranaPolozka.Data.poleHran[0].Vrchol2);
                                //hranaPolozka.Kopie(stejnaHrana);
                                if (stejnaHrana.Vrchol1.GetKlic() == skutecnaHrana.Vrchol1.GetKlic() && stejnaHrana.Vrchol2.GetKlic() == skutecnaHrana.Vrchol2.GetKlic())
                                {
                                    stejnaHrana.Vrchol1.SetKlic(hranaPolozka.Vrchol1.GetKlic());
                                    stejnaHrana.Vrchol2.SetKlic(hranaPolozka.Vrchol2.GetKlic());
                                    stejnaHrana = (Hrana)(ganttuvDiagram.naslednici[ganttuvDiagram.naslednici.Najdi(stejnaHrana, 0)]);
                                    Vrchol predchoziVrchol = new Vrchol(stejnaHrana.Vrchol1.GetKlic());
                                    predchoziVrchol = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(predchoziVrchol, 0)]);
                                    Vrchol predchoziVrcholT = new Vrchol(stejnaHrana.Vrchol2.GetKlic());
                                    predchoziVrcholT = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(predchoziVrcholT, 0)]);
                                    novaSouradniceY = predchoziVrchol.SouradniceY;
                                }
                            }
                            do
                            {
                                klicPR = "V" + posledniKlicPR.ToString("D3");
                                predVrcholGD1.Klic.SetKlic(klicPR);
                                predVrcholGD1.Nazev = vrcholReal1.Nazev;
                                predVrcholGD1.MinimalniCas = vrcholPrastromu1.MinimalniCas;
                                predVrcholGD1.MaximalniCas = vrcholPrastromu2.MinimalniCas;
                                predVrcholGD1.SouradniceX = novaSouradniceX + (int)(nastaveni.Meritko * 100 * (vrcholPrastromu1.MinimalniCas / minimalniOhodnoceni));
                                predVrcholGD1.SouradniceY = novaSouradniceY;
                                pomVlozen = ganttuvDiagram.VlozVrchol(predVrcholGD1);
                                posledniKlicPR++;
                            } while (pomVlozen >= 1);
                            do
                            {
                                klicPR = "V" + posledniKlicPR.ToString("D3");
                                predVrcholGD2.Klic.SetKlic(klicPR);
                                predVrcholGD2.Nazev = vrcholReal2.Nazev;
                                predVrcholGD2.MinimalniCas = vrcholPrastromu2.MinimalniCas;
                                predVrcholGD2.MaximalniCas = vrcholPrastromu2.MinimalniCas;
                                predVrcholGD2.SouradniceX = predVrcholGD1.SouradniceX + (int)(nastaveni.Meritko * 100 * (hranaPrastromu.Data.Vykonano / minimalniOhodnoceni));
                                predVrcholGD2.SouradniceY = novaSouradniceY;
                                pomVlozen = ganttuvDiagram.VlozVrchol(predVrcholGD2);
                                posledniKlicPR++;
                            } while (pomVlozen >= 1);
                            ganttuvDiagramHrana.Vrchol1.SetKlic(predVrcholGD1.Klic.GetKlic());
                            ganttuvDiagramHrana.Vrchol2.SetKlic(predVrcholGD2.Klic.GetKlic());
                            //ganttuvDiagramHrana.Data.poleHranZaloha = ganttuvDiagramHrana.Data.PoleHran;
                            ganttuvDiagramHrana.Data.poleHran = new ADTHrana[1];
                            ADTHrana puvodniHrana = new ADTHrana(vrcholReal1.Klic.GetKlic(), vrcholReal2.Klic.GetKlic());
                            ganttuvDiagramHrana.Data.poleHran[0] = puvodniHrana;
                            ganttuvDiagramHrana.Data.Vykonano = hranaPrastromu.Data.Vykonano;
                            if (ganttuvDiagram.VlozHranu(ganttuvDiagramHrana) != 0) MessageBox.Show("Gantt - nevlozena hrana 05");

                            novaSouradniceY = zalohaY;
                            novaSouradniceY += (int)(nastaveni.Meritko * nastaveni.Velikost + nastaveni.Tloustka * 10 + 10);
                        }
                        reseni.RemoveAt(0);
                    }
                    PoleVrcholu mazaneVrcholy = new PoleVrcholu();
                    foreach (Hrana outHrana in ganttuvDiagram.naslednici)
                    {
                        foreach (Hrana inHrana in ganttuvDiagram.naslednici)
                        {
                            Vrchol outVrchol1 = new Vrchol(outHrana.Vrchol1.GetKlic());
                            outVrchol1 = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(outVrchol1, 0)]);
                            Vrchol outVrchol2 = new Vrchol(outHrana.Vrchol2.GetKlic());
                            outVrchol2 = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(outVrchol2, 0)]);
                            Vrchol inVrchol1 = new Vrchol(inHrana.Vrchol1.GetKlic());
                            inVrchol1 = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(inVrchol1, 0)]);
                            Vrchol inVrchol2 = new Vrchol(inHrana.Vrchol2.GetKlic());
                            inVrchol2 = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(inVrchol2, 0)]);
                            if (outVrchol2.SouradniceX == inVrchol1.SouradniceX && outVrchol2.SouradniceY == inVrchol1.SouradniceY && inHrana.Data.Typ != 2)
                            {
                                outHrana.Data.Vykonano = outHrana.Data.Vykonano + inHrana.Data.Vykonano;
                                string byvalyKlic = outVrchol2.Klic.GetKlic();
                                int byvalyPredchudce = outVrchol2.IndexPrvnihoPredchudce;
                                inVrchol2.Kopie(outVrchol2);
                                outVrchol2.IndexPrvnihoPredchudce = byvalyPredchudce;
                                outVrchol2.Klic.SetKlic(byvalyKlic);
                                mazaneVrcholy.Add(inVrchol1);
                                mazaneVrcholy.Add(inVrchol2);
                            }
                        }
                    }
                    foreach (Vrchol mazanyVrchol in mazaneVrcholy)
                    {
                        ganttuvDiagram.OdeberVrchol(mazanyVrchol);
                    }
                    if (nastaveni.ZobrazitZavislosti == true)
                    {
                        SeznamHran mazaneHrany = new SeznamHran();
                        foreach (Hrana outHrana in ganttuvDiagram.naslednici)
                        {
                            foreach (Hrana inHrana in ganttuvDiagram.naslednici)
                            {
                                Vrchol outVrchol1 = new Vrchol(outHrana.Vrchol1.GetKlic());
                                outVrchol1 = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(outVrchol1, 0)]);
                                Vrchol outVrchol2 = new Vrchol(outHrana.Vrchol2.GetKlic());
                                outVrchol2 = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(outVrchol2, 0)]);
                                Vrchol inVrchol1 = new Vrchol(inHrana.Vrchol1.GetKlic());
                                inVrchol1 = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(inVrchol1, 0)]);
                                Vrchol inVrchol2 = new Vrchol(inHrana.Vrchol2.GetKlic());
                                inVrchol2 = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(inVrchol2, 0)]);
                                if (outVrchol2.SouradniceX >= inVrchol1.SouradniceX - 1 && outVrchol2.SouradniceX <= inVrchol1.SouradniceX + 1 && outVrchol2.SouradniceY != inVrchol1.SouradniceY)
                                {
                                    Hrana hranaZavislosti = new Hrana(outVrchol2.Klic, inVrchol1.Klic);
                                    hranaZavislosti.Data.Nazev = null;
                                    if (hranaZavislosti.Data.Typ == 2) //fiktivni hrana
                                        hranaZavislosti.Data.Typ = 4; //fiktivni hrana zavislosti
                                    else
                                    {
                                        hranaZavislosti.Data.Typ = 3; //hrana zavislosti
                                    }
                                    mazaneHrany.Add(hranaZavislosti);
                                }
                            }
                        }
                        foreach (Hrana mazanaHrana in mazaneHrany)
                        {
                            if (ganttuvDiagram.VlozHranu(mazanaHrana) != 0) MessageBox.Show("Gantt - nevlozena hrana 02"); ;
                        }
                    }
                    prastromReseni = zaloha;
                }
                foreach (Hrana polozka in ganttuvDiagram.naslednici)
                {
                    polozka.Data.Ohodnoceni = polozka.Data.Vykonano;
                }
                return 0;
            }
            double hodnotaCPM = graf.AlgoritmusTopologickehoUsporadani(true);
            graf.AlgoritmusTopologickehoUsporadani2();
            graf.VypocetRezerv();
            int chybaReseni = (int)hodnotaCPM;
            switch (chybaReseni)
            {
                case -1: MessageBox.Show("V síťovém grafu je více než jeden počátečních uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -2: MessageBox.Show("V síťovém grafu je více než jeden koncových uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -3: MessageBox.Show("V síťovém grafu není počáteční uzel.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -4: MessageBox.Show("V síťovém grafu není koncový uzel", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -5: MessageBox.Show("Síťový graf není acyklický.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
            }
            if (chybaReseni < 0)
            {
                return -1;
            }
            graf.VypocetRezerv();

            SeznamHran cinnosti = new SeznamHran();
            SeznamHran zpracovaneCinnosti = new SeznamHran();
            SeznamHran fiktivniCinnosti = new SeznamHran();
            cinnosti.Clear();
            foreach (Hrana polozka in graf.naslednici)
            {
                Vrchol pomVrchol = new Vrchol(polozka.Vrchol1.GetKlic()); //pocatecni vrchol hrany
                pomVrchol = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(pomVrchol, 0)]);
                if (pomVrchol.PocetPredchudcu > 0)
                {
                    polozka.Data.PoleHran = new ADTHrana[pomVrchol.PocetPredchudcu];
                    for (int i = pomVrchol.IndexPrvnihoPredchudce; i < pomVrchol.PocetPredchudcu + pomVrchol.IndexPrvnihoPredchudce; i++)
                    {
                        Hrana pomHrana = ((Hrana)(graf.predchudci[i]));
                        ADTHrana predHrana = new ADTHrana(pomHrana.Vrchol1.GetKlic(), pomHrana.Vrchol2.GetKlic());
                        polozka.Data.PoleHran[i - pomVrchol.IndexPrvnihoPredchudce] = predHrana;
                    }
                }
                cinnosti.Add(polozka);
            }
            Vrchol predVrchol = null;
            Vrchol predVrchol2 = null;
            int posledniKlic = 0;
            int aktSouradniceY = 70;
            string klic;
            int pom;
            foreach (Hrana polozka in cinnosti)
            {
                predVrchol = new Vrchol(polozka.Vrchol1.GetKlic());
                predVrchol = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(predVrchol, 0)]);
                predVrchol2 = new Vrchol(polozka.Vrchol2.GetKlic());
                predVrchol2 = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(predVrchol2, 0)]);

                if (predVrchol.PocetPredchudcu == 0)
                {
                    Hrana vkladanaHrana = new Hrana();
                    Hrana vkladanaHrana2 = new Hrana();
                    Vrchol vrchol1 = new Vrchol();
                    Vrchol vrchol2 = new Vrchol();
                    Vrchol vrchol3 = new Vrchol();
                    if (polozka.Data.Typ == 0)
                        do
                        {
                            klic = "V" + posledniKlic.ToString("D3");
                            vrchol1.Klic.SetKlic(klic);
                            vrchol1.Nazev = predVrchol.Nazev;
                            vrchol1.MaximalniCas = predVrchol.MaximalniCas;
                            vrchol1.MinimalniCas = predVrchol.MinimalniCas;
                            pom = ganttuvDiagram.VlozVrchol(vrchol1);
                            posledniKlic++;
                        } while (pom >= 1);
                    if (polozka.Data.Typ == 0)
                    {
                        do
                        {
                            klic = "V" + posledniKlic.ToString("D3");
                            vrchol2.Klic.SetKlic(klic);
                            vrchol2.Nazev = predVrchol2.Nazev;
                            vrchol2.MinimalniCas = predVrchol.MinimalniCas + polozka.Data.Ohodnoceni;
                            vrchol2.MaximalniCas = predVrchol2.MaximalniCas;
                            pom = ganttuvDiagram.VlozVrchol(vrchol2);
                            posledniKlic++;
                        } while (pom >= 1);
                        /*
                        Hrana predVrchHrana = new Hrana();
                        predVrchHrana.Vrchol1.SetKlic(polozka.Vrchol2.GetKlic());
                        predVrchHrana.Vrchol2.SetKlic(vrchol2.Klic.GetKlic());
                        predVrcholy.Add(predVrchHrana);*/
                        polozka.Data.PredKlic = new KlicVrchol(vrchol2.Klic.GetKlic());
                    }
                    else
                    {
                        /*Hrana predVrchHrana = new Hrana();
                        predVrchHrana.Vrchol1.SetKlic(polozka.Vrchol2.GetKlic());
                        predVrchHrana.Vrchol2.SetKlic(vrchol1.Klic.GetKlic());
                        predVrcholy.Add(predVrchHrana);*/
                        polozka.Data.PredKlic = new KlicVrchol(vrchol1.Klic.GetKlic());
                    }
                    vrchol1.SouradniceX = 100;
                    vrchol1.SouradniceY = aktSouradniceY;
                    vrchol2.SouradniceX = vrchol1.SouradniceX + (int)(nastaveni.Meritko * 100 * (polozka.Data.Ohodnoceni / minimalniOhodnoceni));
                    vrchol2.SouradniceY = aktSouradniceY;
                    polozka.Kopie(vkladanaHrana);
                    vkladanaHrana.Vrchol1.SetKlic(vrchol1.Klic.GetKlic());
                    vkladanaHrana.Vrchol2.SetKlic(vrchol2.Klic.GetKlic());
                    aktSouradniceY += (int)(nastaveni.Meritko * nastaveni.Velikost * 3 + nastaveni.Tloustka * 5 + 10);
                    if (polozka.Data.Typ != 2) ganttuvDiagram.VlozHranu(vkladanaHrana);
                    if (nastaveni.ZobrazitRezervy == true && polozka.Data.RC > 0)
                    {
                        do
                        {
                            klic = "V" + posledniKlic.ToString("D3");
                            vrchol3.Klic.SetKlic(klic);
                            vrchol3.Nazev = predVrchol2.Nazev;
                            vrchol3.MaximalniCas = predVrchol2.MaximalniCas;
                            vrchol3.MinimalniCas = predVrchol2.MaximalniCas;
                            vrchol3.SouradniceX = vrchol1.SouradniceX + (int)(nastaveni.Meritko * 100 * ((polozka.Data.RC + polozka.Data.Ohodnoceni) / minimalniOhodnoceni));
                            vrchol3.SouradniceY = vrchol1.SouradniceY;
                            pom = ganttuvDiagram.VlozVrchol(vrchol3);
                            posledniKlic++;
                        } while (pom >= 1);
                        polozka.Kopie(vkladanaHrana2);
                        vkladanaHrana2.Vrchol1.SetKlic(vrchol2.Klic.GetKlic());
                        vkladanaHrana2.Vrchol2.SetKlic(vrchol3.Klic.GetKlic());
                        vkladanaHrana2.Data.Typ = 1;
                        vkladanaHrana2.Data.Nazev = "Celková rezerva";
                        vkladanaHrana2.Data.Ohodnoceni = vkladanaHrana2.Data.RC;
                        if (ganttuvDiagram.VlozHranu(vkladanaHrana2) != 0) MessageBox.Show("Gantt - nevlozena hrana 06"); ;
                    }
                    /*Vrchol pomVrchol1 = new Vrchol(polozka.Vrchol1.GetKlic());
                    Vrchol pomVrchol2 = new Vrchol(polozka.Vrchol2.GetKlic());
                    pomVrchol1.SouradniceX = vrchol1.SouradniceX;
                    omVrchol1.SouradniceY = vrchol1.SouradniceY;
                    pomVrchol2.SouradniceX = vrchol2.SouradniceX;
                    pomVrchol2.SouradniceY = vrchol2.SouradniceY;*/
                    polozka.Data.DocasneOhodnoceni = vrchol2.SouradniceX;
                    zpracovaneCinnosti.Add(polozka);

                }
            }
            foreach (Hrana polozka in zpracovaneCinnosti)
            {
                cinnosti.OdeberHranu(polozka);
            }
            SeznamHran adeptiNaVlozeni = new SeznamHran();
            bool maPredchudce = true;
            while (cinnosti.Count > 0)
            {
                adeptiNaVlozeni.Clear();
                foreach (Hrana polozka in cinnosti)
                {
                    maPredchudce = true;
                    for (int i = 0; i < polozka.Data.PoleHran.Length; i++)
                    {
                        Hrana hrana = new Hrana(polozka.Data.PoleHran[i].Vrchol1, polozka.Data.PoleHran[i].Vrchol2);
                        //pokud hrana nema zpracovane vsechny predchudce
                        if (zpracovaneCinnosti.Najdi(hrana, 0) < 0)
                        {
                            maPredchudce = false;
                            break;
                        }
                    }
                    //maximalni pocet predchudcu
                    if (maPredchudce == true)
                    {
                        adeptiNaVlozeni.Add(polozka);
                    }
                }
                foreach (Hrana polozka in adeptiNaVlozeni)
                {
                    predVrchol = new Vrchol(polozka.Vrchol1.GetKlic());
                    predVrchol = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(predVrchol, 0)]);
                    predVrchol2 = new Vrchol(polozka.Vrchol2.GetKlic());
                    predVrchol2 = (Vrchol)(graf.vrcholy[graf.vrcholy.Najdi(predVrchol2, 0)]);
                    Hrana vkladanaHrana = new Hrana();
                    Vrchol vrchol1 = new Vrchol();
                    Vrchol vrchol2 = new Vrchol();
                    do
                    {
                        klic = "V" + posledniKlic.ToString("D3");
                        vrchol1.Klic.SetKlic(klic);
                        if (polozka.Data.Typ == 2) vrchol1.Nazev = "Fiktivní činnost";
                        else vrchol1.Nazev = predVrchol.Nazev;
                        vrchol1.MaximalniCas = predVrchol.MaximalniCas;
                        vrchol1.MinimalniCas = predVrchol.MinimalniCas;
                        pom = ganttuvDiagram.VlozVrchol(vrchol1);
                        posledniKlic++;
                    } while (pom >= 1);
                    if (polozka.Data.Typ == 0)
                    {
                        do
                        {
                            klic = "V" + posledniKlic.ToString("D3");
                            vrchol2.Klic.SetKlic(klic);
                            vrchol2.Nazev = predVrchol2.Nazev;
                            vrchol2.MaximalniCas = predVrchol.MinimalniCas + polozka.Data.Ohodnoceni;
                            vrchol2.MinimalniCas = predVrchol2.MinimalniCas;
                            pom = ganttuvDiagram.VlozVrchol(vrchol2);
                            posledniKlic++;
                        } while (pom >= 1);
                        polozka.Data.PredKlic = new KlicVrchol(vrchol2.Klic.GetKlic());
                    }
                    else
                    {
                        polozka.Data.PredKlic = new KlicVrchol(vrchol1.Klic.GetKlic());
                    }
                    int maximumX = 0;
                    for (int i = 0; i < polozka.Data.PoleHran.Length; i++)
                    {
                        Hrana hrana = new Hrana(polozka.Data.PoleHran[i].Vrchol1, polozka.Data.PoleHran[i].Vrchol2);
                        hrana = (Hrana)(zpracovaneCinnosti[zpracovaneCinnosti.Najdi(hrana, 0)]);
                        if (hrana.Data.DocasneOhodnoceni > maximumX) maximumX = (int)(hrana.Data.DocasneOhodnoceni);
                    }
                    vrchol1.SouradniceX = maximumX;
                    vrchol1.SouradniceY = aktSouradniceY;
                    vrchol2.SouradniceX = vrchol1.SouradniceX + (int)(nastaveni.Meritko * 100 * (polozka.Data.Ohodnoceni / minimalniOhodnoceni));
                    vrchol2.SouradniceY = aktSouradniceY;
                    polozka.Kopie(vkladanaHrana);
                    vkladanaHrana.Vrchol1.SetKlic(vrchol1.Klic.GetKlic());
                    vkladanaHrana.Vrchol2.SetKlic(vrchol2.Klic.GetKlic());

                    aktSouradniceY += (int)(nastaveni.Meritko * nastaveni.Velikost * 3 + nastaveni.Tloustka * 5 + 10);
                    if (polozka.Data.Typ != 2) if (ganttuvDiagram.VlozHranu(vkladanaHrana) != 0) MessageBox.Show("Gantt - nevlozena hrana 07");
                    Vrchol pomVrchol1 = new Vrchol(polozka.Vrchol1.GetKlic());
                    Vrchol pomVrchol2 = new Vrchol(polozka.Vrchol2.GetKlic());
                    polozka.Data.DocasneOhodnoceni = vrchol2.SouradniceX;
                    zpracovaneCinnosti.Add(polozka);

                    //(((((((((((((((((((((((((((((((((((((((((((((
                    for (int i = 0; i < polozka.Data.PoleHran.Length; i++)
                    {

                        Hrana hranaZavislosti = new Hrana();
                        hranaZavislosti.Vrchol1.SetKlic(polozka.Data.PoleHran[i].Vrchol1.GetKlic());
                        hranaZavislosti.Vrchol2.SetKlic(polozka.Data.PoleHran[i].Vrchol2.GetKlic());

                        hranaZavislosti = (Hrana)(zpracovaneCinnosti[zpracovaneCinnosti.Najdi(hranaZavislosti, 0)]);
                        Hrana vkladanaHranaZavislosti = new Hrana();
                        hranaZavislosti.Kopie(vkladanaHranaZavislosti);
                        Vrchol predZavislostVrchol = new Vrchol(hranaZavislosti.Data.PredKlic.GetKlic());
                        predZavislostVrchol = (Vrchol)(ganttuvDiagram.vrcholy[ganttuvDiagram.vrcholy.Najdi(predZavislostVrchol, 0)]);
                        if (hranaZavislosti.Data.Typ == 2) //fiktivni hrana
                            vkladanaHranaZavislosti.Data.Typ = 4; //fiktivni hrana zavislosti
                        else vkladanaHranaZavislosti.Data.Typ = 3; //hrana zavislosti 
                        Vrchol vrcholZavislosti2 = new Vrchol(hranaZavislosti.Vrchol2.GetKlic());
                        vkladanaHranaZavislosti.Vrchol1.SetKlic(predZavislostVrchol.Klic.GetKlic());
                        vkladanaHranaZavislosti.Vrchol2.SetKlic(vkladanaHrana.Vrchol1.GetKlic());
                        vkladanaHranaZavislosti.Data.Nazev = null;
                        if (nastaveni.ZobrazitZavislosti == true) if (ganttuvDiagram.VlozHranu(vkladanaHranaZavislosti) != 0) MessageBox.Show("Chyba Gantt 08");
                    }
                    //dfgdfsgddddddddddddddddddddddddddddddddddd

                    Vrchol vrchol3 = new Vrchol();
                    Hrana vkladanaHrana2 = new Hrana();
                    if (polozka.Data.Typ == 0 && nastaveni.ZobrazitRezervy == true && polozka.Data.RC > 0)
                    {
                        do
                        {
                            klic = "V" + posledniKlic.ToString("D3");
                            vrchol3.Klic.SetKlic(klic);
                            vrchol3.Nazev = predVrchol2.Nazev;
                            vrchol3.MaximalniCas = predVrchol2.MaximalniCas;
                            vrchol3.MinimalniCas = predVrchol2.MaximalniCas;
                            vrchol3.SouradniceX = vrchol1.SouradniceX + (int)(nastaveni.Meritko * 100 * ((polozka.Data.RC + polozka.Data.Ohodnoceni) / minimalniOhodnoceni));
                            vrchol3.SouradniceY = vrchol1.SouradniceY;
                            pom = ganttuvDiagram.VlozVrchol(vrchol3);
                            posledniKlic++;
                        } while (pom >= 1);
                        polozka.Kopie(vkladanaHrana2);
                        //vkladanaHrana.Vrchol1.SetKlic(vrchol1.Klic.GetKlic());
                        //vkladanaHrana.Vrchol2.SetKlic(vrchol2.Klic.GetKlic());
                        if (ganttuvDiagram.vrcholy.Najdi(vrchol2, 0) < 0) vkladanaHrana2.Vrchol1.SetKlic(vrchol1.Klic.GetKlic());
                        else vkladanaHrana2.Vrchol1.SetKlic(vrchol2.Klic.GetKlic());
                        vkladanaHrana2.Vrchol2.SetKlic(vrchol3.Klic.GetKlic());
                        vkladanaHrana2.Data.Nazev = "Celková rezerva";
                        vkladanaHrana2.Data.Ohodnoceni = vkladanaHrana2.Data.RC;
                        vkladanaHrana2.Data.Typ = 1;
                        if (ganttuvDiagram.VlozHranu(vkladanaHrana2) != 0) MessageBox.Show("Gantt - nevlozena hrana 03"); ;
                    }
                    cinnosti.OdeberHranu(polozka);
                }
            }
            int maxX = 0;
            int maxY = 0;
            foreach (Vrchol polozka in ganttuvDiagram)
            {
                if (polozka.SouradniceX > maxX) maxX = polozka.SouradniceX;
                if (polozka.SouradniceY > maxY) maxY = polozka.SouradniceY;
            }
            doubleBufPanel1.Width = maxX + nastaveni.Velikost * nastaveni.Meritko * 20;
            doubleBufPanel1.Height = maxY + nastaveni.Velikost * nastaveni.Meritko * 20;
            foreach (Hrana polozka in ganttuvDiagram.naslednici)
            {
                if (polozka.Data.RC == 0)
                {
                    ganttuvDiagram.kritickaCesta.Add(polozka);
                }
            }
            return 0;
        }


        private void toolStripComboBox1_TextChanged(object sender, EventArgs e)
        {
            float hodnota;
            String retezec = toolStripComboBox1.Text;
            if (retezec.Length == 0) return;
            if (retezec[retezec.Length - 1] != '%') return;
            retezec = retezec.Remove(retezec.Length - 1);
            if (float.TryParse(retezec, out hodnota) == false) return;
            if ((float)(hodnota / 100) <= 0 || (float)(hodnota / 100) > 2) return;
            nastaveni.ZoomTest = (float)(hodnota / 100);
            doubleBufPanel1.Invalidate();
        }

        private void cPMZdrojeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (puvodniZobrazeni != null) graf = puvodniZobrazeni;
            Vysledky vysledky = new Vysledky();
            postup.Vycisti();
            postup.Vloz("Začátek projektu: " + graf.ZacatekProjektu.ToLongDateString());
            ZmenaZobrazeni("sitovy graf");
            vysledky.NazevProjektu = graf.Nazev;
            vysledky.ZacatekProjektu = graf.ZacatekProjektu;
            prastromReseni.Init();
            graf.InitGraphics(g);
            double hodnotaCPM = 0; ;
            int chybaReseni = 0;
            hodnotaCPM = graf.AlgoritmusTopologickehoUsporadani(true);
            graf.AlgoritmusTopologickehoUsporadani2();
            chybaReseni = (int)hodnotaCPM;
            switch (chybaReseni)
            {
                case -1: MessageBox.Show("V síťovém grafu je více než jeden počátečních uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -2: MessageBox.Show("V síťovém grafu je více než jeden koncových uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -3: MessageBox.Show("V síťovém grafu není počáteční uzel.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -4: MessageBox.Show("V síťovém grafu není koncový uzel", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -5: MessageBox.Show("Síťový graf není acyklický.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
            }
            if (chybaReseni < 0)
            {
                foreach (Vrchol polozka in graf.vrcholy)
                {
                    polozka.MaximalniCas = 0;
                    polozka.MinimalniCas = 0;
                }
                return;
            }
            //BaBCallback metoda = new BaBCallback(graf.AlgoritmusBaB);
            progressPanel.Show();
            parametry = new ParametryBaB(prastromReseni, postup, progressPanel, doubleBufPanel1, nastaveni.ZobrazitVsechnaReseni, nastaveni);
            Thread t = new Thread(graf.AlgoritmusBaB);
            t.Start(parametry);
            //graf.AlgoritmusBaB(prastromReseni, informaceAlgoritmusListBox, projektProgressBar);
            //doubleBufPanel1.Enabled = false;
            //t.Join();
            zobrazPrastrom = true;
            //DialogPreruseni dialog = new DialogPreruseni();
            //dialog.Vlakno = t;
            //Thread t2 = new Thread(dialog.Show);
            //t2.Start();
            //dialog.ShowDialog();
            t.Join();
            //dialog.Close();
            //t2.Abort();
            zobrazPrastrom = false;
            if (parametry.chybaAlgoritmu > 0)
            {
                MessageBox.Show("Nedostatek zdrojů", "Upozornění", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                foreach (String polozka in postup.VycetPostupu1)
                {
                    postup.Vloz(polozka);
                }
                return;
            }
            int maxX = 0;
            int maxY = 0;
            foreach (Vrchol polozka in prastromReseni.vrcholy)
            {
                if (polozka.SouradniceX > maxX) maxX = polozka.SouradniceX;
                if (polozka.SouradniceY > maxY) maxY = polozka.SouradniceY;
            }
            doubleBufPanel1.Width = maxX + nastaveni.Velikost * nastaveni.Meritko * 20;
            doubleBufPanel1.Height = maxY + nastaveni.Velikost * nastaveni.Meritko * 20;
            ZmenaZobrazeni("prastrom reseni");
            postup.Vloz("Algoritmus ukončen. Výsledky: ");
            PoleVrcholu reseni = new PoleVrcholu();
            graf.kritickaCesta.Clear();
            double minimumOptimum = double.MaxValue;
            foreach (Vrchol polozka in prastromReseni.vrcholy)
            {
                if (polozka.MinimalniCas == polozka.MaximalniCas && polozka.MinimalniCas < minimumOptimum)
                    minimumOptimum = polozka.MinimalniCas;
            }
            foreach (Vrchol polozka in prastromReseni.vrcholy)
            {
                if (polozka.MinimalniCas == polozka.MaximalniCas && polozka.MinimalniCas == minimumOptimum)
                    reseni.Add(polozka);
            }
            if (nastaveni.Jednotky == "dny")
            {
                postup.Vloz("Trvání projektu: " + minimumOptimum.ToString() + " (ve dnech)");
            }
            else
            {
                postup.Vloz("Trvání projektu: " + minimumOptimum.ToString() + " (v hodinách)");
            }
            vysledky.DelkaProjektu = minimumOptimum;
            String retezec = "";
            String retezecZdroje = "";
            int citacReseni = 1;
            foreach (Vrchol polozka in reseni)
            {
                foreach (Zdroj zdrojItem in puvodniZobrazeni.zdroje)
                {
                    zdrojItem.Vyuziti = 0;
                    if (zdrojItem.Obnovitelnost == false) zdrojItem.AktualniKapacita = 0;
                }
                retezec = "";
                retezecZdroje = "";
                VypocetPostup jednoReseni = new VypocetPostup();
                Vrchol item = polozka;
                {
                    Vrchol pomItem = new Vrchol();
                    item.Kopie(pomItem);
                    while (pomItem.PocetPredchudcu > 0)
                    {
                        Hrana novaHrana = (Hrana)(prastromReseni.predchudci[pomItem.IndexPrvnihoPredchudce]);
                        Hrana vkladanaHrana = new Hrana(novaHrana.Vrchol1, novaHrana.Vrchol2);
                        novaHrana.Kopie(vkladanaHrana);
                        prastromReseni.kritickaCesta.Add(vkladanaHrana);
                        retezec = retezec.Insert(0, vkladanaHrana.Data.Nazev + " -> ");
                        jednoReseni.VlozNaZacatek(vkladanaHrana.Data.Nazev + " - Vykonáno: " + vkladanaHrana.Data.Vykonano);
                        foreach (ZdrojHrany polozkaZdroj in vkladanaHrana.Data.zdrojeHrany)
                        {
                            retezecZdroje = retezecZdroje.Insert(0, polozkaZdroj.Nazev + "-" + polozkaZdroj.Spotreba + ", ");
                            Zdroj pomkaZdroj = new Zdroj();
                            pomkaZdroj.Nazev = polozkaZdroj.Nazev;
                            pomkaZdroj = (Zdroj)(puvodniZobrazeni.zdroje[puvodniZobrazeni.zdroje.Najdi(pomkaZdroj, 0)]);
                            if (pomkaZdroj.Obnovitelnost == true)
                                pomkaZdroj.Vyuziti += (vkladanaHrana.Data.Vykonano / minimumOptimum) * (polozkaZdroj.Spotreba / pomkaZdroj.PuvodniKapacita);
                        }
                        pomItem = new Vrchol(novaHrana.Vrchol1.GetKlic());
                        pomItem = (Vrchol)(prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(pomItem, 0)]);
                    }
                }
                if (retezec.Length >= 4)
                    retezec = retezec.Remove(retezec.Length - 4, 4);
                postup.Vloz(citacReseni.ToString() + ". reseni: " + retezec);
                vysledky.Reseni.Add(jednoReseni);
                citacReseni++;
            }
            reseni.Clear();
            if (nastaveni.Jednotky == "dny")
            {
                postup.Vloz("Konec projektu: " + (puvodniZobrazeni.ZacatekProjektu.AddDays(minimumOptimum)).ToLongDateString());
                vysledky.KonecProjektu = puvodniZobrazeni.ZacatekProjektu.AddDays(minimumOptimum);
            }
            else
            {
                postup.Vloz("Konec projektu: " + (puvodniZobrazeni.ZacatekProjektu.AddDays(minimumOptimum)).ToLongDateString());
                vysledky.KonecProjektu = puvodniZobrazeni.ZacatekProjektu.AddHours(minimumOptimum);
            }
            vysledky.VycetZdroju.Vycisti();
            vysledky.NakladyProjektu.Vycisti();
            vysledky.CelkoveNaklady = 0;
            foreach (Hrana itemHrana in puvodniZobrazeni.naslednici)
            {
                foreach (ZdrojHrany polozkaZdroj in itemHrana.Data.zdrojeHrany)
                {
                    Zdroj pomkaZdroj = new Zdroj();
                    pomkaZdroj.Nazev = polozkaZdroj.Nazev;
                    pomkaZdroj = (Zdroj)(puvodniZobrazeni.zdroje[puvodniZobrazeni.zdroje.Najdi(pomkaZdroj, 0)]);
                    if (pomkaZdroj.Obnovitelnost == false)
                        pomkaZdroj.AktualniKapacita += polozkaZdroj.Spotreba;
                }
            }
            foreach (Zdroj itemZ in puvodniZobrazeni.zdroje)
            {
                if (itemZ.Obnovitelnost == true)
                    vysledky.VycetZdroju.Vloz(itemZ.Nazev + " - Kapacita: " + itemZ.PuvodniKapacita + " - Obnovitelný zdroj");
                else
                {
                    vysledky.VycetZdroju.Vloz(itemZ.Nazev + " - Spotřebováno: " + itemZ.AktualniKapacita + " - Spotřební zdroj");
                    vysledky.NakladyProjektu.Vloz(itemZ.Nazev + " - Náklady (počet * cena): " + (itemZ.AktualniKapacita).ToString() + " * " + itemZ.Cena.ToString() + " = " + String.Format("{0:F2}",(itemZ.Cena * itemZ.AktualniKapacita)));
                    vysledky.CelkoveNaklady += itemZ.Cena * itemZ.AktualniKapacita;
                }
            }
            postup.Vloz(" ");
            postup.Vloz("Vytíženost obnovitelných zdrojů");
            vysledky.VytizenostZdroju.Vycisti();
            foreach (Zdroj zdrojItem in puvodniZobrazeni.zdroje)
            {
                if (zdrojItem.Obnovitelnost == true)
                {
                    postup.Vloz(zdrojItem.Nazev + ": " + (zdrojItem.Vyuziti * 100) + "%");
                    vysledky.VytizenostZdroju.Vloz(zdrojItem.Nazev + ": " + String.Format("{0:F2}", (zdrojItem.Vyuziti * 100)) + "%");
                    vysledky.NakladyProjektu.Vloz(zdrojItem.Nazev + " - Náklady (dny * cena): " + (minimumOptimum).ToString() + " * " + zdrojItem.Cena.ToString() + " = " + String.Format("{0:F2}", (zdrojItem.Cena * (minimumOptimum * zdrojItem.PuvodniKapacita))) + " Skutečně odpracováno: " +String.Format("{0:F2}", (zdrojItem.Cena * zdrojItem.Vyuziti * minimumOptimum * zdrojItem.PuvodniKapacita)));
                    vysledky.CelkoveNaklady += zdrojItem.Cena * minimumOptimum * zdrojItem.PuvodniKapacita;
                }
            }
            postup.Vloz("Cena neobnovitelných zdrojů");
            foreach (Zdroj item in puvodniZobrazeni.zdroje)
            {
                if (item.Obnovitelnost == false)
                {
                    postup.Vloz("Zdroj " + item.Nazev + " spotřebován: " + item.AktualniKapacita + ". Náklady: " + (item.AktualniKapacita * item.Cena).ToString());
                    item.AktualniKapacita = 0;
                }
            }
            progressPanel.Hide();
            dialogVysledky.NazevProjektu = puvodniZobrazeni.Nazev;
            dialogVysledky.Vysledky = vysledky;
            dialogVysledky.Zobraz();
            dialogVysledky.Show();
            if (nastaveni.VyukovyMod == false) VypisAlgoritmu();
            else
            {
                dialogVypis.Postup = postup;
                dialogVypis.NazevProjektu = puvodniZobrazeni.Nazev;
                dialogVypis.Zobraz();
                dialogVypis.Show();

            }
            if (prastromReseni.vrcholy.Count > 5000)
            {
                MessageBox.Show("Není doporučeno zobrazit prastrom řešení z důvodu velkého počtu uzlů prastromu\nPočet uzlů v prastromu: " + prastromReseni.vrcholy.Count);
                zobrazPrastrom = false;
                ZmenaZobrazeni("sitovy graf");
            }
            else
            {
                int maxWidth = 0, maxHeight = 0;
                foreach (Vrchol uzelItem in prastromReseni)
                {
                    if (uzelItem.SouradniceX > maxWidth) maxWidth = uzelItem.SouradniceX;
                    if (uzelItem.SouradniceY > maxHeight) maxHeight = uzelItem.SouradniceY;
                }
                doubleBufPanel1.Width = maxWidth + 50;
                doubleBufPanel1.Height = maxHeight + 50;
            }
            doubleBufPanel1.Invalidate();
        }

        private void clearToolStripButton_Click(object sender, EventArgs e)
        {
            if (graf.TypOhodnoceniHran == "PERT")
            {
                doubleBufPanel1.Cursor = Cursors.Cross;
                clearButton();
                clearToolStripButton.Checked = true;
                predchozitypEditace = typEditace;
                typEditace = "PERT";
            }
        }

        private void tabulkováEditaceToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (puvodniZobrazeni != null)
                using (DialogTabulkovaEditace dialog = new DialogTabulkovaEditace(puvodniZobrazeni))
                {
                    dialog.TypEditace = graf.TypOhodnoceniHran;
                    if (dialog.ShowDialog() == DialogResult.OK)
                    {
                        if (graf.vrcholy.Count == 0) return;
                        graf.kritickaCesta.Clear();
                    }
                }
            else
            using (DialogTabulkovaEditace dialog = new DialogTabulkovaEditace(graf))
            {
                dialog.TypEditace = graf.TypOhodnoceniHran;
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    if (graf.vrcholy.Count == 0) return;
                    graf.kritickaCesta.Clear();
                }
            }
        }

        private void panel1_Scroll(object sender, ScrollEventArgs e)
        {
            if (panel1.HorizontalScroll.Value >= puvodniH)
            {
                puvodniH += nastaveni.Velikost * nastaveni.Meritko * 20;
                doubleBufPanel1.Width += nastaveni.Velikost * nastaveni.Meritko * 20;
            }
            if (panel1.VerticalScroll.Value >= puvodniV)
            {
                puvodniV += nastaveni.Velikost * nastaveni.Meritko * 20;
                doubleBufPanel1.Height += nastaveni.Velikost * nastaveni.Meritko * 20;
            }

        }

        private void konecToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void posunToolStripButton_Click(object sender, EventArgs e)
        {
            if (doubleBufPanel1.Width > 5000) return;
            doubleBufPanel1.Width += doubleBufPanel1.Width;
            doubleBufPanel1.Height += doubleBufPanel1.Height;
        }

        private void panel1_SizeChanged(object sender, EventArgs e)
        {
            doubleBufPanel1.Width = panel1.Width + 1000;
            doubleBufPanel1.Height = panel1.Height + 1000;
        }

        private void sítovýGrafToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ZmenaZobrazeni("sitovy graf");
        }

        private void tabulkaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ZmenaZobrazeni("tabulka");
        }

        private void prastromŘešeníToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ZmenaZobrazeni("prastrom reseni");
        }

        private void ganntůvDiagramToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (puvodniZobrazeni != null) graf = puvodniZobrazeni;
            ZmenaZobrazeni("ganttuv diagram");
        }

        private void pERTToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ZmenaZobrazeni("sitovy graf");
            postup.Vycisti();
            postup.Vloz("Začátek projektu: " + graf.ZacatekProjektu.ToLongDateString());
            Vysledky vysledky = new Vysledky();
            vysledky.NazevProjektu = graf.Nazev;
            vysledky.ZacatekProjektu = graf.ZacatekProjektu;
            vysledky.Typ = 1;
            double hodnotaCPM = 0; ;
            int chybaReseni = 0;
            double hodnota;
            string cislo;
            foreach (Hrana polozka in graf.naslednici)
            {
                cislo = String.Format("{0:F2}", (polozka.Data.A + 4 * polozka.Data.M + polozka.Data.B) / 6);
                hodnota = double.Parse(cislo);
                polozka.Data.Ohodnoceni = hodnota;
            }
            if (nastaveni.VyukovyMod == true)
            {
                hodnotaCPM = graf.AlgoritmusTopologickehoUsporadani(true, postup);
                graf.AlgoritmusTopologickehoUsporadani2(postup);
            }
            else
            {
                hodnotaCPM = graf.AlgoritmusTopologickehoUsporadani(true, postup);
                graf.AlgoritmusTopologickehoUsporadani2(postup);
            }
            chybaReseni = (int)hodnotaCPM;
            switch (chybaReseni)
            {
                case -1: MessageBox.Show("V síťovém grafu je více než jeden počátečních uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -2: MessageBox.Show("V síťovém grafu je více než jeden koncových uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -3: MessageBox.Show("V síťovém grafu není počáteční uzel.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -4: MessageBox.Show("V síťovém grafu není koncový uzel", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
                case -5: MessageBox.Show("Síťový graf není acyklický.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    break;
            }
            graf.VypocetRezerv();
            postup.Vloz(" ");
            postup.Vloz("Délka projektu: " + hodnotaCPM.ToString() + " (ve dnech)");
            postup.Vloz("Konec projektu: " + (graf.ZacatekProjektu.AddDays(hodnotaCPM)).ToLongDateString());
            postup.Vloz("Kritické činnosti:");
            String retezec = ""; 
            VypocetPostup jednoReseni = new VypocetPostup();
            foreach (Hrana polozka in graf.kritickaCesta)
            {
                if (nastaveni.PopisCinnosti == 1) retezec += polozka.Data.Zkratka + ",";
                else retezec += polozka.Data.Nazev + ",";

                jednoReseni.VlozNaZacatek(polozka.Data.Nazev + " - ohodnocení: " + polozka.Data.A + "-" +polozka.Data.M+"-"+polozka.Data.B+"("+polozka.Data.Ohodnoceni+")");
            }
            vysledky.DelkaProjektu = hodnotaCPM;
            vysledky.KonecProjektu=graf.ZacatekProjektu.AddDays(hodnotaCPM);
            vysledky.Reseni = new System.Collections.ArrayList(1);
            vysledky.Reseni.Add(jednoReseni);
            if (retezec.Length >= 1) retezec = retezec.Remove(retezec.Length - 1, 1);
            postup.Vloz(retezec);
            if (nastaveni.VyukovyMod == true) graf.VypocetRezerv(postup, nastaveni);
            if (nastaveni.VyukovyMod == false) VypisAlgoritmu();
            else
            {
                using (DialogVypis dialog = new DialogVypis())
                {
                    dialog.Postup = postup;
                    if (dialog.ShowDialog() == DialogResult.OK)
                    {

                    }
                }
            }
            dialogVysledky.NazevProjektu = graf.Nazev;
            dialogVysledky.Vysledky = vysledky;
            dialogVysledky.Zobraz();
            dialogVysledky.Show();
            if (nastaveni.VyukovyMod == false) VypisAlgoritmu();
            else
            {
                dialogVypis.Postup = postup;
                dialogVypis.NazevProjektu = graf.Nazev;
                dialogVypis.Zobraz();
                dialogVypis.Show();

            }
            doubleBufPanel1.Invalidate();
        }

        private void hlavniOkno_Resize(object sender, EventArgs e)
        {
            ResizePlatno();
        }

        private void reseniToolStripComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (reseniToolStripComboBox.Items.Count > 0)
            {
                ZmenaZobrazeni("ganttuv diagram", reseniToolStripComboBox.SelectedIndex);
                doubleBufPanel1.Invalidate();
            }
        }

        private void cpmTypToolStripMenuItem_Click(object sender, EventArgs e)
        {
            pertTypToolStripMenuItem.Checked = false;
            cpmTypToolStripMenuItem.Checked = true;
            graf.TypOhodnoceniHran = "CPM";
        }

        private void pertTypToolStripMenuItem_Click(object sender, EventArgs e)
        {
            pertTypToolStripMenuItem.Checked = true;
            cpmTypToolStripMenuItem.Checked = false;
            graf.TypOhodnoceniHran = "PERT";
        }

        private void nastaveníVrhToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NastaveniVzhled backup = nastaveni;
            using (DialogVzhledVrcholHrana dialog = new DialogVzhledVrcholHrana(graf))
            {
                dialog.Nastaveni = nastaveni;
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    if (puvodniZobrazeni != null) graf = puvodniZobrazeni;
                    doubleBufPanel1.Invalidate();
                }
                else nastaveni = backup;
            }
        }

        private void button1_Click_1(object sender, EventArgs e)
        {

        }

        private void informaceAlgoritmusListBox_DoubleClick(object sender, EventArgs e)
        {
        }

        private void časovéJednotkyToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void nastaveníČasuToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void progressPanel_Paint(object sender, PaintEventArgs e)
        {
            if (parametry == null) return;
            g3.DrawRectangle(new Pen(new SolidBrush(Color.Black)), 0, 30, 200, 25);
            g3.FillRectangle(new SolidBrush(Color.GreenYellow), 1, 31, parametry.progress, 23);
            g3.DrawString("Počet podproblémů: " + parametry.pocetPodreseni.ToString(), nastaveni.PismoFont, new SolidBrush(Color.Black), 10, 10);
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {

        }

        private void hlavniOkno_Load(object sender, EventArgs e)
        {
            nazevProjektToolStripTextBox.Text = graf.Nazev;
            nastaveni.Nacti();
        }

        private void nazevProjektToolStripTextBox_Click(object sender, EventArgs e)
        {
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            otevritToolStripMenuItem_Click(sender, e);
        }

        private void uložitNastaveníToolStripMenuItem_Click(object sender, EventArgs e)
        {
            nastaveni.Uloz();
        }

        private void ulozitStripButton1_Click(object sender, EventArgs e)
        {
            ulozitToolStripMenuItem_Click(sender, e);
        }

        private void ohodnocenToolStripMenuItem_Click(object sender, EventArgs e)
        {
        
        }

        private void prerusitButton_Click(object sender, EventArgs e)
        {

        }

        private void nápovědaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            dialogNapoveda.naplneniHesly();
            dialogNapoveda.Show();
        }

        private void nazevProjektToolStripTextBox_TextChanged(object sender, EventArgs e)
        {
            graf.Nazev = nazevProjektToolStripTextBox.Text;
        }
    }
}
