﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using System.Collections;
using System.Drawing.Drawing2D;
using System.Threading;

namespace DiplomovaPrace
{
    public class Graf : IEnumerable
    {
        private String nazev;

        private int pocetVrcholu = 0;
        private Vrchol nejblizsiUzel = null;
        private Hrana nejblizsiHrana = null;

        private double hodnotaCPM = 0;
        private int poradiVrcholuPrastromu = 1;
        public bool kreslitBaB = false;
        private string typOhodnoceniHran = "CPM";

        private Graphics grafika;
        private DateTime zacatekProjektu;
        public SeznamHran kritickaCesta = new SeznamHran();
        private PoleVrcholu neprozkoumanaReseni = new PoleVrcholu();
        private double minimumProjektu = double.MaxValue;

        #region vlastnosti
        public String Nazev
        {
            get { return nazev; }
            set { nazev = value; }
        }
        public DateTime ZacatekProjektu
        {
            get { return zacatekProjektu; }
            set { zacatekProjektu = value; }
        }
        public string TypOhodnoceniHran
        {
            get { return typOhodnoceniHran; }
            set { typOhodnoceniHran = value; }
        }

        public Hrana NejblizsiHrana
        {
            get { return nejblizsiHrana; }
            set { nejblizsiHrana = value; }
        }

        public Vrchol NejblizsiUzel
        {
            get { return nejblizsiUzel; }
            set { nejblizsiUzel = value; }
        }
        public int PocetVrcholu
        {
            get { return pocetVrcholu; }
            set { pocetVrcholu = value; }
        }
        #endregion

        public PoleVrcholu vrcholy = new PoleVrcholu();
        public SeznamHran predchudci = new SeznamHran();
        public SeznamHran naslednici = new SeznamHran();
        public SeznamZdroju zdroje = new SeznamZdroju();

        /// <summary>
        /// Vloží do grafu vrchol.
        /// </summary>
        /// <param name="vrchol">Vkládaný vrchol.</param>
        /// <returns>Vrací 1, pokud se uzel nepodařilo vložit. 0, pokud vše proběhlo v pořádku.</returns>
        public int VlozVrchol(Vrchol vrchol)
        {
            int pom = vrcholy.Najdi(vrchol, 0);
            if (pom >= 0)
            {
                return 1; //nastala chyba, vrchol jiz existuje
            }
            vrchol.IndexPrvnihoNaslednika = -1;
            vrchol.PocetNasledniku = 0;
            vrchol.IndexPrvnihoPredchudce = -1;
            vrchol.PocetPredchudcu = 0;
            //vrcholy.Add(vrchol);
            vrcholy.Insert(vrchol);
            //MessageBox.Show(naslednici.Count.ToString(), "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);           
            pocetVrcholu++;
            return 0; //nenastala chyba
        }

        /// <summary>
        /// Vkládá hranu do grafu.
        /// </summary>
        /// <param name="hrana">Vkládaná hrana.</param>
        /// <returns>
        /// 0 - hrana byla vložena
        /// 1 - hrana nebyla vložena z důvodu nenalezení jednoho nebo obou z incidentních vrcholů
        /// 2 - v grafu je násobná hrana
        /// 3 - v grafu je smyčka
        /// </returns>
        public int VlozHranu(Hrana hrana)
        {
            Vrchol pomVrchol = new Vrchol();
            if (vrcholy.Sorted == false) vrcholy.Sort(new CompareKlic(true));
            pomVrchol.Klic.SetKlic(hrana.Vrchol1.GetKlic());
            int indexPrvnihoVrcholu = vrcholy.Najdi(pomVrchol, 0); //0 - vyhledavani podle klice
            if (indexPrvnihoVrcholu <= -1)
            {
                return 1;
            }
            pomVrchol.Klic.SetKlic(hrana.Vrchol2.GetKlic());
            int indexDruhehoVrcholu = vrcholy.Najdi(pomVrchol, 0);
            if (indexDruhehoVrcholu <= -1)
            {
                return 1;
            }
            if (naslednici.Najdi(hrana, 0) >= 0) return 2; //nasobna hrana
            if (hrana.Vrchol1.GetKlic() == hrana.Vrchol2.GetKlic()) return 3; //smycka
            //***** Vkladani Nasledniku *****
            pomVrchol = (Vrchol)vrcholy[indexPrvnihoVrcholu];
            if (naslednici.Count > 0) //seznam nasledniku neni prazdny
            {
                int nalezenaPozice = -1;
                if (pomVrchol.PocetNasledniku == 0) //vrchol jeste nema definovany index naslednika
                {
                    //nalezeni nejvyssiho indexu naslednika z predchazejicich vrcholu
                    for (int i = indexPrvnihoVrcholu - 1; i >= 0; i--)
                    {
                        if (((Vrchol)vrcholy[i]).PocetNasledniku > 0)
                        {
                            nalezenaPozice = ((Vrchol)vrcholy[i]).IndexPrvnihoNaslednika + ((Vrchol)vrcholy[i]).PocetNasledniku;
                            break; //nasel pozici, na kterou vlozit
                        }
                    }
                    if (nalezenaPozice == -1) nalezenaPozice = 0; //pokud nema zadny predchozi vrchol naslednika
                    pomVrchol.IndexPrvnihoNaslednika = nalezenaPozice;
                }
            }
            else //seznam nasledniku je prazdny
            {
                pomVrchol.IndexPrvnihoNaslednika = 0;
            }
            naslednici.Insert(pomVrchol.IndexPrvnihoNaslednika, hrana);
            pomVrchol.PocetNasledniku += 1;
            //zbytek vrcholu musi posunout svuj index o 1
            for (int i = indexPrvnihoVrcholu + 1; i < vrcholy.Count; i++)
            {
                if (((Vrchol)vrcholy[i]).PocetNasledniku > 0)
                    ((Vrchol)vrcholy[i]).IndexPrvnihoNaslednika = ((Vrchol)vrcholy[i]).IndexPrvnihoNaslednika + 1;
            }
            //***** Konec vkladani Nasledniku *****

            //***** Vkladani Predchudcu *****
            pomVrchol = (Vrchol)vrcholy[indexDruhehoVrcholu];

            if (predchudci.Count > 0) //seznam nasledniku neni prazdny
            {
                int nalezenaPozice = -1;
                if (pomVrchol.PocetPredchudcu == 0)
                {
                    //nalezeni nejvyssiho indexu naslednika z predchazejicich vrcholu
                    for (int i = indexDruhehoVrcholu - 1; i >= 0; i--)
                    {
                        if (((Vrchol)vrcholy[i]).PocetPredchudcu > 0)
                        {
                            nalezenaPozice = ((Vrchol)vrcholy[i]).IndexPrvnihoPredchudce + ((Vrchol)vrcholy[i]).PocetPredchudcu;
                            break; //nasel pozici, na kterou vlozit
                        }
                    }
                    if (nalezenaPozice == -1) nalezenaPozice = 0; //pokud nema zadny predchozi vrchol naslednika
                    pomVrchol.IndexPrvnihoPredchudce = nalezenaPozice;
                }
            }
            else //seznam nasledniku je prazdny
            {
                pomVrchol.IndexPrvnihoPredchudce = 0;
            }
            predchudci.Insert(pomVrchol.IndexPrvnihoPredchudce, hrana);
            pomVrchol.PocetPredchudcu += 1;
            //zbytek vrcholu musi posunout svuj index o 1
            for (int i = indexDruhehoVrcholu + 1; i < vrcholy.Count; i++)
            {
                if (((Vrchol)vrcholy[i]).PocetPredchudcu > 0)
                    ((Vrchol)vrcholy[i]).IndexPrvnihoPredchudce = ((Vrchol)vrcholy[i]).IndexPrvnihoPredchudce + 1;
            }
            //***** Konec vkladani Predchudcu *****
            return 0;
        }

        public IEnumerator GetEnumerator(Vrchol vrchol)
        {
            for (int i = vrchol.IndexPrvnihoPredchudce; i < vrchol.IndexPrvnihoPredchudce + vrchol.PocetPredchudcu; i++)
            {
                yield return predchudci[i];
            }
        }

        /// <summary>
        /// Vrací předchůdce uzlu.
        /// </summary>
        /// <param name="vrchol">Uzel pro který chceme zjistit předchůdce.</param>
        /// <returns>Předchůdce uzlu.</returns>
        public IEnumerable ZpristupniPredchudce(Vrchol vrchol)
        {
            for (int i = vrchol.IndexPrvnihoPredchudce; i < vrchol.IndexPrvnihoPredchudce + vrchol.PocetPredchudcu; i++)
            {
                yield return predchudci[i];
            }
        }
        /// <summary>
        /// Vrací následníky uzlu.
        /// </summary>
        /// <param name="vrchol">Uzel pro který chceme zjistit následníky.</param>
        /// <returns>Následníci uzlu.</returns>
        public IEnumerable ZpristupniNasledniky(Vrchol vrchol)
        {
            for (int i = vrchol.IndexPrvnihoNaslednika; i < vrchol.IndexPrvnihoNaslednika + vrchol.PocetNasledniku; i++)
            {
                yield return naslednici[i];
            }
        }

        /// <summary>
        /// Odebere hranu z grafu.
        /// </summary>
        /// <param name="hrana">Odebíraná hrana.</param>
        /// <returns>Vrací 0, pokud proběhlo odebírání v pořádku, jinak vrací 1.</returns>
        public int OdeberHranu(Hrana hrana)
        {
            if (naslednici.Count <= 0) return 1; //seznam nasledniku je prazdny, tudiz i predchudcu
            Vrchol pomVrchol = new Vrchol();
            if (vrcholy.Sorted == false) vrcholy.Sort(new CompareKlic(true));
            pomVrchol.Klic.SetKlic(hrana.Vrchol1.GetKlic());
            int indexPrvnihoVrcholu = vrcholy.Najdi(pomVrchol, 0); //0 - vyhledavani podle klice
            if (indexPrvnihoVrcholu <= -1)
            {
                return 1;
            }
            pomVrchol.Klic.SetKlic(hrana.Vrchol2.GetKlic());
            int indexDruhehoVrcholu = vrcholy.Najdi(pomVrchol, 0);
            if (indexDruhehoVrcholu <= -1)
            {
                return 1;
            }
            //***** Odebrani z Nasledniku *****
            pomVrchol = (Vrchol)vrcholy[indexPrvnihoVrcholu];
            int pocetIncidenci = pomVrchol.PocetNasledniku;
            int mazanaPozice = -1;
            for (int i = pomVrchol.IndexPrvnihoNaslednika; i < pomVrchol.IndexPrvnihoNaslednika + pocetIncidenci; i++)
            {
                if (((Hrana)naslednici[i]).Vrchol2.GetKlic() == ((Vrchol)vrcholy[indexDruhehoVrcholu]).Klic.GetKlic())
                {
                    mazanaPozice = i;
                    break;
                }
            }
            if (mazanaPozice == -1) return 1; //nenalezen hledany naslednik
            //dealokace pameti
            naslednici[mazanaPozice] = null;

            naslednici.RemoveAt(mazanaPozice);
            pomVrchol.PocetNasledniku = pomVrchol.PocetNasledniku - 1;
            if (pomVrchol.PocetNasledniku == 0) pomVrchol.IndexPrvnihoNaslednika = -1;
            //zbytek vrcholu musi posunout svuj index o 1
            for (int i = indexPrvnihoVrcholu + 1; i < vrcholy.Count; i++)
            {
                if (((Vrchol)vrcholy[i]).PocetNasledniku > 0)
                    ((Vrchol)vrcholy[i]).IndexPrvnihoNaslednika = ((Vrchol)vrcholy[i]).IndexPrvnihoNaslednika - 1;
            }
            //***** Konec odebirani z nasledniku *****

            //***** Odebrani z predchudcu *****
            pomVrchol = (Vrchol)vrcholy[indexDruhehoVrcholu];
            pocetIncidenci = pomVrchol.PocetPredchudcu;
            mazanaPozice = -1;
            for (int i = pomVrchol.IndexPrvnihoPredchudce; i < pomVrchol.IndexPrvnihoPredchudce + pocetIncidenci; i++)
            {
                if (((Hrana)predchudci[i]).Vrchol1.GetKlic() == ((Vrchol)vrcholy[indexPrvnihoVrcholu]).Klic.GetKlic())
                {
                    mazanaPozice = i;
                    break;
                }
            }
            if (mazanaPozice == -1) return 1; //nenalezen hledany predchudce
            //dealokace pameti
            predchudci[mazanaPozice] = null;

            predchudci.RemoveAt(mazanaPozice);
            pomVrchol.PocetPredchudcu -= 1;
            if (pomVrchol.PocetPredchudcu == 0) pomVrchol.IndexPrvnihoPredchudce = -1;
            //zbytek vrcholu musi posunout svuj index o 1
            for (int i = indexDruhehoVrcholu + 1; i < vrcholy.Count; i++)
            {
                if (((Vrchol)vrcholy[i]).PocetPredchudcu > 0)
                    ((Vrchol)vrcholy[i]).IndexPrvnihoPredchudce = ((Vrchol)vrcholy[i]).IndexPrvnihoPredchudce - 1;
            }
            //***** Konec odebirani z predchudcu *****
            return 0;
        }
        
        /// <summary>
        /// Odebere uzel z grafu.
        /// </summary>
        /// <param name="vrchol">Odebíraný uzel.</param>
        /// <returns>Vrací 0, pokud proběhlo odebírání v pořádku, jinak vrací 1.</returns>
        public int OdeberVrchol(Vrchol vrchol)
        {
            Hrana mazanaHrana;
            if (vrcholy.Sorted == false) vrcholy.Sort(new CompareKlic(true));
            int pom = vrcholy.Najdi(vrchol, 0);
            if (pom < 0)
            {
                //MessageBox.Show("Prvek s tímto kličem neexistuje", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return 1; //nastala chyba, vrchol neexistuje
            }

            //odebrani vsech incidenci k naslednikum
            int pocetMazanychHran = ((Vrchol)vrcholy[pom]).PocetNasledniku;
            for (int i = 0; i < pocetMazanychHran; i++)
            {
                //MessageBox.Show(((Vrchol)vrcholy[pom]).Klic.GetKlic() + ((Hrana)naslednici[((Vrchol)vrcholy[pom]).IndexPrvnihoNaslednika]).Vrchol2.GetKlic());

                mazanaHrana = (Hrana)naslednici[((Vrchol)vrcholy[pom]).IndexPrvnihoNaslednika];
                if (OdeberHranu(mazanaHrana) <= -1)
                {
                    //MessageBox.Show("Chyba při odstraňování vrcholu.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return 1; //nastala chyba, hrana se neodstranila
                }
            }
            //odebrani vsech incidenci k predchudcum
            pocetMazanychHran = ((Vrchol)vrcholy[pom]).PocetPredchudcu;
            for (int i = 0; i < pocetMazanychHran; i++)
            {
                //MessageBox.Show(((Hrana)predchudci[((Vrchol)vrcholy[pom]).IndexPrvnihoPredchudce]).Vrchol1.GetKlic() + ((Vrchol)vrcholy[pom]).Klic.GetKlic());

                mazanaHrana = (Hrana)predchudci[((Vrchol)vrcholy[pom]).IndexPrvnihoPredchudce];
                if (OdeberHranu(mazanaHrana) <= -1)
                {
                    //MessageBox.Show("Chyba při odstraňování vrcholu.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return 1; //nastala chyba, hrana se neodstranila
                }
            }

            //dealokace pameti
            vrcholy[pom] = null;
            vrcholy.RemoveAt(pom);

            return 0;
        }

        /// <summary>
        /// Zpřístupní počáteční uzel grafu.
        /// </summary>
        /// <returns>Počáteční uzel grafu typu <typeparamref name="Vrchol"/></returns>
        public Vrchol ZpristupniPocatecniUzel()
        {
            foreach (Vrchol polozka in vrcholy)
            {
                if (polozka.PocetPredchudcu == 0) return polozka;
            }
            return null;
        }

        /// <summary>
        /// Vrcholová prohlídka grafu od uzlu.
        /// </summary>
        /// <param name="vrchol">Uzel, od kterého začíná prohlídka.</param>
        public void Prohlidka(Vrchol vrchol)
        {
            if (vrchol.PocetNasledniku > 0)
            {
                foreach (Hrana polozka in ZpristupniNasledniky(vrchol))
                {
                    int maximum = 0;
                    foreach (Hrana item in ZpristupniPredchudce(vrchol))
                    {
                        Vrchol pomVrchol = new Vrchol(item.Vrchol1.GetKlic());
                        pomVrchol = (Vrchol)(vrcholy[vrcholy.Najdi(pomVrchol, 0)]);
                        if (pomVrchol.SouradniceY > maximum) maximum = pomVrchol.SouradniceY;
                    }
                    if (vrchol.SouradniceY < maximum) vrchol.SouradniceY = maximum;
                    Vrchol pomVrcholNext = new Vrchol(polozka.Vrchol2.GetKlic());
                    pomVrcholNext = (Vrchol)(vrcholy[vrcholy.Najdi(pomVrcholNext, 0)]);
                    vrchol = pomVrcholNext;
                    Prohlidka(vrchol);
                }
            }
        }

        /// <summary>
        /// Vkladá zdroj do seznamu
        /// Vrací <p>int</p>
        /// 0 - bez chyby
        /// 1 - zdroj již existuje
        /// </summary>
        /// <param name="zdroj">vkládaný zdroj</param>
        /// <returns>vrací <p>int</p></para></returns>
        public int VlozZdroj(Zdroj zdroj)
        {
            if (zdroje.Najdi(zdroj, 0) >= 0) return 1;
            zdroje.Add(zdroj);
            return 0;
        }
        /// <summary>
        /// Odebere zdroj ze seznamu zdrojů
        /// 0 - zdroj byl odebrán
        /// 1 - zdroj nebyl nalezen
        /// </summary>
        /// <param name="zdroj">mazaný zdroj</param>
        /// <returns></returns>
        public int OdeberZdroj(Zdroj zdroj)
        {
            int pozice = zdroje.Najdi(zdroj, 0);
            if (pozice < 0) return 1;
            zdroje.RemoveAt(pozice);
            return 0;
        }

        public void InitGraphics(Graphics g)
        {
            this.grafika = g;
        }
        
        /// <summary>
        /// Inicializuje atributy grafu.
        /// </summary>
        public void Init()
        {
            ZacatekProjektu = DateTime.Now;
            vrcholy.Clear();
            naslednici.Clear();
            predchudci.Clear();
            zdroje.Clear();
            kritickaCesta.Clear();
            pocetVrcholu = 0;
            nejblizsiUzel = null;
            nejblizsiHrana = null;
            hodnotaCPM = 0;
            poradiVrcholuPrastromu = 1;
        }

        /// <summary>
        /// Inicializuje atributy grafu, ale ponechává seznam zdrojů.
        /// </summary>
        public void InitKeepZdroje()
        {
            vrcholy.Clear();
            naslednici.Clear();
            predchudci.Clear();
            kritickaCesta.Clear();
            pocetVrcholu = 0;
            nejblizsiUzel = null;
            nejblizsiHrana = null;
            hodnotaCPM = 0;
            poradiVrcholuPrastromu = 1;
        }
        public IEnumerator GetEnumerator()
        {
            for (int i = 0; i < vrcholy.Count; i++)
            {
                yield return (Vrchol)vrcholy[i];
            }
        }

        /// <summary>
        /// Vrací činnosti bezprostředně následující za uzlem.
        /// </summary>
        /// <param name="vrchol">Uzel pro který chceme vypsat následné činnosti.</param>
        /// <returns></returns>
        public IEnumerable IncidenceNaslednici(Vrchol vrchol)
        {
            for (int i = vrchol.IndexPrvnihoNaslednika; i < vrchol.IndexPrvnihoNaslednika + vrchol.PocetNasledniku; i++)
            {
                yield return (Hrana)naslednici[i];
            }
        }
        /// <summary>
        /// Vrací činnosti bezprostředně předcházející před uzlem.
        /// </summary>
        /// <param name="vrchol">Uzel pro který chceme vypsat předcházející činnosti.</param>
        /// <returns></returns>
        public IEnumerable IncidencePredchudci(Vrchol vrchol)
        {
            for (int i = vrchol.IndexPrvnihoPredchudce; i < vrchol.IndexPrvnihoPredchudce + vrchol.PocetPredchudcu; i++)
            {
                yield return (Hrana)predchudci[i];
            }
        }

        #region prace se soubory
        /// <summary>
        /// Načte data z datového proudu z textového souboru.
        /// </summary>
        /// <param name="sr">Datový proud.</param>
        public void NactiZeSouboru(StreamReader sr)
        {
            nazev = sr.ReadLine();
            typOhodnoceniHran = sr.ReadLine();
            string datum = sr.ReadLine();
            DateTime.TryParse(datum, out zacatekProjektu);
            NactiVrcholy(sr);
            NactiHrany(sr);
            NactiZdroje(sr);
        }
        /// <summary>
        /// Načte data z datového proudu z binárního souboru.
        /// </summary>
        /// <param name="br">Datový proud.</param>
        public void NactiZeSouboru(BinaryReader br)
        {
            nazev = br.ReadString();
            typOhodnoceniHran = br.ReadString();
            string datum = br.ReadString();
            DateTime.TryParse(datum, out zacatekProjektu);
            NactiVrcholy(br);
            NactiHrany(br);
            NactiZdroje(br);
        }
        /// <summary>
        /// Uloží data do textového souboru.
        /// </summary>
        /// <param name="sw">Datový proud.</param>
        public void UlozDoSouboru(StreamWriter sw)
        {
            sw.WriteLine(nazev);
            sw.WriteLine(typOhodnoceniHran);
            sw.WriteLine(ZacatekProjektu.ToString());
            UlozVrcholy(sw);
            UlozHrany(sw);
            UlozZdroje(sw);
        }
        /// <summary>
        /// Uloží data do binárního souboru.
        /// </summary>
        /// <param name="bw">Datový proud.</param>
        public void UlozDoSouboru(BinaryWriter bw)
        {
            bw.Write(nazev);
            bw.Write(typOhodnoceniHran);
            bw.Write(zacatekProjektu.ToString());
            UlozVrcholy(bw);
            UlozHrany(bw);
            UlozZdroje(bw);
        }
        /// <summary>
        /// Načte vrcholy grafu z textového souboru.
        /// </summary>
        /// <param name="sr">Datový proud.</param>
        public void NactiVrcholy(StreamReader sr)
        {
            vrcholy.Clear();
            try
            {
                pocetVrcholu = vrcholy.Nacti(sr);
            }
            catch (Exception)
            {
                vrcholy.Clear();
                throw;
            }
        }
        /// <summary>
        /// Načte vrcholy grafu z binárního souboru.
        /// </summary>
        /// <param name="br">Datový proud.</param>
        public void NactiVrcholy(BinaryReader br)
        {
            vrcholy.Clear();
            try
            {
                pocetVrcholu = vrcholy.Nacti(br);
            }
            catch (Exception)
            {
                vrcholy.Clear();
                throw;
            }
        }
        /// <summary>
        /// Uloží vrcholy grafu do textového souboru.
        /// </summary>
        /// <param name="sw">Datový proud.</param>
        public void UlozVrcholy(StreamWriter sw)
        {
            vrcholy.Uloz(sw);
        }
        /// <summary>
        /// Uloží vrcholy grafu do binárního souboru.
        /// </summary>
        /// <param name="bw">Datový proud.</param>
        public void UlozVrcholy(BinaryWriter bw)
        {
            vrcholy.Uloz(bw);
        }
        /// <summary>
        /// Načte hrany grafu z textového souboru.
        /// </summary>
        /// <param name="sr">Datový proud.</param>
        public void NactiHrany(StreamReader sr)
        {
            predchudci.Clear();
            naslednici.Clear();
            try
            {
                int pocet = Int32.Parse(sr.ReadLine()); //pocet hran - je to i kontrolni
                for (int i = 0; i < pocet; i++)
                {
                    Hrana hrana = new Hrana();
                    hrana.Nacti(sr);
                    VlozHranu(hrana);
                }
            }
            catch (Exception)
            {
                predchudci.Clear();
                naslednici.Clear();
                throw;
            }
        }
        /// <summary>
        /// Načte hrany grafu z binárního souboru.
        /// </summary>
        /// <param name="br">Datový proud.</param>
        public void NactiHrany(BinaryReader br)
        {
            predchudci.Clear();
            naslednici.Clear();
            try
            {
                int pocet = br.ReadInt32();
                for (int i = 0; i < pocet; i++)
                {
                    Hrana hrana = new Hrana();
                    hrana.Nacti(br);
                    VlozHranu(hrana);
                }
            }
            catch (Exception)
            {
                predchudci.Clear();
                naslednici.Clear();
                throw;
            }
        }
        /// <summary>
        /// Uloží hrany grafu do textového souboru.
        /// </summary>
        /// <param name="sw">Datový proud.</param>
        public void UlozHrany(StreamWriter sw)
        {
            sw.WriteLine(naslednici.Count);
            foreach (Hrana polozka in naslednici)
            {
                polozka.Uloz(sw);
            }

        }
        /// <summary>
        /// Uloží hrany grafu do binárního souboru.
        /// </summary>
        /// <param name="bw">Datový proud.</param>
        public void UlozHrany(BinaryWriter bw)
        {
            bw.Write(naslednici.Count);
            foreach (Hrana polozka in naslednici)
            {
                polozka.Uloz(bw);
            }
        }
        /// <summary>
        /// Načte zdroje grafu z textového souboru.
        /// </summary>
        /// <param name="sr">Datový proud.</param>
        public void NactiZdroje(StreamReader sr)
        {
            zdroje.Clear();
            try
            {
                int pocet = Int32.Parse(sr.ReadLine()); //pocet zdroju - je to i kontrolni
                for (int i = 0; i < pocet; i++)
                {
                    Zdroj zdroj = new Zdroj();
                    zdroj.Nacti(sr);
                    VlozZdroj(zdroj);
                }
            }
            catch (Exception exception)
            {
                zdroje.Clear();
                throw new Exception("Špatný formát souboru", exception);
            }
        }
        /// <summary>
        /// Načte zdroje grafu z binárního souboru.
        /// </summary>
        /// <param name="br">Datový proud.</param>
        public void NactiZdroje(BinaryReader br)
        {
            zdroje.Clear();
            try
            {
                int pocet = br.ReadInt32();
                for (int i = 0; i < pocet; i++)
                {
                    Zdroj zdroj = new Zdroj();
                    zdroj.Nacti(br);
                    VlozZdroj(zdroj);
                }
            }
            catch (Exception)
            {
                zdroje.Clear();
                throw;
            }
        }
        /// <summary>
        /// Uloží zdroje grafu do textového souboru.
        /// </summary>
        /// <param name="sw">Datový proud.</param>
        public void UlozZdroje(StreamWriter sw)
        {
            sw.WriteLine(zdroje.Count);
            foreach (Zdroj polozka in zdroje)
            {
                polozka.Uloz(sw);
            }

        }
        /// <summary>
        /// Uloží zdroje grafu do binárního souboru.
        /// </summary>
        /// <param name="bw">Datový proud.</param>
        public void UlozZdroje(BinaryWriter bw)
        {
            bw.Write(zdroje.Count);
            foreach (Zdroj polozka in zdroje)
            {
                polozka.Uloz(bw);
            }
        }
        #endregion

        #region algoritmy

        /// <summary>
        /// Algoritmus postupného průchodu celým grafem označí hodnoty všech vrcholů.
        /// Testuje správnost zadání síťového grafu. Jediný počáteční a koncový uzel.
        /// Kontrolu těchto uzlů lze zakázat pomocí <param name="kontrola">parametru</param>
        /// typu <c>bool</c>. Dále testuje acykličnost grafu. 
        /// </summary>
        /// <returns>Vrací hodnotu typu <c>double</c>.
        /// 0 - výpočet a kontroly proběhly v pořádku
        /// -1 - více než jeden počáteční uzel
        /// -2 - více než jeden koncový uzel
        /// -3 - není definován počáteční uzel
        /// -4 - není definován koncový uzel
        /// -5 - graf není acyklický
        /// kladné číslo > 0 vrací hodnotu kritické cesty
        /// </returns>
        public double AlgoritmusTopologickehoUsporadani(bool kontrola)
        {
            TopologickeUsporadaniVrchol aktualniVrchol = null;
            bool testKoncovyVrchol = false;
            double hodnotaCPM = 0.0;

            PoleTopologickeUsporadani vrcholyGrafu = new PoleTopologickeUsporadani();
            PoleTopologickeUsporadani mnozinaM = new PoleTopologickeUsporadani();
            foreach (Vrchol polozka in vrcholy)
            {
                //Inicializace základní množiny vrcholů grafu, atributy jsou inicializovány přímo ve třídě TopologickeUsporadani
                TopologickeUsporadaniVrchol topVrchol = new TopologickeUsporadaniVrchol(polozka);
                topVrchol.OhodnoceniC = topVrchol.Vrchol.PocetPredchudcu; //C(Kv) vrcholu;
                topVrchol.Vrchol.MinimalniCas = Double.NegativeInfinity;
                topVrchol.Vrchol.SigmaMin = Double.NegativeInfinity;
                if (kontrola == false && topVrchol.Vrchol.PocetPredchudcu == 0 && topVrchol.Vrchol.PocetNasledniku == 0)
                {
                    //osamocený uzel
                }
                else
                {
                    vrcholyGrafu.Insert(topVrchol);
                    //testovaní počátečního uzlu - síťový graf má pouze jeden počáteční uzel
                    if (topVrchol.Vrchol.PocetPredchudcu == 0)
                    {
                        if (aktualniVrchol != null && kontrola) return -1; //nesplňuje počet počátečních uzlů
                        aktualniVrchol = topVrchol; //vrchol je počátečním vrcholem
                        aktualniVrchol.OhodnoceniC = 0;
                    }
                    //analogie - test koncov0ho uzlu
                    if (topVrchol.Vrchol.PocetNasledniku == 0)
                    {
                        if (testKoncovyVrchol == true && kontrola) return -2; //nesplňuje počet koncových uzlů
                        testKoncovyVrchol = true; //nalezen koncový uzel
                    }
                }
            }
            if (aktualniVrchol == null) return -3; //síťový graf nemá počáteční uzel
            if (testKoncovyVrchol == false) return -4; //síťový graf nemá koncový uzel
            vrcholyGrafu.Sort(new CompareKlicTU(true)); //setřídení podle klíčů vrcholů
            aktualniVrchol.CasoveOhodnoceni = 0;
            aktualniVrchol.Vrchol.MinimalniCas = 0;
            aktualniVrchol.Vrchol.SigmaMin = 0;
            VyberVrcholuTU(vrcholyGrafu, mnozinaM); //vybere všechny vrcholy z vrcholyGrafu s ohodnoceniC == 0
            while (mnozinaM.Count > 0) //algoritmus končí - je dokončen, množina M je prázdná
            {
                aktualniVrchol = (TopologickeUsporadaniVrchol)mnozinaM[0]; //vyberem první vrchol z množiny M
                TopologickeUsporadaniVrchol pomKoncovyVrchol; //koncový vrchol hrany - či-li následník
                Hrana pomHrana;
                int pozice = -1;
                //pro všechny následné hrany vrcholu z množiny M
                for (int i = aktualniVrchol.Vrchol.IndexPrvnihoNaslednika; i < aktualniVrchol.Vrchol.IndexPrvnihoNaslednika + aktualniVrchol.Vrchol.PocetNasledniku; i++)
                {
                    pomHrana = (Hrana)naslednici[i];
                    TopologickeUsporadaniVrchol pomVrchol = new TopologickeUsporadaniVrchol();
                    pomVrchol.Vrchol.Klic.SetKlic(pomHrana.Vrchol2.GetKlic()); //klíč Kv(h)
                    pozice = vrcholyGrafu.Najdi(pomVrchol, 0); //nalezení indexu vrcholu ve vrcholech grafu
                    if (pozice < 0) MessageBox.Show("TopolUspor chyba - 001");
                    pomKoncovyVrchol = (TopologickeUsporadaniVrchol)vrcholyGrafu[pozice];
                    pomKoncovyVrchol.OhodnoceniC -= 1;//C(Kv(h))-1
                    if (aktualniVrchol.Vrchol.MinimalniCas + pomHrana.Data.Ohodnoceni > pomKoncovyVrchol.Vrchol.MinimalniCas)
                    {
                        pomKoncovyVrchol.Vrchol.MinimalniCas = aktualniVrchol.Vrchol.MinimalniCas + pomHrana.Data.Ohodnoceni;
                        pomKoncovyVrchol.Vrchol.SigmaMin = aktualniVrchol.Vrchol.SigmaMin + (Math.Pow((pomHrana.Data.B - pomHrana.Data.A), 2)) / 36;
                        hodnotaCPM = pomKoncovyVrchol.Vrchol.MinimalniCas;
                    }
                }
                pozice = vrcholyGrafu.Najdi(aktualniVrchol, 0);
                if (pozice < 0)
                {
                    MessageBox.Show("TopolUspor chyba - 002\n" + aktualniVrchol.Vrchol.Klic.GetKlic());
                    return -6; //nespecifikovaná chyba
                }
                vrcholyGrafu.RemoveAt(pozice); //vrchol už je označen, nebude brán v potaz
                mnozinaM.RemoveAt(0); //vrchol byl finálně ohodnocen
                VyberVrcholuTU(vrcholyGrafu, mnozinaM);
            }
            if (vrcholyGrafu.Count > 0) return -5;
            return hodnotaCPM;
        }

        public double AlgoritmusTopologickehoUsporadani(bool kontrola, VypocetPostup postup)
        {
            TopologickeUsporadaniVrchol aktualniVrchol = null;
            bool testKoncovyVrchol = false;
            double hodnotaCPM = 0.0;
            postup.Vloz("Vypočítáme nejdříve možné začátky všech činností");
            postup.Vloz("Čas nejdříve možného začátku všech uzlů nastavíme na -nekonečno.");
            PoleTopologickeUsporadani vrcholyGrafu = new PoleTopologickeUsporadani();
            PoleTopologickeUsporadani mnozinaM = new PoleTopologickeUsporadani();
            foreach (Vrchol polozka in vrcholy)
            {
                //Inicializace základní množiny vrcholů grafu, atributy jsou inicializovány přímo ve třídě TopologickeUsporadani
                TopologickeUsporadaniVrchol topVrchol = new TopologickeUsporadaniVrchol(polozka);
                topVrchol.OhodnoceniC = topVrchol.Vrchol.PocetPredchudcu; //C(Kv) vrcholu;
                topVrchol.Vrchol.MinimalniCas = Double.NegativeInfinity;
                topVrchol.Vrchol.SigmaMin = Double.NegativeInfinity;
                if (kontrola == false && topVrchol.Vrchol.PocetPredchudcu == 0 && topVrchol.Vrchol.PocetNasledniku == 0)
                {
                    //osamocený uzel
                }
                else
                {
                    vrcholyGrafu.Insert(topVrchol);
                    //testovaní počátečního uzlu - síťový graf má pouze jeden počáteční uzel
                    if (topVrchol.Vrchol.PocetPredchudcu == 0)
                    {
                        if (aktualniVrchol != null && kontrola) return -1; //nesplňuje počet počátečních uzlů
                        aktualniVrchol = topVrchol; //vrchol je počátečním vrcholem
                        aktualniVrchol.OhodnoceniC = 0;
                    }
                    //analogie - test koncov0ho uzlu
                    if (topVrchol.Vrchol.PocetNasledniku == 0)
                    {
                        if (testKoncovyVrchol == true && kontrola) return -2; //nesplňuje počet koncových uzlů
                        testKoncovyVrchol = true; //nalezen koncový uzel
                    }
                }
            }
            if (aktualniVrchol == null) return -3; //síťový graf nemá počáteční uzel
            if (testKoncovyVrchol == false) return -4; //síťový graf nemá koncový uzel
            vrcholyGrafu.Sort(new CompareKlicTU(true)); //setřídení podle klíčů vrcholů
            aktualniVrchol.CasoveOhodnoceni = 0;
            aktualniVrchol.Vrchol.MinimalniCas = 0;
            aktualniVrchol.Vrchol.SigmaMin = 0;
            postup.Vloz("Počáteční uzel: " + aktualniVrchol.Vrchol.Nazev + " má nejdříve možný začátek: 0.");
            VyberVrcholuTU(vrcholyGrafu, mnozinaM); //vybere všechny vrcholy z vrcholyGrafu s ohodnoceniC == 0
            while (mnozinaM.Count > 0) //algoritmus končí - je dokončen, množina M je prázdná
            {
                aktualniVrchol = (TopologickeUsporadaniVrchol)mnozinaM[0]; //vyberem první vrchol z množiny M
                TopologickeUsporadaniVrchol pomKoncovyVrchol; //koncový vrchol hrany - či-li následník
                Hrana pomHrana;
                int pozice = -1;
                //pro všechny následné hrany vrcholu z množiny M
                for (int i = aktualniVrchol.Vrchol.IndexPrvnihoNaslednika; i < aktualniVrchol.Vrchol.IndexPrvnihoNaslednika + aktualniVrchol.Vrchol.PocetNasledniku; i++)
                {
                    pomHrana = (Hrana)naslednici[i];
                    TopologickeUsporadaniVrchol pomVrchol = new TopologickeUsporadaniVrchol();
                    pomVrchol.Vrchol.Klic.SetKlic(pomHrana.Vrchol2.GetKlic()); //klíč Kv(h)
                    pozice = vrcholyGrafu.Najdi(pomVrchol, 0); //nalezení indexu vrcholu ve vrcholech grafu
                    if (pozice < 0) MessageBox.Show("TopolUspor chyba - 001");
                    pomKoncovyVrchol = (TopologickeUsporadaniVrchol)vrcholyGrafu[pozice];
                    pomKoncovyVrchol.OhodnoceniC -= 1;//C(Kv(h))-1
                    if (aktualniVrchol.Vrchol.MinimalniCas + pomHrana.Data.Ohodnoceni > pomKoncovyVrchol.Vrchol.MinimalniCas)
                    {
                        postup.Vloz("Uzel " + pomKoncovyVrchol.Vrchol.Nazev + " - Původní ohodnocení: " + pomKoncovyVrchol.Vrchol.MinimalniCas + " Nové ohodnocení: " + (aktualniVrchol.Vrchol.MinimalniCas + pomHrana.Data.Ohodnoceni));
                        pomKoncovyVrchol.Vrchol.MinimalniCas = aktualniVrchol.Vrchol.MinimalniCas + pomHrana.Data.Ohodnoceni;
                        pomKoncovyVrchol.Vrchol.SigmaMin = aktualniVrchol.Vrchol.SigmaMin + (Math.Pow((pomHrana.Data.B - pomHrana.Data.A), 2)) / 36;
                        hodnotaCPM = pomKoncovyVrchol.Vrchol.MinimalniCas;
                    }
                }
                pozice = vrcholyGrafu.Najdi(aktualniVrchol, 0);
                if (pozice < 0)
                {
                    MessageBox.Show("TopolUspor chyba - 002\n" + aktualniVrchol.Vrchol.Klic.GetKlic());
                    return -6; //nespecifikovaná chyba
                }
                vrcholyGrafu.RemoveAt(pozice); //vrchol už je označen, nebude brán v potaz
                mnozinaM.RemoveAt(0); //vrchol byl finálně ohodnocen
                VyberVrcholuTU(vrcholyGrafu, mnozinaM);
            }
            if (vrcholyGrafu.Count > 0) return -5;
            return hodnotaCPM;
        }

        public double AlgoritmusTopologickehoUsporadaniMin(bool kontrola)
        {
            TopologickeUsporadaniVrchol aktualniVrchol = null;
            bool testKoncovyVrchol = false;
            double hodnotaCPM = 0.0;

            PoleTopologickeUsporadani vrcholyGrafu = new PoleTopologickeUsporadani();
            PoleTopologickeUsporadani mnozinaM = new PoleTopologickeUsporadani();
            foreach (Vrchol polozka in vrcholy)
            {
                //Inicializace základní množiny vrcholů grafu, atributy jsou inicializovány přímo ve třídě TopologickeUsporadani
                TopologickeUsporadaniVrchol topVrchol = new TopologickeUsporadaniVrchol(polozka);
                topVrchol.OhodnoceniC = topVrchol.Vrchol.PocetPredchudcu; //C(Kv) vrcholu;
                topVrchol.Vrchol.MaximalniCas = Double.PositiveInfinity;
                if (kontrola == false && topVrchol.Vrchol.PocetPredchudcu == 0 && topVrchol.Vrchol.PocetNasledniku == 0)
                {
                    //osamocený uzel
                }
                else
                {
                    vrcholyGrafu.Insert(topVrchol);
                    //testovaní počátečního uzlu - síťový graf má pouze jeden počáteční uzel
                    if (topVrchol.Vrchol.PocetPredchudcu == 0)
                    {
                        if (aktualniVrchol != null && kontrola) return -1; //nesplňuje počet počátečních uzlů
                        aktualniVrchol = topVrchol; //vrchol je počátečním vrcholem
                        aktualniVrchol.OhodnoceniC = 0;
                    }
                    //analogie - test koncov0ho uzlu
                    if (topVrchol.Vrchol.PocetNasledniku == 0)
                    {
                        if (testKoncovyVrchol == true && kontrola) return -2; //nesplňuje počet koncových uzlů
                        testKoncovyVrchol = true; //nalezen koncový uzel
                    }
                }
            }
            if (aktualniVrchol == null) return -3; //síťový graf nemá počáteční uzel
            if (testKoncovyVrchol == false) return -4; //síťový graf nemá koncový uzel
            vrcholyGrafu.Sort(new CompareKlicTU(true)); //setřídení podle klíčů vrcholů
            aktualniVrchol.CasoveOhodnoceni = 0;
            aktualniVrchol.Vrchol.MaximalniCas = 0;
            VyberVrcholuTU(vrcholyGrafu, mnozinaM); //vybere všechny vrcholy z vrcholyGrafu s ohodnoceniC == 0
            while (mnozinaM.Count > 0) //algoritmus končí - je dokončen, množina M je prázdná
            {
                aktualniVrchol = (TopologickeUsporadaniVrchol)mnozinaM[0]; //vyberem první vrchol z množiny M
                TopologickeUsporadaniVrchol pomKoncovyVrchol; //koncový vrchol hrany - či-li následník
                Hrana pomHrana;
                int pozice = -1;
                //pro všechny následné hrany vrcholu z množiny M
                for (int i = aktualniVrchol.Vrchol.IndexPrvnihoNaslednika; i < aktualniVrchol.Vrchol.IndexPrvnihoNaslednika + aktualniVrchol.Vrchol.PocetNasledniku; i++)
                {
                    pomHrana = (Hrana)naslednici[i];
                    TopologickeUsporadaniVrchol pomVrchol = new TopologickeUsporadaniVrchol();
                    pomVrchol.Vrchol.Klic.SetKlic(pomHrana.Vrchol2.GetKlic()); //klíč Kv(h)
                    pozice = vrcholyGrafu.Najdi(pomVrchol, 0); //nalezení indexu vrcholu ve vrcholech grafu
                    if (pozice < 0) MessageBox.Show("TopolUspor chyba - 001");
                    pomKoncovyVrchol = (TopologickeUsporadaniVrchol)vrcholyGrafu[pozice];
                    pomKoncovyVrchol.OhodnoceniC -= 1;//C(Kv(h))-1
                    if (aktualniVrchol.Vrchol.MaximalniCas + pomHrana.Data.Ohodnoceni < pomKoncovyVrchol.Vrchol.MaximalniCas)
                    {
                        pomKoncovyVrchol.Vrchol.MaximalniCas = aktualniVrchol.Vrchol.MaximalniCas + pomHrana.Data.Ohodnoceni;
                        hodnotaCPM = pomKoncovyVrchol.Vrchol.MaximalniCas;
                    }
                }
                pozice = vrcholyGrafu.Najdi(aktualniVrchol, 0);
                if (pozice < 0)
                {
                    MessageBox.Show("TopolUspor chyba - 002\n" + aktualniVrchol.Vrchol.Klic.GetKlic());
                    return -6; //nespecifikovaná chyba
                }
                vrcholyGrafu.RemoveAt(pozice); //vrchol už je označen, nebude brán v potaz
                mnozinaM.RemoveAt(0); //vrchol byl finálně ohodnocen
                VyberVrcholuTU(vrcholyGrafu, mnozinaM);
            }
            if (vrcholyGrafu.Count > 0) return -5;
            return hodnotaCPM;
        }

        public int AlgoritmusTopologickehoUsporadani2(VypocetPostup postup)
        {
            TopologickeUsporadaniVrchol aktualniVrchol = null;
            bool testPocatecniVrchol = false;
            PoleTopologickeUsporadani vrcholyGrafu = new PoleTopologickeUsporadani();
            PoleTopologickeUsporadani mnozinaM = new PoleTopologickeUsporadani();
            foreach (Vrchol polozka in vrcholy)
            {
                //Inicializace základní množiny vrcholů grafu, atributy jsou inicializovány přímo ve třídě TopologickeUsporadani
                TopologickeUsporadaniVrchol topVrchol = new TopologickeUsporadaniVrchol(polozka);
                topVrchol.OhodnoceniC = topVrchol.Vrchol.PocetNasledniku; //C(Kv) vrcholu;
                topVrchol.Vrchol.MaximalniCas = Double.PositiveInfinity;
                topVrchol.Vrchol.SigmaMax = Double.PositiveInfinity;
                vrcholyGrafu.Insert(topVrchol);
                if (topVrchol.Vrchol.PocetNasledniku == 0)
                {
                    if (aktualniVrchol != null) return 1; //nesplňuje počet počátečních uzlů
                    aktualniVrchol = topVrchol; //vrchol je koncovým vrcholem
                    aktualniVrchol.OhodnoceniC = 0;
                    aktualniVrchol.Vrchol.MaximalniCas = aktualniVrchol.Vrchol.MinimalniCas;
                    aktualniVrchol.Vrchol.SigmaMax = 0;
                    hodnotaCPM = aktualniVrchol.Vrchol.MinimalniCas;
                }
                if (topVrchol.Vrchol.PocetPredchudcu == 0)
                {
                    if (testPocatecniVrchol == true) return 2; //nesplňuje počet počátečních uzlů
                    testPocatecniVrchol = true; //nalezen ¨počáteční uzel
                }
            }
            if (aktualniVrchol == null) return 3; //síťový graf nemá počáteční uzel
            if (testPocatecniVrchol == false) return 4; //síťový graf nemá koncový uzel
            vrcholyGrafu.Sort(new CompareKlicTU(true)); //setřídení podle klíčů vrcholů
            aktualniVrchol.CasoveOhodnoceni = 0;

            postup.Vloz("Čas nejpozději možného ukončení projektu: " + hodnotaCPM);
            postup.Vloz("Vypočítáme nejpozději možné ukončení všech činností");
            postup.Vloz("Koncový uzel: " + aktualniVrchol.Vrchol.Nazev + " má nejpozději možné ukončení: " + hodnotaCPM);
            VyberVrcholuTU(vrcholyGrafu, mnozinaM);
            while (mnozinaM.Count > 0) //algoritmus končí - je dokončen, množina M je prázdná
            {
                aktualniVrchol = (TopologickeUsporadaniVrchol)mnozinaM[0]; //vyberem první vrchol z množiny M
                TopologickeUsporadaniVrchol pomPocatecniVrchol; //koncový vrchol hrany - či-li následník
                Hrana pomHrana;
                int pozice = -1;
                //pro všechny vstupní hrany do vrcholu z množiny M
                for (int i = aktualniVrchol.Vrchol.IndexPrvnihoPredchudce; i < aktualniVrchol.Vrchol.IndexPrvnihoPredchudce + aktualniVrchol.Vrchol.PocetPredchudcu; i++)
                {
                    pomHrana = (Hrana)predchudci[i];
                    TopologickeUsporadaniVrchol pomVrchol = new TopologickeUsporadaniVrchol();
                    pomVrchol.Vrchol.Klic.SetKlic(pomHrana.Vrchol1.GetKlic()); //klíč Pv(h)
                    pozice = vrcholyGrafu.Najdi(pomVrchol, 0); //nalezení indexu vrcholu ve vrcholech grafu
                    if (pozice < 0) MessageBox.Show("TU2 chyba - 001");
                    pomPocatecniVrchol = (TopologickeUsporadaniVrchol)vrcholyGrafu[pozice];
                    pomPocatecniVrchol.OhodnoceniC -= 1;//C(Pv(h))-1
                    if (aktualniVrchol.Vrchol.MaximalniCas - pomHrana.Data.Ohodnoceni < pomPocatecniVrchol.Vrchol.MaximalniCas)
                    {
                        postup.Vloz("Uzel " + pomPocatecniVrchol.Vrchol.Nazev + " - Původní ohodnocení: " + pomPocatecniVrchol.Vrchol.MaximalniCas + " Nové ohodnocení: " + (aktualniVrchol.Vrchol.MaximalniCas - pomHrana.Data.Ohodnoceni));
                        pomPocatecniVrchol.Vrchol.MaximalniCas = aktualniVrchol.Vrchol.MaximalniCas - pomHrana.Data.Ohodnoceni;
                        pomPocatecniVrchol.Vrchol.SigmaMax = aktualniVrchol.Vrchol.SigmaMax + (Math.Pow((pomHrana.Data.B - pomHrana.Data.A), 2)) / 36;
                    }
                }
                pozice = vrcholyGrafu.Najdi(aktualniVrchol, 0);
                if (pozice < 0)
                {
                    MessageBox.Show("TU2 chyba - 002\n" + aktualniVrchol.Vrchol.Klic.GetKlic());
                    return 6; //nespecifikovaná chyba
                }
                vrcholyGrafu.RemoveAt(pozice); //vrchol už je označen, nebude brán v potaz
                mnozinaM.RemoveAt(0); //vrchol byl finálně ohodnocen
                VyberVrcholuTU(vrcholyGrafu, mnozinaM);
            }
            //vytvoreni kriticke cesty
            VypocetRezerv();
            kritickaCesta.Clear();
            foreach (Hrana polozka in naslednici)
            {
                Vrchol vrchol1 = new Vrchol();
                Vrchol vrchol2 = new Vrchol();
                vrchol1.Klic.SetKlic(polozka.Vrchol1.GetKlic());
                vrchol2.Klic.SetKlic(polozka.Vrchol2.GetKlic());
                vrchol1 = (Vrchol)(vrcholy[vrcholy.Najdi(vrchol1, 0)]);
                vrchol2 = (Vrchol)(vrcholy[vrcholy.Najdi(vrchol2, 0)]);
                //if (vrchol1.MaximalniCas == vrchol1.MinimalniCas && vrchol2.MinimalniCas == vrchol2.MaximalniCas && vrchol1.MaximalniCas == vrchol2.MaximalniCas - polozka.Data.Ohodnoceni)
                if (polozka.Data.RC == 0) kritickaCesta.Add(polozka);
            }


            if (vrcholyGrafu.Count > 0) return 5;
            return 0;
        }

        public int AlgoritmusTopologickehoUsporadani2()
        {
            TopologickeUsporadaniVrchol aktualniVrchol = null;
            bool testPocatecniVrchol = false;
            PoleTopologickeUsporadani vrcholyGrafu = new PoleTopologickeUsporadani();
            PoleTopologickeUsporadani mnozinaM = new PoleTopologickeUsporadani();
            foreach (Vrchol polozka in vrcholy)
            {
                //Inicializace základní množiny vrcholů grafu, atributy jsou inicializovány přímo ve třídě TopologickeUsporadani
                TopologickeUsporadaniVrchol topVrchol = new TopologickeUsporadaniVrchol(polozka);
                topVrchol.OhodnoceniC = topVrchol.Vrchol.PocetNasledniku; //C(Kv) vrcholu;
                topVrchol.Vrchol.MaximalniCas = Double.PositiveInfinity;
                topVrchol.Vrchol.SigmaMax = Double.PositiveInfinity;
                vrcholyGrafu.Insert(topVrchol);
                if (topVrchol.Vrchol.PocetNasledniku == 0)
                {
                    if (aktualniVrchol != null) return 1; //nesplňuje počet počátečních uzlů
                    aktualniVrchol = topVrchol; //vrchol je koncovým vrcholem
                    aktualniVrchol.OhodnoceniC = 0;
                    aktualniVrchol.Vrchol.MaximalniCas = aktualniVrchol.Vrchol.MinimalniCas;
                    aktualniVrchol.Vrchol.SigmaMax = 0;
                    hodnotaCPM = aktualniVrchol.Vrchol.MinimalniCas;
                }
                if (topVrchol.Vrchol.PocetPredchudcu == 0)
                {
                    if (testPocatecniVrchol == true) return 2; //nesplňuje počet počátečních uzlů
                    testPocatecniVrchol = true; //nalezen ¨počáteční uzel
                }
            }
            if (aktualniVrchol == null) return 3; //síťový graf nemá počáteční uzel
            if (testPocatecniVrchol == false) return 4; //síťový graf nemá koncový uzel
            vrcholyGrafu.Sort(new CompareKlicTU(true)); //setřídení podle klíčů vrcholů
            aktualniVrchol.CasoveOhodnoceni = 0;

            VyberVrcholuTU(vrcholyGrafu, mnozinaM);
            while (mnozinaM.Count > 0) //algoritmus končí - je dokončen, množina M je prázdná
            {
                aktualniVrchol = (TopologickeUsporadaniVrchol)mnozinaM[0]; //vyberem první vrchol z množiny M
                TopologickeUsporadaniVrchol pomPocatecniVrchol; //koncový vrchol hrany - či-li následník
                Hrana pomHrana;
                int pozice = -1;
                //pro všechny vstupní hrany do vrcholu z množiny M
                for (int i = aktualniVrchol.Vrchol.IndexPrvnihoPredchudce; i < aktualniVrchol.Vrchol.IndexPrvnihoPredchudce + aktualniVrchol.Vrchol.PocetPredchudcu; i++)
                {
                    pomHrana = (Hrana)predchudci[i];
                    TopologickeUsporadaniVrchol pomVrchol = new TopologickeUsporadaniVrchol();
                    pomVrchol.Vrchol.Klic.SetKlic(pomHrana.Vrchol1.GetKlic()); //klíč Pv(h)
                    pozice = vrcholyGrafu.Najdi(pomVrchol, 0); //nalezení indexu vrcholu ve vrcholech grafu
                    if (pozice < 0) MessageBox.Show("TU2 chyba - 001");
                    pomPocatecniVrchol = (TopologickeUsporadaniVrchol)vrcholyGrafu[pozice];
                    pomPocatecniVrchol.OhodnoceniC -= 1;//C(Pv(h))-1
                    if (aktualniVrchol.Vrchol.MaximalniCas - pomHrana.Data.Ohodnoceni < pomPocatecniVrchol.Vrchol.MaximalniCas)
                    {
                        pomPocatecniVrchol.Vrchol.MaximalniCas = aktualniVrchol.Vrchol.MaximalniCas - pomHrana.Data.Ohodnoceni;
                        pomPocatecniVrchol.Vrchol.SigmaMax = aktualniVrchol.Vrchol.SigmaMax + (Math.Pow((pomHrana.Data.B - pomHrana.Data.A), 2)) / 36;
                    }
                }
                pozice = vrcholyGrafu.Najdi(aktualniVrchol, 0);
                if (pozice < 0)
                {
                    MessageBox.Show("TU2 chyba - 002\n" + aktualniVrchol.Vrchol.Klic.GetKlic());
                    return 6; //nespecifikovaná chyba
                }
                vrcholyGrafu.RemoveAt(pozice); //vrchol už je označen, nebude brán v potaz
                mnozinaM.RemoveAt(0); //vrchol byl finálně ohodnocen
                VyberVrcholuTU(vrcholyGrafu, mnozinaM);
            }
            //vytvoreni kriticke cesty
            VypocetRezerv();
            kritickaCesta.Clear();
            foreach (Hrana polozka in naslednici)
            {
                Vrchol vrchol1 = new Vrchol();
                Vrchol vrchol2 = new Vrchol();
                vrchol1.Klic.SetKlic(polozka.Vrchol1.GetKlic());
                vrchol2.Klic.SetKlic(polozka.Vrchol2.GetKlic());
                vrchol1 = (Vrchol)(vrcholy[vrcholy.Najdi(vrchol1, 0)]);
                vrchol2 = (Vrchol)(vrcholy[vrcholy.Najdi(vrchol2, 0)]);
                //if (vrchol1.MaximalniCas == vrchol1.MinimalniCas && vrchol2.MinimalniCas == vrchol2.MaximalniCas && vrchol1.MaximalniCas == vrchol2.MaximalniCas - polozka.Data.Ohodnoceni)
                if (polozka.Data.RC == 0) kritickaCesta.Add(polozka);
            }


            if (vrcholyGrafu.Count > 0) return 5;
            return 0;
        }
        /// <summary>
        /// Vybere uzel pro algoritmus topologického uspořádání.
        /// </summary>
        /// <param name="vrcholyGrafu">Sezanm vrcholů v posloupnosti.</param>
        /// <param name="mnozinaM">Polsoupnost ohodnocených uzlů.</param>
        /// <returns></returns>
        private int VyberVrcholuTU(PoleTopologickeUsporadani vrcholyGrafu, PoleTopologickeUsporadani mnozinaM)
        {
            foreach (TopologickeUsporadaniVrchol polozka in vrcholyGrafu)
            {
                if (polozka.OhodnoceniC == 0)
                {
                    if (mnozinaM.Najdi(polozka, 0) < 0) mnozinaM.Add(polozka);
                }
            }
            return 0;
        }
        /// <summary>
        /// Vypočítá rezervy činností grafu.
        /// Celkovou, volnou a nezávislou rezervu.
        /// </summary>
        public void VypocetRezerv()
        {
            foreach (Hrana polozka in naslednici)
            {
                Vrchol vrchol1 = new Vrchol(polozka.Vrchol1.GetKlic());
                Vrchol vrchol2 = new Vrchol(polozka.Vrchol2.GetKlic());
                vrchol1 = (Vrchol)(vrcholy[vrcholy.Najdi(vrchol1, 0)]);
                vrchol2 = (Vrchol)(vrcholy[vrcholy.Najdi(vrchol2, 0)]);
                polozka.Data.RC = vrchol2.MaximalniCas - vrchol1.MinimalniCas - polozka.Data.Ohodnoceni;
                polozka.Data.RN = Math.Max(0, vrchol2.MinimalniCas - vrchol1.MaximalniCas - polozka.Data.Ohodnoceni);
                polozka.Data.RV = vrchol2.MinimalniCas - vrchol1.MinimalniCas - polozka.Data.Ohodnoceni;
                string cislo = String.Format("{0:F2}", polozka.Data.RC);
                polozka.Data.RC = double.Parse(cislo);
            }
        }
        /// <summary>
        /// Vypočítá rezervy činností grafu.
        /// Celkovou, volnou a nezávislou rezervu včetně postupu.
        /// </summary>
        /// <param name="postup">Výpis postupu.</param>
        /// <param name="nastaveni">Nastavení aplikace.</param>        
        public void VypocetRezerv(VypocetPostup postup, NastaveniVzhled nastaveni)
        {
            postup.Vloz(" ");
            postup.Vloz("Výpočet rezerv činností");
            foreach (Hrana polozka in naslednici)
            {
                Vrchol vrchol1 = new Vrchol(polozka.Vrchol1.GetKlic());
                Vrchol vrchol2 = new Vrchol(polozka.Vrchol2.GetKlic());
                vrchol1 = (Vrchol)(vrcholy[vrcholy.Najdi(vrchol1, 0)]);
                vrchol2 = (Vrchol)(vrcholy[vrcholy.Najdi(vrchol2, 0)]);
                Hrana pomHrana = new Hrana(vrchol1.Klic, vrchol2.Klic);
                pomHrana = (Hrana)(naslednici[naslednici.Najdi(pomHrana, 0)]);
                polozka.Data.RC = vrchol2.MaximalniCas - vrchol1.MinimalniCas - polozka.Data.Ohodnoceni;
                polozka.Data.RN = Math.Max(0, vrchol2.MinimalniCas - vrchol1.MaximalniCas - polozka.Data.Ohodnoceni);
                polozka.Data.RV = vrchol2.MinimalniCas - vrchol1.MinimalniCas - polozka.Data.Ohodnoceni;
                if (nastaveni.PopisCinnosti == 1) postup.Vloz("Činnost: " + pomHrana.Data.Zkratka);
                else postup.Vloz("Činnost: " + pomHrana.Data.Nazev);                
                string cislo = String.Format("{0:F2}", polozka.Data.RC);
                polozka.Data.RC = double.Parse(cislo);
                cislo = String.Format("{0:F2}", polozka.Data.RN);
                polozka.Data.RN = double.Parse(cislo);
                cislo = String.Format("{0:F2}", polozka.Data.RV);
                polozka.Data.RV = double.Parse(cislo);
                postup.Vloz("Celková rezerva = " + String.Format("{0:F2}", vrchol2.MaximalniCas) + " - " + String.Format("{0:F2}", vrchol1.MinimalniCas) + " - " + String.Format("{0:F2}", polozka.Data.Ohodnoceni) + " = " + polozka.Data.RC);
                postup.Vloz("Volná rezerva = " + String.Format("{0:F2}", vrchol2.MinimalniCas) + " - " + String.Format("{0:F2}", vrchol1.MinimalniCas) + " - " + String.Format("{0:F2}", polozka.Data.Ohodnoceni) + " = " + polozka.Data.RV);
                postup.Vloz("Nezávislá rezerva = " + String.Format("{0:F2}",vrchol2.MinimalniCas) + " - " + String.Format("{0:F2}", vrchol1.MaximalniCas) + " - " + String.Format("{0:F2}", polozka.Data.Ohodnoceni) + " = " + polozka.Data.RN);             
            }
        }
        /// <summary>
        /// Vytvoří podgraf podproblému. Z jednoho podproblému <paramref name="Graf"/> uzlu s indexem
        /// <paramref name="int"/> určí a vrátí pole vrcholů, ze kterých mohou být počítany kombinace
        /// činností, které mohou běžet současně.
        /// </summary>
        /// <param name="indexUzlu"></param>
        /// <param name="prastromReseni"></param>
        /// <param name="hranyPodproblemu"></param>
        /// <param name="vypisListBox"></param>
        /// <returns>Vrací typ <c>ADTVrchol</c>. Pole vrcholů bez předchozích dokončených činností a zároveň na ně navazují
        /// nedokončené činosti.</returns>
        private ADTVrchol[] VytvorGrafPodproblemu(int indexUzlu, Graf prastromReseni, SeznamHran hranyPodproblemu, VypocetPostup postup, bool vyukovyMod)
        {
            Vrchol pomUzel = (Vrchol)(prastromReseni.vrcholy[indexUzlu]); //najdu uzel v prastromu
            Vrchol aktualniUzel = new Vrchol();
            pomUzel.Kopie(aktualniUzel);
            Hrana aktualniHrana = null;
            //nastaveni hran - vsechny nezpracovane
            foreach (Hrana polozka in naslednici)
            {
                polozka.Data.Dokonceno = 0;
                polozka.Data.Vykonano = 0;
            }
            //pricitame dokonceni hran
            if (vyukovyMod == true && prastromReseni.vrcholy.Count > 1)
            {
                postup.Vloz("Aktualizujeme činnosti, které předcházeli uzlu " + aktualniUzel.Nazev);
                postup.Vloz("Byla-li činnost již dokončena, nebereme ji v daném podproblému v potaz. Jinak aktualizujeme její vykonaný čas.");
            }
            if (aktualniUzel.PocetPredchudcu > 0)
                do
                {
                    //vrchol ve stromu ma pouze jedineho predchudce
                    aktualniHrana = (Hrana)(prastromReseni.predchudci[aktualniUzel.IndexPrvnihoPredchudce]);
                    //nastaveni atributu podle jiz vykonanych cinnosti
                    for (int i = 0; i < aktualniHrana.Data.PoleHran.Length; i++)
                    {
                        Hrana pomHrana = new Hrana(aktualniHrana.Data.PoleHran[i].Vrchol1, aktualniHrana.Data.PoleHran[i].Vrchol2);
                        int indexHrany = naslednici.Najdi(pomHrana, 0);
                        if (indexHrany >= 0)
                        {
                            pomHrana = (Hrana)(naslednici[indexHrany]);
                            //hrana dokoncila casovou hodnotu hrany prastromu
                            pomHrana.Data.Dokonceno += aktualniHrana.Data.Vykonano;
                            pomHrana.Data.Vykonano = pomHrana.Data.Dokonceno;
                            //pokud je jiz hrana dokoncena, pak ji vlozim do hran podproblemu, ktere ted nebude resit
                            if (vyukovyMod == true && prastromReseni.vrcholy.Count > 1)
                            {
                                postup.Vloz("Aktualizujeme činnost " + pomHrana.Data.Nazev + ". Činnost má dokončeno " + pomHrana.Data.Dokonceno + " z " + pomHrana.Data.Ohodnoceni);
                            }
                            if (pomHrana.Data.Dokonceno >= pomHrana.Data.Ohodnoceni)
                            {
                                if (vyukovyMod == true && prastromReseni.vrcholy.Count > 1)
                                {
                                    postup.Vloz("Činnost " + pomHrana.Data.Nazev + " odebereme z podproblému. Je již dokončena.");
                                }
                                if (hranyPodproblemu.Najdi(pomHrana, 0) < 0)
                                {
                                    hranyPodproblemu.Add(pomHrana);
                                    pomHrana.Data.Dokonceno = pomHrana.Data.Ohodnoceni;
                                }
                                //pomHrana.Data.Vykonano = pomHrana.Data.Dokonceno; //hrana byla cela vykonana
                            }
                        }
                    }
                    //posun na predchozi uzel
                    aktualniUzel.Klic.SetKlic(aktualniHrana.Vrchol1.GetKlic());
                    indexUzlu = prastromReseni.vrcholy.Najdi(aktualniUzel, 0);
                    pomUzel = (Vrchol)(prastromReseni.vrcholy[indexUzlu]);
                    pomUzel.Kopie(aktualniUzel);
                } while (aktualniUzel.PocetPredchudcu > 0);

            int pocet = 0;
            //odebrani hran, ktere jsou vykonany cele
            foreach (Hrana polozka in hranyPodproblemu)
            {
                OdeberHranu(polozka);
            }
            foreach (Hrana polozka in naslednici)
            {
                if (polozka.Data.Typ == 2)
                {
                    Vrchol fikUzel = new Vrchol(polozka.Vrchol1.GetKlic());
                    fikUzel = (Vrchol)(vrcholy[vrcholy.Najdi(fikUzel, 0)]);
                    if (fikUzel.PocetPredchudcu == 0)
                    {
                        hranyPodproblemu.Add(polozka);
                    }
                }
            }
            foreach (Hrana polozka in hranyPodproblemu)
            {
                OdeberHranu(polozka);
            }

            //zjisteni velikosti pole, kde budou vrcholy, ktere maji vsechny cinnosti dokoncene, ale take zaroven z nich nejake vystupuji
            foreach (Vrchol polozka in vrcholy)
            {
                if (polozka.PocetPredchudcu == 0 && polozka.PocetNasledniku > 0)
                {
                    pocet++;
                }
            }
            ADTVrchol[] poleVrcholu = new ADTVrchol[pocet];
            pocet = 0;
            //ulozeni uzlu do pole
            foreach (Vrchol polozka in vrcholy)
            {
                if (polozka.PocetPredchudcu == 0 && polozka.PocetNasledniku > 0)
                {
                    poleVrcholu[pocet] = polozka;
                    pocet++;
                }
            }
            return poleVrcholu;
        }

        /// <summary>
        /// Je metoda obsahující algoritmus pro rozmístění kapacitně omezených zdrojů v síťovém grafu.
        /// </summary>
        /// <param name="param">Parametry projektu.</param>
        public void AlgoritmusBaB(object param)
        {
            minimumProjektu = double.MaxValue;
            neprozkoumanaReseni.Clear();
            neprozkoumanaReseni.Sort(new CompareOhodnoceni(true));
            lock (this)
            {
                ParametryBaB parametry = (ParametryBaB)param;
                Graf prastromReseni = parametry.prastromReseni;
                prastromReseni.vrcholy.Sort(new CompareKlic(true));
                VypocetPostup postup = parametry.vypocetPostup;
                Panel progBar = parametry.projektProgressBar;
                Vrchol aktualniUzel = null;
                foreach (Hrana cinnost in naslednici)
                {
                    foreach (ZdrojHrany zdrojH in cinnost.Data.zdrojeHrany)
                    {
                        Zdroj zdrojProjektu = new Zdroj();
                        zdrojProjektu.Nazev = zdrojH.Nazev;
                        zdrojProjektu = (Zdroj)(zdroje[zdroje.Najdi(zdrojProjektu, 0)]);
                        if (zdrojProjektu.AktualniKapacita < zdrojH.Spotreba && zdrojProjektu.Obnovitelnost == true)
                        {
                            parametry.chybaAlgoritmu = 1;
                            parametry.vypocetPostup.Vloz("Nedostatek zdrojů pro vykonání " + cinnost.Data.Nazev);
                            return;
                        }
                    }
                }
                //vytvoření prvního vrcholu prastromu řešení
                foreach (Vrchol polozka in vrcholy)
                {
                    if (polozka.PocetPredchudcu == 0)
                    {
                        Vrchol vkladanyUzel = new Vrchol();
                        vkladanyUzel.Klic.SetKlic("V" + poradiVrcholuPrastromu.ToString("D4"));
                        vkladanyUzel.Nazev = vkladanyUzel.Klic.GetKlic();
                        vkladanyUzel.MinimalniCas = polozka.MinimalniCas;
                        vkladanyUzel.MaximalniCas = hodnotaCPM;
                        vkladanyUzel.SouradniceX = 70;
                        vkladanyUzel.SouradniceY = 70;
                        prastromReseni.VlozVrchol(vkladanyUzel);
                        poradiVrcholuPrastromu += 1;
                        aktualniUzel = vkladanyUzel;
                    }
                }
                neprozkoumanaReseni.Insert(aktualniUzel);
                if (parametry.nastaveni.VyukovyMod == true)
                {
                    postup.Vloz("Vložíme počáteční uzel jako kořen prastromu a označíme jej " + aktualniUzel.Nazev + ".");
                    postup.Vloz("Pro tento uzel uběhnutý čas od začátku projektu a hodnotu kritické cesty bez kapacitního omezení.");
                    postup.Vloz("Vrchol: " + aktualniUzel.Nazev + " Uběhnutý čas: " + aktualniUzel.MinimalniCas + " CPM: " + aktualniUzel.MaximalniCas);
                }

                //VypocitejSouradnice(prastromReseni, parametry);

                SeznamHran hranyPodproblemu = new SeznamHran();
                //ziskat index nejnizsiho casu
                int index = ziskatIndexListuNejnizsihoCasu(prastromReseni, parametry.nastaveni); //melo by být 0, páč přechozí vrchol je zatím jediným vrcholem prastromu řešení
                int index2 = index;
                while (index2 > -1)
                {
                    parametry.progress++;
                    if (parametry.progress >= 200) parametry.progress = 0;
                    progBar.Invalidate();

                    //index vrcholu v prastromu řešení, který bude následně zpracován
                    //index = ziskatIndexListuNejnizsihoCasu(prastromReseni);

                    aktualniUzel = (Vrchol)(neprozkoumanaReseni[index2]);

                    index = prastromReseni.vrcholy.Najdi(aktualniUzel,0);
                    if (index < 0) MessageBox.Show("Nenalezen uzel");
                    aktualniUzel = (Vrchol)prastromReseni.vrcholy[index];
                    neprozkoumanaReseni.RemoveAt(index2);
                    
                    if (prastromReseni.vrcholy.Count > 1 && parametry.nastaveni.VyukovyMod == true)
                    {
                        postup.Vloz("Vybereme uzel, který je listem a má nejmenší ohodnocení odhadu na dokončení projektu a zároveň tento uzel neurčuje konec projektu.");
                        postup.Vloz("Vybereme uzel " + aktualniUzel.Nazev + " s odhadem " + aktualniUzel.MaximalniCas.ToString());
                    }
                    hranyPodproblemu.Clear();
                    ADTVrchol[] poleVrcholu = VytvorGrafPodproblemu(index, prastromReseni, hranyPodproblemu, postup, parametry.nastaveni.VyukovyMod);
                    int delkaPole = 0;
                    //pocet vsech vychazejich hran z uzlu, ktere maji zpracovane predchudce
                    for (int i = 0; i < poleVrcholu.Length; i++)
                    {
                        delkaPole += poleVrcholu[i].PocetNasledniku;
                    }
                    ADTHrana[] poleHran = new ADTHrana[delkaPole];
                    delkaPole = 0;
                    for (int j = 0; j < poleVrcholu.Length; j++)
                    {
                        Vrchol zkoumanyVrchol = new Vrchol();
                        zkoumanyVrchol.Klic.SetKlic(((ADTVrchol)poleVrcholu[j]).Klic.GetKlic());
                        zkoumanyVrchol = (Vrchol)(vrcholy[vrcholy.Najdi(zkoumanyVrchol, 0)]);
                        for (int i = zkoumanyVrchol.IndexPrvnihoNaslednika; i < zkoumanyVrchol.IndexPrvnihoNaslednika + zkoumanyVrchol.PocetNasledniku; i++)
                        {
                            Hrana hrana = new Hrana();
                            hrana.Vrchol1.SetKlic(zkoumanyVrchol.Klic.GetKlic());
                            hrana.Vrchol2.SetKlic(((Hrana)(naslednici[i])).Vrchol2.GetKlic());
                            poleHran[delkaPole] = hrana;
                            delkaPole++;
                        }
                    }
                    //VypisListBox1(vypisListBox, poleHran);
                    //vytvoříme kombinace hran daného podproblému
                    ADTHrana[][] kombinaceHran = null;
                    //vytvoří všechny možné kombinace bez ohledu na zdroje
                    kombinaceHran = Kombinace.ziskejVsechnyKombinace(poleHran);
                    if (parametry.nastaveni.VyukovyMod == true)
                    {
                        postup.Vloz(kombinaceHran.Length.ToString() + " kombinace - možné: ");
                        VypisMoznosti(postup, kombinaceHran, parametry.nastaveni);
                    }
                    //vyrazeni nemoznych reseni s ohledem na zdroje
                    int pocetMoznosti = 0;
                    for (int i = 0; i < kombinaceHran.GetLength(0); i++)
                    {
                        SeznamZdroju zdrojeSoubeznychCinnosti = new SeznamZdroju();
                        double kapacitaZdroje = 0;
                        KopirujSeznam(zdrojeSoubeznychCinnosti);
                        bool kombinaceRealna = true;
                        for (int j = 0; j < kombinaceHran[i].GetLength(0); j++)
                        {
                            Hrana zkoumanaHrana = new Hrana(kombinaceHran[i][j].Vrchol1, kombinaceHran[i][j].Vrchol2);
                            zkoumanaHrana = ((Hrana)naslednici[naslednici.Najdi(zkoumanaHrana, 0)]);


                            if (zkoumanaHrana.Data.zdrojeHrany.Count > 0)
                            {
                                foreach (ZdrojHrany polozka in zkoumanaHrana.Data.zdrojeHrany)
                                {
                                    Zdroj pomZdroj = new Zdroj();
                                    pomZdroj.Nazev = polozka.Nazev;
                                    int pozice = zdrojeSoubeznychCinnosti.Najdi(pomZdroj, 0);
                                    pomZdroj = (Zdroj)(zdrojeSoubeznychCinnosti[zdrojeSoubeznychCinnosti.Najdi(pomZdroj, 0)]);
                                    kapacitaZdroje = pomZdroj.AktualniKapacita;
                                    if (pomZdroj.Obnovitelnost == true && kapacitaZdroje < polozka.Spotreba) //zdroje nestaci
                                    {
                                        kombinaceRealna = false;
                                        break;
                                    }
                                    else ((Zdroj)zdrojeSoubeznychCinnosti[pozice]).AktualniKapacita -= polozka.Spotreba;
                                }
                            }
                            if (kombinaceRealna == false) break;
                        }
                        if (kombinaceRealna == false) kombinaceHran[i] = null;
                        else pocetMoznosti++;
                    }
                    if (parametry.nastaveni.VyukovyMod == true)
                    {
                        postup.Vloz(pocetMoznosti + " kombinace, pro které je dostatek zdrojů: ");
                        VypisMoznosti(postup, kombinaceHran, parametry.nastaveni);
                    }
                    //odstranění nejméně možných využití kombinací
                    int pocetPoRedukci = 0;
                    if (parametry.vsechnaReseni == false)
                    {
                        int maximum = 0;
                        for (int i = 0; i < kombinaceHran.GetLength(0); i++)
                        {
                            if (kombinaceHran[i] != null)
                                if (kombinaceHran[i].GetLength(0) > maximum)
                                    maximum = kombinaceHran[i].GetLength(0);
                        }
                        for (int i = 0; i < kombinaceHran.GetLength(0); i++)
                        {
                            if (kombinaceHran[i] != null)
                                if (kombinaceHran[i].GetLength(0) < maximum)
                                    kombinaceHran[i] = null;
                                else pocetPoRedukci++;
                        }
                        if (parametry.nastaveni.VyukovyMod == true)
                        {
                            postup.Vloz(pocetPoRedukci.ToString() + " kombinací po redukci řešení: ");
                            VypisMoznosti(postup, kombinaceHran, parametry.nastaveni);
                        }
                    }

                    //parametry.dbPanel.Invalidate();
                    /*using (Mini dialog = new Mini())
                    {
                        if (dialog.ShowDialog() == DialogResult.OK)
                        {
                            
                        }
                    }*/
                    //888888888888888888888888888888888888888
                    int konec = VytvorNovaSubreseni(prastromReseni, aktualniUzel, kombinaceHran, hranyPodproblemu, postup, parametry.nastaveni, parametry);
                    if (parametry.nastaveni.VyukovyMod == true) postup.Vloz(" ");
                    index2 = ziskatIndexListuNejnizsihoCasu(prastromReseni,parametry.nastaveni);
                    if (konec == -1) index = -2;
                }
                if (aktualniUzel.PocetPredchudcu > 0)
                {
                    Hrana praHrana = (Hrana)(prastromReseni.predchudci[aktualniUzel.IndexPrvnihoPredchudce]);
                    for (int q = 0; q < praHrana.Data.poleHran.Length; q++)
                    {
                        Hrana pomHrana = new Hrana(praHrana.Data.PoleHran[q].Vrchol1, praHrana.Data.PoleHran[q].Vrchol2);
                        pomHrana = (Hrana)(naslednici[naslednici.Najdi(pomHrana, 0)]);
                        if (pomHrana.Data.zdrojeHrany.Count > 0)
                        {
                            foreach (ZdrojHrany zdrojPolozka in pomHrana.Data.zdrojeHrany)
                            {
                                Zdroj pomZdroj = new Zdroj();
                                pomZdroj.Nazev = zdrojPolozka.Nazev;
                                int pozice = zdroje.Najdi(pomZdroj, 0);
                                pomZdroj = (Zdroj)(zdroje[zdroje.Najdi(pomZdroj, 0)]);
                                if (pomZdroj.Obnovitelnost == false)
                                {
                                    pomZdroj.AktualniKapacita += zdrojPolozka.Spotreba;
                                }
                            }
                        }
                    }
                }
                //MessageBox.Show(smazuTentoCitac.ToString());
                VypocitejSouradnice(prastromReseni, parametry);
                parametry.progress = 199;
                //return 0;
            }
        }
        /// <summary>
        /// Vytvoří nové vrcholy prastromu z kombinace hran. <paramref name="kombinaceHran"/>.
        /// </summary>
        /// <param name="prastromReseni">Aktuální prastrom řešení.</param>
        /// <param name="aktualniUzel">Výchozí uzel.</param>
        /// <param name="kombinaceHran">Kombinace činností.</param>
        /// <param name="vypisListBox">Odkaz na výpis algoritmu.</param>
        /// <returns></returns>
        private int VytvorNovaSubreseni(Graf prastromReseni, Vrchol aktualniUzel, ADTHrana[][] kombinaceHran, SeznamHran hranyPodproblemu, VypocetPostup postup, NastaveniVzhled nastaveni, ParametryBaB parametry)
        {
            foreach (Hrana polozka in hranyPodproblemu)
            {
                VlozHranu(polozka);
            }
            double minimum = 0;
            double hodnotaCPM;
            string retezec, retezec2;
            //projdu všechny možné kombinace
            for (int i = 0; i < kombinaceHran.GetLength(0); i++)
            {
                foreach (Hrana polozka in naslednici)
                {
                    //polozka.Data.Ohodnoceni -= polozka.Data.Vykonano;
                    polozka.Data.Dokonceno = 0.0;
                    polozka.Data.DocasneOhodnoceni = polozka.Data.Ohodnoceni;
                    polozka.Data.Ohodnoceni -= polozka.Data.Vykonano;
                    if (polozka.Data.Ohodnoceni < 0) polozka.Data.Ohodnoceni=0; //nevim, co s tim doublem je, ale zaporne cislo to obcas hodi, ale vetsinou je to tesne 10^-50 treba
                }
                if (kombinaceHran[i] != null) //zamítnuté kombinace přeskakuji
                {
                    //výpočet jedné kombinace - start
                    minimum = double.MaxValue;
                    hodnotaCPM = 0;
                    retezec = "";
                    retezec2 = "";
                    //najdu minimální ohodnocení z činností kombinace
                    SeznamZdrojeHrany zdrojeKombinace = new SeznamZdrojeHrany();
                    for (int j = 0; j < kombinaceHran[i].GetLength(0); j++)
                    {
                        Hrana zkoumanaHrana = new Hrana(kombinaceHran[i][j].Vrchol1, kombinaceHran[i][j].Vrchol2);
                        zkoumanaHrana = ((Hrana)naslednici[naslednici.Najdi(zkoumanaHrana, 0)]);
                        retezec2 = retezec2 + zkoumanaHrana.Data.Zkratka + ", ";
                        retezec = retezec + zkoumanaHrana.Data.Nazev + ", ";
                        foreach (ZdrojHrany polozkaZdroj in zkoumanaHrana.Data.zdrojeHrany)
                        {
                            ZdrojHrany pomZdroj = new ZdrojHrany();
                            pomZdroj.Nazev = polozkaZdroj.Nazev;
                            pomZdroj.Spotreba = polozkaZdroj.Spotreba;
                            int indexPoziceZdroj = zdrojeKombinace.Najdi(pomZdroj,0);
                            if (indexPoziceZdroj < 0) zdrojeKombinace.Add(pomZdroj);
                            else ((ZdrojHrany)(zdrojeKombinace[indexPoziceZdroj])).Spotreba += polozkaZdroj.Spotreba;
                        }
                        if (zkoumanaHrana.Data.Ohodnoceni < minimum)
                        {
                            //vyberu nejkratší hranu z kombinace
                            if (minimum<0) minimum=0;
                            minimum = zkoumanaHrana.Data.Ohodnoceni;
                        }
                    }
                    for (int j = 0; j < kombinaceHran[i].GetLength(0); j++)
                    {
                        Hrana zkoumanaHrana = new Hrana(kombinaceHran[i][j].Vrchol1, kombinaceHran[i][j].Vrchol2);
                        zkoumanaHrana = ((Hrana)naslednici[naslednici.Najdi(zkoumanaHrana, 0)]);
                        zkoumanaHrana.Data.Dokonceno += minimum;
                        // zkoumanaHrana.Data.Vykonano = zkoumanaHrana.Data.Dokonceno;
                    }
                    foreach (Hrana polozka in naslednici)
                    {
                        //polozka.Data.Vykonano = polozka.Data.Dokonceno - minimum;
                        polozka.Data.Ohodnoceni -= polozka.Data.Dokonceno;
                        if (polozka.Data.Ohodnoceni < 0)
                        {
                            //MessageBox.Show("Chyba " + polozka.Data.Nazev);
                            //polozka.Data.Dokonceno = 0;
                            polozka.Data.Ohodnoceni = 0;
                        }
                        //polozka.Data.Ohodnoceni -= minimum;
                    }
                    hodnotaCPM = AlgoritmusTopologickehoUsporadani(false);
                    // ************************
                    int chybaReseni = (int)hodnotaCPM;
                    switch (chybaReseni)
                    {
                        case -1: MessageBox.Show("-1 V síťovém grafu je více než jeden počátečních uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            break;
                        case -2: MessageBox.Show("-2 V síťovém grafu je více než jeden koncových uzlů.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            break;
                        case -3: MessageBox.Show("-3 V síťovém grafu není počáteční uzel.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            break;
                        case -4: MessageBox.Show("-4 V síťovém grafu není koncový uzel", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            break;
                        case -5: MessageBox.Show("-5 Síťový graf není acyklický.", "Špatný vstup", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            break;
                    }
                    if (retezec.Length>2) retezec = retezec.Remove(retezec.Length - 2, 1);
                    if (retezec2.Length > 2) retezec2 = retezec2.Remove(retezec2.Length - 2, 1);
                    Vrchol vkladanyUzel = new Vrchol();
                    vkladanyUzel.Klic.SetKlic("V" + poradiVrcholuPrastromu.ToString("D4"));
                    vkladanyUzel.Nazev = vkladanyUzel.Klic.GetKlic();
                    vkladanyUzel.MinimalniCas = aktualniUzel.MinimalniCas + minimum;
                    vkladanyUzel.MaximalniCas = hodnotaCPM + vkladanyUzel.MinimalniCas;
                    vkladanyUzel.SouradniceX = 70 + 80 * 1;
                    vkladanyUzel.SouradniceY = 70 + 70 * i;
                    prastromReseni.VlozVrchol(vkladanyUzel);
                    parametry.pocetPodreseni++;
                    poradiVrcholuPrastromu += 1;
                    neprozkoumanaReseni.Insert(vkladanyUzel);
                    Hrana vkladanaHrana = new Hrana();
                    vkladanaHrana.Vrchol1 = aktualniUzel.Klic;
                    vkladanaHrana.Vrchol2 = vkladanyUzel.Klic;
                    vkladanaHrana.Data.Nazev = retezec;
                    vkladanaHrana.Data.Zkratka = retezec2;
                    vkladanaHrana.Data.PoleHran = kombinaceHran[i];
                    vkladanaHrana.Data.Vykonano = minimum;
                    vkladanaHrana.Data.Ohodnoceni = vkladanaHrana.Data.Vykonano;
                    vkladanaHrana.Data.zdrojeHrany = zdrojeKombinace;
                    prastromReseni.VlozHranu(vkladanaHrana);
                    if (nastaveni.VyukovyMod == true)
                    {
                        postup.Vloz("Vykonáme činnosti: " + retezec);
                        postup.Vloz("Vložíme do prastromu nový uzel " + vkladanyUzel.Nazev);
                        postup.Vloz("Vložíme do prastromu novou činnost s názvem: " + retezec);
                        postup.Vloz("Tuto činnost ohodnotíme minimálním ohodnocením z vykonávaných činností: " + minimum.ToString());
                        postup.Vloz("Určíme délku cesty od právě vloženého vrcholu ke kořeni: " + vkladanyUzel.MinimalniCas.ToString());
                        postup.Vloz("Určíme kritickou cestu z vkládaného vrcholu bez kapacitního omezení: " + vkladanyUzel.MaximalniCas.ToString());
                    }
                    if (parametry.nastaveni.GrafickyPostup == true)
                    {
                        VypocitejSouradnice(prastromReseni, parametry);
                        parametry.dbPanel.Invalidate();
                        MessageBox.Show("");
                    }
                    // ************************
                }
                foreach (Hrana polozka in naslednici)
                {
                    //polozka.Data.Ohodnoceni += minimum;
                    polozka.Data.Ohodnoceni = polozka.Data.DocasneOhodnoceni;
                    polozka.Data.Dokonceno = 0;
                }
            }
            return 0;
        }

        /// <summary>
        /// Vytvoří kopii seznamu.
        /// </summary>
        /// <param name="seznam">Cílová kopie.</param>
        private void KopirujSeznam(SeznamZdroju seznam)
        {
            foreach (Zdroj polozka in zdroje)
            {
                Zdroj pomZdroj = new Zdroj();
                pomZdroj.Nazev = polozka.Nazev;
                pomZdroj.Obnovitelnost = polozka.Obnovitelnost;
                pomZdroj.PuvodniKapacita = polozka.PuvodniKapacita;
                pomZdroj.AktualniKapacita = polozka.AktualniKapacita;
                if (seznam.Najdi(pomZdroj, 0) < 0) seznam.Add(pomZdroj);
            }
        }
        /// <summary>
        /// Určí souřadnice všech uzlů prastromu řešení.
        /// </summary>
        /// <param name="prastromReseni">Prastrom řešení.</param>
        /// <param name="parametry">Parametry nastavení.</param>
        private void VypocitejSouradnice(Graf prastromReseni, ParametryBaB parametry)
        {
            int vnoreni;
            int maximum = 0;
            foreach (Vrchol polozka in prastromReseni.vrcholy)
            {
                vnoreni = 0;
                if (polozka.PocetPredchudcu > 0)
                {
                    Vrchol predVrchol = new Vrchol(((Hrana)prastromReseni.predchudci[polozka.IndexPrvnihoPredchudce]).Vrchol1.GetKlic()); //klic predchoziho vrcholu
                    predVrchol = (Vrchol)prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(predVrchol, 0)];
                    vnoreni += 1;
                    while (predVrchol.PocetPredchudcu > 0)
                    {
                        predVrchol = new Vrchol(((Hrana)prastromReseni.predchudci[predVrchol.IndexPrvnihoPredchudce]).Vrchol1.GetKlic()); //klic predchoziho vrcholu
                        predVrchol = (Vrchol)prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(predVrchol, 0)];
                        vnoreni += 1;
                    }
                }
                polozka.SouradniceX = 70 + 140 * vnoreni;
                if (vnoreni > maximum) maximum = vnoreni;

            }
            int[] poleVnoreni = new int[maximum + 1];
            foreach (Vrchol polozka in prastromReseni.vrcholy)
            {
                vnoreni = 0;
                if (polozka.PocetPredchudcu > 0)
                {
                    Vrchol predVrchol = new Vrchol(((Hrana)prastromReseni.predchudci[polozka.IndexPrvnihoPredchudce]).Vrchol1.GetKlic()); //klic predchoziho vrcholu
                    predVrchol = (Vrchol)prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(predVrchol, 0)];
                    vnoreni += 1;
                    while (predVrchol.PocetPredchudcu > 0)
                    {
                        predVrchol = new Vrchol(((Hrana)prastromReseni.predchudci[predVrchol.IndexPrvnihoPredchudce]).Vrchol1.GetKlic()); //klic predchoziho vrcholu
                        predVrchol = (Vrchol)prastromReseni.vrcholy[prastromReseni.vrcholy.Najdi(predVrchol, 0)];
                        vnoreni += 1;
                    }
                }
                poleVnoreni[vnoreni] += 1;
                polozka.SouradniceY = 70 * poleVnoreni[vnoreni];
            }
        }
        private void VypisListBox1(ListBox vypisListBox, ADTHrana[] poleHran)
        {
            vypisListBox.BeginUpdate();
            foreach (ADTHrana polozka in poleHran)
            {
                vypisListBox.Items.Add(polozka.Vrchol1.GetKlic() + polozka.Vrchol2.GetKlic());
            }
            vypisListBox.Items.Add("----------");
            vypisListBox.EndUpdate();
        }
        private void VypisMoznosti(VypocetPostup postup, ADTHrana[][] poleHran, NastaveniVzhled nastaveni)
        {
            string retezec = "";
            for (int i = 0; i < poleHran.GetLength(0); i++)
            {
                if (poleHran[i] != null)
                {
                    retezec = "";
                    for (int j = 0; j < poleHran[i].GetLength(0); j++)
                    {
                        Hrana pomHrana = new Hrana(poleHran[i][j].Vrchol1, poleHran[i][j].Vrchol2);
                        pomHrana = (Hrana)(naslednici[naslednici.Najdi(pomHrana, 0)]);
                        if (nastaveni.PopisCinnosti == 1) retezec = retezec + pomHrana.Data.Zkratka + ",";
                        else retezec = retezec + pomHrana.Data.Nazev + ",";
                    }
                    retezec = retezec.Remove(retezec.Length - 1, 1);
                    postup.Vloz(retezec);
                }
            }
        }

        /// <summary>
        /// Vrátí index vrcholu, který je listem. Má nejnižší potřebnou dobu pro dokončení činností.
        /// A zároveň již není vrcholem, kterým končí projekt (tj. zbývající čas kritické cesty je > 0).
        /// 0 .. n - vrací nalezený index
        /// -1 .. neexistuje žádný list
        /// </summary>
        /// <returns><p>int</p> index Vrcholu ze SortedArrayList prastrom reseni</returns>
        private int ziskatIndexListuNejnizsihoCasu(Graf prastromReseni, NastaveniVzhled nastaveni)
        {
            double minimum = double.MaxValue;
            int index = -1;
            //hledam minimum
            for (int i = 0; i < neprozkoumanaReseni.Count; i++)
            {
                if (((Vrchol)(neprozkoumanaReseni[i])).MaximalniCas <= minimum && ((Vrchol)(neprozkoumanaReseni[i])).MaximalniCas<=minimumProjektu)
                {
                    index = i;
                    minimum = ((Vrchol)(neprozkoumanaReseni[i])).MaximalniCas;
                    break;
                }
            }
            if (index == -1) return -1; //reseni nenalezeno, algoritmus konci
            if (((Vrchol)(neprozkoumanaReseni[index])).MaximalniCas == ((Vrchol)(neprozkoumanaReseni[index])).MinimalniCas)
            {
                if (((Vrchol)(neprozkoumanaReseni[index])).MaximalniCas < minimumProjektu) minimumProjektu = ((Vrchol)(neprozkoumanaReseni[index])).MaximalniCas;
                neprozkoumanaReseni.RemoveAt(index); //reseni uz je prozkoumano
                
                //najdu novy vrchol, pokud existuje
                index = -1;
                for (int i = 0; i < neprozkoumanaReseni.Count; i++)
                {
                    if (((Vrchol)(neprozkoumanaReseni[i])).PocetNasledniku == 0 && ((Vrchol)(neprozkoumanaReseni[i])).MaximalniCas == minimum && ((Vrchol)(neprozkoumanaReseni[i])).MaximalniCas > ((Vrchol)(neprozkoumanaReseni[i])).MinimalniCas && ((Vrchol)(neprozkoumanaReseni[i])).MaximalniCas<=minimumProjektu)
                    {
                        index = i;
                        break;
                    }
                }
            }
            return index;
        }
        #endregion

        #region grafika
        /// <summary>
        /// Vypočítá vzdálenost bodu od hrany.
        /// </summary>
        /// <param name="x">X-ová souřadnice</param>
        /// <param name="y">Y-ová souřadnice</param>
        /// <param name="hrana">Hrana</param>
        /// <param name="nastaveni">Nastavení aplikace</param>
        /// <returns></returns>
        private double SpoctiVzdalenost(int x, int y, Hrana hrana, NastaveniVzhled nastaveni)
        {
            Vrchol pomVrchol = new Vrchol();
            Vrchol pomVrchol1, pomVrchol2;
            pomVrchol.Klic.SetKlic(hrana.Vrchol1.GetKlic());
            pomVrchol1 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
            pomVrchol.Klic.SetKlic(hrana.Vrchol2.GetKlic());
            pomVrchol2 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
            int x1 = (int)(pomVrchol1.SouradniceX * nastaveni.ZoomTest);
            int y1 = (int)(pomVrchol1.SouradniceY * nastaveni.ZoomTest);
            int x2 = (int)(pomVrchol2.SouradniceX * nastaveni.ZoomTest);
            int y2 = (int)(pomVrchol2.SouradniceY * nastaveni.ZoomTest);
            //přímka vertikální
            if (x1 == x2)
            {
                //return VypocetVzdalenosti(new Point(x1, y), new Point(x,y));
                Point stred = new Point((int)(Math.Min(x1, x2) + (Math.Abs(x1 - x2) / 2)), (int)(Math.Min(y1, y2) + (Math.Abs(y1 - y2) / 2)));
                return Math.Sqrt(Math.Pow(Math.Abs(stred.X - x), 2) + Math.Pow(Math.Abs(stred.Y - y), 2));

            }
            else if (y1 == y2) //přímka horizontální
            {
                //return VypocetVzdalenosti(new Point(x, y1), new Point(x,y));
                Point stred = new Point((int)(Math.Min(x1, x2) + (Math.Abs(x1 - x2) / 2)), (int)(Math.Min(y1, y2) + (Math.Abs(y1 - y2) / 2)));
                return Math.Sqrt(Math.Pow(Math.Abs(stred.X - x), 2) + Math.Pow(Math.Abs(stred.Y - y), 2));
            }
            else //obecná přímka
            {
                Point smerovyVektor = new Point(x2 - x1, y2 - y1);
                Point normalovyVektor = new Point(smerovyVektor.Y, -smerovyVektor.X);
                double c1 = -(normalovyVektor.X * x1 + normalovyVektor.Y * y1);
                double c2 = -(smerovyVektor.X * x + smerovyVektor.Y * y);
                double vystupMin = double.MaxValue;
                for (int i = (int)Math.Min(x1, x2); i <= Math.Max(x1, x2); i++)
                {
                    double j = (-normalovyVektor.X * i - c1) / (double)normalovyVektor.Y;
                    double vystup = Math.Round(smerovyVektor.X * i + smerovyVektor.Y * j + c2, 0);
                    if (i != Math.Min(x1, x2))
                    {
                        if (vystupMin * vystup < 0)
                        {
                            return VypocetVzdalenosti(new Point(i, (int)j), new Point(x, y), nastaveni);
                        }
                    }
                    vystupMin = vystup;
                }
            }
            return double.MaxValue;
        }
        /// <summary>
        /// Vrací vzdálenost dvou bodů.
        /// </summary>
        /// <param name="point1">Bod 1.</param>
        /// <param name="point2">Bod 2.</param>
        /// <param name="nastaveni">Nastavení aplikace.</param>
        /// <returns></returns>
        private double VypocetVzdalenosti(Point point1, Point point2, NastaveniVzhled nastaveni)
        {
           return Math.Abs(Math.Sqrt(Math.Pow((point2.X * nastaveni.ZoomTest) - (point1.X * nastaveni.ZoomTest), 2.0) + Math.Pow((point2.Y * nastaveni.ZoomTest) - (point1.Y * nastaveni.ZoomTest), 2.0)));
        }
        /// <summary>
        /// Nalezne nejbližší uzel k bodu a nastaví jej.
        /// </summary>
        /// <param name="x">X-ová souřadnice</param>
        /// <param name="y">Y-ová souřadnice</param>
        /// <param name="nastaveni">Nastavení aplikace.</param>
        public void NajdiNejblizsiUzel(int x, int y, NastaveniVzhled nastaveni)
        {
            // najde nejblizsi uzel u kurzoru a vlozi ho do promenne
            nejblizsiUzel = null;
            double max = double.MaxValue;
            double prepona = 0;
            try
            {
                foreach (Vrchol uzel in vrcholy)
                {
                    //prepona = Math.Sqrt((Math.Pow(Math.Abs(x - uzel.SouradniceX), 2) + Math.Pow(Math.Abs(y - uzel.SouradniceY), 2)));
                    prepona = Math.Sqrt((Math.Pow(Math.Abs(x - (uzel.SouradniceX * nastaveni.ZoomTest)), 2) + Math.Pow(Math.Abs(y - (uzel.SouradniceY * nastaveni.ZoomTest)), 2)));
                    if (prepona <= max && prepona < 25)
                    {
                        max = prepona;
                        nejblizsiUzel = uzel;
                        break;
                    }
                    else
                    {
                        nejblizsiUzel = null;
                    }
                }
            }
            catch (Exception) { }
        }
        /// <summary>
        /// Nalezne nejbližší hranu k bodu a nastaví jej.
        /// </summary>
        /// <param name="x">X-ová souřadnice</param>
        /// <param name="y">Y-ová souřadnice</param>
        /// <param name="nastaveni">Nastavení aplikace.</param>
        public void NajdiNejblizsiHranu(int x, int y, NastaveniVzhled nastaveni)
        { // nalezne hranu co je nejblize ke kurzoru
            nejblizsiHrana = null;
            double max = double.MaxValue;
            double vzdalenost = double.MaxValue;
            if (naslednici.Count == 0) return;
            try
            {
                foreach (Hrana polozka in naslednici)
                {
                    vzdalenost = SpoctiVzdalenost(x, y, polozka, nastaveni);
                    if (max > vzdalenost && vzdalenost < 25)
                    {
                        Vrchol pomVrchol = new Vrchol();
                        Vrchol pomVrchol1, pomVrchol2;
                        pomVrchol.Klic.SetKlic(polozka.Vrchol1.GetKlic());
                        pomVrchol1 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
                        pomVrchol.Klic.SetKlic(polozka.Vrchol2.GetKlic());
                        pomVrchol2 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
                        max = vzdalenost;
                        nejblizsiHrana = polozka;
                        polozka.NastavStred(new Point((int)(Math.Min(pomVrchol1.SouradniceX, pomVrchol2.SouradniceX) + (Math.Abs(pomVrchol1.SouradniceX - pomVrchol2.SouradniceX) / 2)), (int)(Math.Min(pomVrchol1.SouradniceY, pomVrchol2.SouradniceY) + (Math.Abs(pomVrchol1.SouradniceY - pomVrchol2.SouradniceY) / 2))));
                    }
                }
            }
            catch (Exception) {}
        }

        /// <summary>
        /// Vykreslí graf.
        /// </summary>
        /// <param name="g">Grafika.</param>
        /// <param name="nastaveni">Nastavení aplikace.</param>
        public void Kresli(Graphics g, NastaveniVzhled nastaveni)
        {
            try
            {
                foreach (Hrana polozka in naslednici)
                {
                    if (polozka.Kreslit == true)
                    {
                        // kresli XOR
                        KresliXOR(g, new Pen(Color.Red, 2), polozka);
                        continue;
                    }
                    else
                    // kresli hranu normalne
                    {
                        switch (polozka.Data.Typ)
                        {
                            case 0: KresliHranu(g, nastaveni, polozka, 0); //obycejna hrana
                                break;
                            case 1: KresliHranu(g, nastaveni, polozka, 3); //rezerva
                                break;
                            case 2: KresliHranu(g, nastaveni, polozka, 4); //fiktivni hrana
                                break;
                            case 3: KresliHranu(g, nastaveni, polozka, 6); //hrana zavislosti
                                break;
                            case 4: KresliHranu(g, nastaveni, polozka, 7); //fiktivni hrana zavislosti
                                break;
                        }
                    }
                }
                if (kritickaCesta.Count > 0)
                {
                    //MessageBox.Show("fsdfd");
                    foreach (Hrana polozka in kritickaCesta)
                    {
                        switch (polozka.Data.Typ)
                        {
                            case 0: KresliHranu(g, nastaveni, polozka, 2);
                                break;
                            case 3: KresliHranu(g, nastaveni, polozka, 8);
                                break;
                            case 4:
                                KresliHranu(g, nastaveni, polozka, 9);
                                break;
                            default:
                                KresliHranu(g, nastaveni, polozka, 5);
                                break;
                        }
                    }
                }
                if (nejblizsiHrana != null)
                {
                    KresliHranu(g, nastaveni, nejblizsiHrana, 1);
                }

                foreach (Vrchol polozka in vrcholy)
                {
                    if (polozka.Kreslit == true)
                    {
                        polozka.KresliXorUzel(g, new SolidBrush(Color.Blue), nastaveni.Meritko * 5);
                    }
                    else
                    {
                        //polozka.VykresliUzel(g, new SolidBrush(Color.MediumPurple), meritko*5);
                        polozka.VykresliUzel(g, nastaveni.Barva1Vrchol, nastaveni.Barva2Vrchol, nastaveni.Velikost, kreslitBaB, nastaveni.PismoFont, nastaveni.Tvar, this.TypOhodnoceniHran, nastaveni);
                    }
                }
                if (nejblizsiUzel != null) nejblizsiUzel.VykresliUzel(g, nastaveni.Barva1NejUzel, nastaveni.Barva2NejUzel, nastaveni.Velikost, kreslitBaB, nastaveni.PismoFont, nastaveni.Tvar, this.TypOhodnoceniHran, nastaveni);
            }
            catch (Exception) {}
        }

        /// <summary>
        /// Vykreslí hranu grafu.
        /// </summary>
        /// <param name="g">Grafika.</param>
        /// <param name="nastaveni">Nastavení aplikace.</param>
        /// <param name="polozka">Hrana.</param>
        /// <param name="varianta">Typ hrany.</param>
        private void KresliHranu(Graphics g, NastaveniVzhled nastaveni, Hrana polozka, int varianta)
        {
            int difX, difY;
            Vrchol pomVrchol = new Vrchol();
            Vrchol pomVrchol1, pomVrchol2;
            pomVrchol.Klic.SetKlic(polozka.Vrchol1.GetKlic());
            pomVrchol1 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
            pomVrchol.Klic.SetKlic(polozka.Vrchol2.GetKlic());
            pomVrchol2 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
            Pen pero = null;
            switch (varianta)
            {
                case 0: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva1Hrana, nastaveni.Barva2Hrana), nastaveni.Tloustka * nastaveni.Meritko);
                    break;
                case 2: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva1KritickaHrana, nastaveni.Barva2KritickaHrana), nastaveni.Tloustka * nastaveni.Meritko);
                    break;
                case 1: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva1NejHrana, nastaveni.Barva2NejHrana), nastaveni.Tloustka * nastaveni.Meritko);
                    break;
                case 3: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva1Rezerva, nastaveni.Barva2Rezerva), nastaveni.Meritko);
                    pero.DashPattern = new float[] { 4.0F, 2.0F, 4.0F, 2.0F };
                    break;
                case 4: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva1FiktivniHrana, nastaveni.Barva2FiktivniHrana), nastaveni.Tloustka * nastaveni.Meritko);
                    pero.DashPattern = new float[] { 4.0F, 2.0F, 4.0F, 2.0F };
                    break;
                case 5: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva1FiktivniKritickaHrana, nastaveni.Barva2FiktivniKritickaHrana), nastaveni.Tloustka * nastaveni.Meritko);
                    pero.DashPattern = new float[] { 4.0F, 2.0F, 4.0F, 2.0F };
                    break;
                case 6: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva1Zavislost, nastaveni.Barva2Zavislost), nastaveni.Tloustka2 * nastaveni.Meritko);
                    break;
                case 7: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva1FiktivniZavislost, nastaveni.Barva2FiktivniZavislost), nastaveni.Tloustka2 * nastaveni.Meritko);
                    pero.DashPattern = new float[] { 2.0F, 3.0F, 2.0F, 3.0F };
                    break;
                case 8: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva1KritickaZavislost, nastaveni.Barva2KritickaZavislost), nastaveni.Tloustka2 * nastaveni.Meritko);
                    break;
                case 9: pero = new Pen(new LinearGradientBrush(new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + 1), nastaveni.Barva2FiktivniKritickaZavislost, nastaveni.Barva2FiktivniKritickaZavislost), nastaveni.Tloustka2 * nastaveni.Meritko);
                    pero.DashPattern = new float[] { 2.0F, 3.0F, 2.0F, 3.0F };
                    break;
            }
            pero.DashCap = System.Drawing.Drawing2D.DashCap.Round;
            pero.EndCap = System.Drawing.Drawing2D.LineCap.ArrowAnchor;

            if (pomVrchol1.Klic.GetKlic().Equals(pomVrchol2.Klic.GetKlic()) == true) //smyčka
            {
                pero.Color = Color.Red;
                g.DrawEllipse(pero, pomVrchol1.SouradniceX - nastaveni.Meritko * 30, pomVrchol1.SouradniceY - nastaveni.Meritko * 30, nastaveni.Meritko * 30, nastaveni.Meritko * 30);
            }
            else
            {
                difX = Math.Abs(pomVrchol2.SouradniceX - pomVrchol1.SouradniceX);
                difY = Math.Abs(pomVrchol2.SouradniceY - pomVrchol1.SouradniceY);

                //polozka.NastavStred(new Point((int)(difX / 2), (int)(difY / 2)));
                if (varianta == 3)
                {
                    g.DrawRectangle(pero, pomVrchol1.SouradniceX, pomVrchol1.SouradniceY, pomVrchol2.SouradniceX - pomVrchol1.SouradniceX - nastaveni.Velikost * 2, nastaveni.Tloustka);
                    //double jednotka=(pomVrchol2.SouradniceX-pomVrchol1.SouradniceX)/polozka.Data.RC;
                    //g.DrawRectangle(pero, pomVrchol1.SouradniceX, pomVrchol1.SouradniceY, (int)(jednotka*polozka.Data.RV) - nastaveni.Velikost * 2, nastaveni.Tloustka);
                    //g.DrawRectangle(pero, pomVrchol1.SouradniceX, pomVrchol1.SouradniceY, (int)(jednotka * polozka.Data.RN) - nastaveni.Velikost * 2, nastaveni.Tloustka);
                }
                else
                {
                    if (pomVrchol1.SouradniceX < pomVrchol2.SouradniceX) //zleva doprava
                    {
                        if (pomVrchol1.SouradniceY < pomVrchol2.SouradniceY) // zeshora dolu a zleva doprava - 1. kvadrant
                        {
                            if (difX > difY)
                            {

                                if (nastaveni.Tvar == 0)
                                    g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX - nastaveni.Velikost * 2, pomVrchol2.SouradniceY));
                                else g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX - nastaveni.Velikost * 6, pomVrchol2.SouradniceY));

                            }
                            else g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY - nastaveni.Velikost * 2));
                        }
                        else //zezdola nahoru a zleva doprava - 4. kvadrant
                            if (difX > difY)
                            {
                                if (nastaveni.Tvar == 0)
                                    g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX - nastaveni.Velikost * 2, pomVrchol2.SouradniceY));
                                else g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX - nastaveni.Velikost * 6, pomVrchol2.SouradniceY));
                            }
                            else g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + nastaveni.Velikost * 2));
                    }
                    else //zprava doleva
                    {
                        if (pomVrchol1.SouradniceY < pomVrchol2.SouradniceY) // zeshora dolu a zleva doprava - 1. kvadrant
                        {
                            if (difX > difY)
                            {
                                if (nastaveni.Tvar == 0)
                                    g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX + nastaveni.Velikost * 2, pomVrchol2.SouradniceY));
                                else g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX + nastaveni.Velikost * 6, pomVrchol2.SouradniceY));
                            }
                            else g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY - nastaveni.Velikost * 2));
                        }
                        else //zezdola nahoru a zleva doprava - 4. kvadrant
                            if (difX > difY)
                            {
                                if (nastaveni.Tvar == 0)
                                    g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX + nastaveni.Velikost * 2, pomVrchol2.SouradniceY));
                                else g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX + nastaveni.Velikost * 6, pomVrchol2.SouradniceY));
                            }
                            else g.DrawLine(pero, new Point(pomVrchol1.SouradniceX, pomVrchol1.SouradniceY), new Point(pomVrchol2.SouradniceX, pomVrchol2.SouradniceY + nastaveni.Velikost * 2));
                    }
                }
                if (polozka.Data != null && polozka.Data.Nazev != null)
                {
                    if (pomVrchol1.SouradniceX < pomVrchol2.SouradniceX)
                    {
                        if (pomVrchol1.SouradniceY > pomVrchol2.SouradniceY)
                        {
                            switch (nastaveni.PopisCinnosti)
                            {
                                case 0: g.DrawString(polozka.Data.Nazev, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX + (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY + (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko * 2 - 3);
                                    break;
                                case 1: g.DrawString(polozka.Data.Zkratka, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX + (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY + (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko * 2 - 3);
                                    break;
                            }
                            string retezec = String.Format("{0:F2}", polozka.Data.Ohodnoceni);
                            if (TypOhodnoceniHran == "CPM" && nastaveni.ZobrazitCinnostOhodnoceni == true) g.DrawString(retezec, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX + (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY + (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko - 1);
                            else if (nastaveni.ZobrazitCinnostOhodnoceni == true) g.DrawString(polozka.Data.A.ToString() + "-" + polozka.Data.M.ToString() + "-" + polozka.Data.B.ToString(), nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX + (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY + (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko - 1);
                        }
                        else
                        {
                            switch (nastaveni.PopisCinnosti)
                            {
                                case 0: g.DrawString(polozka.Data.Nazev, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX + (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY - (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko * 2 - 3);
                                    break;
                                case 1: g.DrawString(polozka.Data.Zkratka, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX + (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY - (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko * 2 - 3);
                                    break;
                            }
                            string retezec = String.Format("{0:F2}", polozka.Data.Ohodnoceni);
                            if (TypOhodnoceniHran == "CPM" && nastaveni.ZobrazitCinnostOhodnoceni == true) g.DrawString(retezec, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX + (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY - (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko - 1);
                            else if (nastaveni.ZobrazitCinnostOhodnoceni == true) g.DrawString(polozka.Data.A.ToString() + "-" + polozka.Data.M.ToString() + "-" + polozka.Data.B.ToString(), nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX + (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY - (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko - 1);
                        }
                    }
                    else
                    {
                        if (pomVrchol1.SouradniceY > pomVrchol2.SouradniceY)
                        {
                            switch (nastaveni.PopisCinnosti)
                            {
                                case 0: g.DrawString(polozka.Data.Nazev, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX - (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY + (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko * 2 - 3);
                                    break;
                                case 1: g.DrawString(polozka.Data.Zkratka, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX - (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY + (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko * 2 - 3);
                                    break;
                            }
                            string retezec = String.Format("{0:F2}", polozka.Data.Ohodnoceni);
                            if (TypOhodnoceniHran == "CPM" && nastaveni.ZobrazitCinnostOhodnoceni == true) g.DrawString(retezec, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX - (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY + (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko - 1);
                            else if (nastaveni.ZobrazitCinnostOhodnoceni == true) g.DrawString(polozka.Data.A.ToString() + "-" + polozka.Data.M.ToString() + "-" + polozka.Data.B.ToString(), nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX - (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY + (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko - 1);
                        }
                        else
                        {
                            switch (nastaveni.PopisCinnosti)
                            {
                                case 0: g.DrawString(polozka.Data.Nazev, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX - (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY - (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko * 2 - 3);
                                    break;
                                case 1: g.DrawString(polozka.Data.Zkratka, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX - (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY - (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko * 2 - 3);
                                    break;
                            }
                            string retezec = String.Format("{0:F2}", polozka.Data.Ohodnoceni);
                            if (TypOhodnoceniHran == "CPM" && nastaveni.ZobrazitCinnostOhodnoceni == true) g.DrawString(retezec, nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX - (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY - (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko - 1);
                            else if (nastaveni.ZobrazitCinnostOhodnoceni == true) g.DrawString(polozka.Data.A.ToString() + "-" + polozka.Data.M.ToString() + "-" + polozka.Data.B.ToString(), nastaveni.HranaFont, nastaveni.BarvaPismaHrana, pomVrchol1.SouradniceX - (int)(difX / 2) - (int)(((polozka.Data.Nazev.Length / 3) * nastaveni.HranaFont.Size * nastaveni.Meritko)), pomVrchol2.SouradniceY - (int)(difY / 2) - nastaveni.HranaFont.Size * nastaveni.Meritko - 1);
                        }
                    }
                }
            }
        }

        public void KresliXOR(Graphics g, Pen pen, Hrana hrana)
        {
            Vrchol pomVrchol = new Vrchol();
            Vrchol pomVrchol1, pomVrchol2;
            pomVrchol.Klic.SetKlic(hrana.Vrchol1.GetKlic());
            pomVrchol1 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
            pomVrchol.Klic.SetKlic(hrana.Vrchol2.GetKlic());
            pomVrchol2 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
            //XOR
            IntPtr HDC = g.GetHdc();
            IntPtr aktPen = ApiGdi.CreatePen((int)PenStyles.PS_DASH, 2, ApiGdi.RGB(0, 0, 255));
            IntPtr oldPen = ApiGdi.SelectObject(HDC, aktPen);
            IntPtr aktBrush = ApiGdi.CreateSolidBrush(ApiGdi.RGB(140, 0, 0));
            IntPtr oldBrush = ApiGdi.SelectObject(HDC, aktBrush);

            int oldMode = ApiGdi.SetROP2(HDC, (int)RasterOps.R2_XORPEN);

            // kresli vyplnenou kruznici
            ApiGdi.MoveToEx(HDC, pomVrchol1.SouradniceX, pomVrchol1.SouradniceY, IntPtr.Zero);
            ApiGdi.LineTo(HDC, pomVrchol2.SouradniceX, pomVrchol2.SouradniceY);
            ApiGdi.SelectObject(HDC, oldPen);
            ApiGdi.DeleteObject(aktPen);
            ApiGdi.SelectObject(HDC, oldBrush);
            ApiGdi.DeleteObject(aktBrush);
            ApiGdi.SetROP2(HDC, oldMode);
            g.ReleaseHdc(HDC);
        }

        public void NastavIncidenceVrcholuXOR(Vrchol vrchol)
        {
            //je to hotove, ale zatim vypinam
            /*
            int i, index;
            if (vrchol.PocetPredchudcu > 0)
            {
                index = vrchol.IndexPrvnihoPredchudce;
                for (i = 0; i < vrchol.PocetPredchudcu; i++)
                {
                    ((Hrana)predchudci[index + i]).Kreslit = true;
                }
            }
            if (vrchol.PocetNasledniku > 0)
            {
                index = vrchol.IndexPrvnihoNaslednika;
                for (i = 0; i < vrchol.PocetNasledniku; i++)
                {
                    ((Hrana)naslednici[index + i]).Kreslit = true;
                }
            }*/
        }

        public void NastavHranuXOR(Hrana hrana)
        {
            //funguje, ale zatim vypnuto
            /*
            Vrchol pomVrchol = new Vrchol();
            Vrchol pomVrchol1;
            hrana.Kreslit = true;
            pomVrchol.Klic.SetKlic(hrana.Vrchol1.GetKlic());
            pomVrchol1 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
            pomVrchol1.Kreslit = true;
            NastavIncidenceVrcholuXOR(pomVrchol1);
            pomVrchol.Klic.SetKlic(hrana.Vrchol2.GetKlic());
            pomVrchol1 = (Vrchol)vrcholy[vrcholy.Najdi(pomVrchol, 0)];
            pomVrchol1.Kreslit = true;
            NastavIncidenceVrcholuXOR(pomVrchol1);*/
        }
        public void XorNuluj()
        {
            //je to hotove, ale zatim to vypnu
            /*foreach (Hrana polozka in naslednici)
            {
                polozka.Kreslit = false;
            }
            foreach (Hrana polozka in predchudci)
            {
                polozka.Kreslit = false;
            }
            foreach (Vrchol polozka in vrcholy)
            {
                polozka.Kreslit = false;
            }*/
        }
        #endregion
    }
}
