﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;
using System.Windows.Forms;

namespace DiplomovaPrace
{
    public class Hrana : ADTHrana
    {
        private DataHrana data;
        private bool kreslit = false; //nekreslit XOR
        private Point stred;
        //private String zdroje = new String[];
        #region Vlastnosti
        public Point Stred
        {
            get { return stred; }
            set
            {
                stred = value;
            }
        }
        public bool Kreslit
        {
            get { return kreslit; }
            set { kreslit = value; }
        }
        public DataHrana Data
        {
            get { return data; }
            set { data = value; }
        }
        #endregion
        public Hrana() : base("", "")
        {
            data = new DataHrana();
        }
        public Hrana(KlicVrchol Klic1, KlicVrchol Klic2, String nazev, double ohodnoceni)
            : base("", "")
        {
            data = new DataHrana();
            Vrchol1.SetKlic(Klic1.GetKlic());
            Vrchol2.SetKlic(Klic2.GetKlic());
            this.data.Nazev = nazev;
            this.data.Ohodnoceni = ohodnoceni;
        }
        public Hrana(KlicVrchol Klic1, KlicVrchol Klic2)
            : base("", "")
        {
            data = new DataHrana();
            Vrchol1.SetKlic(Klic1.GetKlic());
            Vrchol2.SetKlic(Klic2.GetKlic());
        }
        /// <summary>
        /// Přidá hraně zdroj.
        /// </summary>
        /// <param name="zdroj">Přidávaný zdroj.</param>
        /// <returns>0 - v pořádku, 1 - chyba</returns>
        public int PridejZdroj(ZdrojHrany zdroj)
        {
            int pom = data.zdrojeHrany.Najdi(zdroj, 0);
            if (pom >= 0)
            {
                return 1; //nastala chyba, zdroj jiz existuje
            }       
            Data.zdrojeHrany.Add(zdroj);
            return 0;
        }
        /// <summary>
        /// Odebírá hraně zdroj.
        /// </summary>
        /// <param name="zdroj">Odebíraný zdroj.</param>
        /// <returns>0 - v pořádku, 1 - chyba</returns>
        public int OdeberZdroj(ZdrojHrany zdroj)
        {
            int pozice = data.zdrojeHrany.Najdi(zdroj, 0);
            if (pozice < 0) return 1;
            Data.zdrojeHrany.RemoveAt(pozice);
            return 0;
        }

        public override string ToString()
        {
            return "Klíč: " + Vrchol1.GetKlic() + Vrchol2.GetKlic() + " Název: " + data.Nazev + " Ohodnocení: " + data.Ohodnoceni.ToString();
        }
        public void NastavStred(Point stred)
        {
            this.stred.X = stred.X;
            this.stred.Y = stred.Y;
        }
        /// <summary>
        /// Vytvoří kopii hrany.
        /// </summary>
        /// <param name="hrana">Cílová kopie.</param>
        public void Kopie(Hrana hrana)
        {
            hrana.Data.Nazev = this.Data.Nazev;
            hrana.Data.zdrojeHrany = this.Data.zdrojeHrany;
            hrana.Data.Ohodnoceni = this.Data.Ohodnoceni;
            hrana.Data.DocasneOhodnoceni = this.Data.DocasneOhodnoceni;
            hrana.Data.PoleHran = this.Data.PoleHran;
            hrana.Data.Vykonano = this.Data.Vykonano;
            hrana.Data.RC = this.Data.RC;
            hrana.Data.RN = this.Data.RN;
            hrana.Data.RV = this.Data.RV;
            hrana.Data.Typ = this.Data.Typ;
            hrana.Data.PredKlic = this.Data.PredKlic;
            hrana.Data.Zkratka = this.data.Zkratka;
            hrana.Data.Info = this.data.Info;
            hrana.Data.A = this.Data.A;
            hrana.Data.M = this.Data.M;
            hrana.Data.B = this.Data.B;
        }
        /// <summary>
        /// Vytvoří kopii hrany.
        /// </summary>
        /// <param name="hrana">Cílová kopie.</param>
        public void Kopie2(Hrana hrana)
        {
            //hrana.Vrchol1.SetKlic(this.Vrchol1.GetKlic());
            //hrana.Vrchol2.SetKlic(this.Vrchol2.GetKlic());
            hrana.Data.Nazev = this.Data.Nazev;
            hrana.Data.zdrojeHrany = this.Data.zdrojeHrany;
            hrana.Data.Ohodnoceni = this.Data.Ohodnoceni;
            hrana.Data.DocasneOhodnoceni = this.Data.DocasneOhodnoceni;
            hrana.Data.Vykonano = this.Data.Vykonano;
            hrana.Data.RC = this.Data.RC;
            hrana.Data.RN = this.Data.RN;
            hrana.Data.RV = this.Data.RV;
            hrana.Data.Typ = this.Data.Typ;
            hrana.Data.Zkratka = this.data.Zkratka;
            hrana.Data.Info = this.data.Info;
            hrana.Data.A = this.Data.A;
            hrana.Data.M = this.Data.M;
            hrana.Data.B = this.Data.B;
        }
        #region praceSeSoubory
        /// <summary>
        /// Uloží hranu do textového souboru.
        /// </summary>
        /// <param name="sw">Datový proud.</param>
        public void Uloz(StreamWriter sw)
        {
            data.Info = data.Info.Replace("\n", " ");
            Vrchol1.Uloz(sw);
            Vrchol2.Uloz(sw);
            sw.WriteLine(data.Nazev);
            sw.WriteLine(data.Ohodnoceni);
            sw.WriteLine(data.A);
            sw.WriteLine(data.M);
            sw.WriteLine(data.B);
            sw.WriteLine(data.Typ);
            sw.WriteLine(data.Zkratka);
            sw.WriteLine(data.Info);
            data.zdrojeHrany.Uloz(sw); 
        }
        /// <summary>
        /// Načte hranu z textového souboru.
        /// </summary>
        /// <param name="sr">Datový proud.</param>
        public void Nacti(StreamReader sr)
        {
            Vrchol1.Nacti(sr);
            Vrchol2.Nacti(sr);
            data.Nazev = sr.ReadLine();
            string pomRet = sr.ReadLine();
            double pomCislo;
            if (double.TryParse(pomRet, out pomCislo) == false)
                throw new Exception("Chyba pří načítání ohodnocení ze souboru");
            data.Ohodnoceni = pomCislo;
            pomRet = sr.ReadLine();
            if (double.TryParse(pomRet, out pomCislo) == false)
                throw new Exception("Chyba pří načítání ohodnocení ze souboru");
            data.A = pomCislo;
            pomRet = sr.ReadLine();
            if (double.TryParse(pomRet, out pomCislo) == false)
                throw new Exception("Chyba pří načítání ohodnocení ze souboru");
            data.M = pomCislo;
            pomRet = sr.ReadLine();
            if (double.TryParse(pomRet, out pomCislo) == false)
                throw new Exception("Chyba pří načítání ohodnocení ze souboru");
            data.B = pomCislo;
            pomRet = sr.ReadLine();
            int pomCisloInt;
            if (int.TryParse(pomRet, out pomCisloInt) == false)
                throw new Exception("Chyba pří načítání ohodnocení ze souboru");
            data.Typ = pomCisloInt;
            data.Zkratka = sr.ReadLine();
            data.Info = sr.ReadLine();
            data.zdrojeHrany.Nacti(sr);
         }
        /// <summary>
        /// Uloží hranu do binárního souboru.
        /// </summary>
        /// <param name="bw">Datový proud.</param>
        public void Uloz(BinaryWriter bw)
        {
            data.Info = data.Info.Replace("\n", " ");
            Vrchol1.Uloz(bw);
            Vrchol2.Uloz(bw);
            bw.Write(data.Nazev);
            bw.Write(data.Ohodnoceni);
            bw.Write(data.A);
            bw.Write(data.M);
            bw.Write(data.B);
            bw.Write(data.Typ);
            bw.Write(data.Zkratka);
            bw.Write(data.Info);
            data.zdrojeHrany.Uloz(bw);
        }
        /// <summary>
        /// Načte hranu z binárního souboru.
        /// </summary>
        /// <param name="br">Datový proud.</param>
        public void Nacti(BinaryReader br)
        {
            Vrchol1.Nacti(br);
            Vrchol2.Nacti(br);
            data.Nazev = br.ReadString();
            data.Ohodnoceni = br.ReadDouble();
            data.A = br.ReadDouble();
            data.M = br.ReadDouble();
            data.B = br.ReadDouble();
            data.Typ = br.ReadInt32();
            data.Zkratka = br.ReadString();
            data.Info = br.ReadString();
            data.zdrojeHrany.Nacti(br);
        }
#endregion

    }
}
