﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DiplomovaPrace
{
    public partial class HranaDialog : Form
    {
        private Vrchol vrchol;
        private Hrana hrana;
        private Zdroj zdroj;
        private Graf graf;
        private bool zmena = false;
        private KlicVrchol staryKlic1;
        private KlicVrchol staryKlic2;
        #region vlastnosti
        public Hrana Hrana
        {
            get { return hrana; }
            set { hrana = value; }
        }
        public Zdroj Zdroj
        {
            get { return zdroj; }
            set { zdroj = value; }
        }
        public Vrchol Vrchol
        {
            get { return vrchol; }
            set { vrchol = value; }
        }
        #endregion

        public HranaDialog(Graf graf)
        {
            this.graf=graf;
            InitializeComponent();
            graf.zdroje.Sort(new CompareNazevZdroj(true));
            foreach (Vrchol polozka in graf.vrcholy)
            {
                pocatekComboBox.Items.Add(polozka.Klic.GetKlic());
                konecComboBox.Items.Add(polozka.Klic.GetKlic());
            }
            pocatekComboBox.DisplayMember = "Klíč vrcholu";
            foreach (Zdroj polozka in graf.zdroje)
            {
                //if (polozka.Obnovitelnost==true) zdrojComboBox.Items.Add(polozka.Nazev +"(AK:"+polozka.AktualniKapacita+" PK:"+polozka.PuvodniKapacita+" O)");
                //else zdrojComboBox.Items.Add(polozka.Nazev + "(AK:" + polozka.AktualniKapacita + " PK:" + polozka.PuvodniKapacita + " N)");
                zdrojComboBox.Items.Add(polozka.Nazev);
                //zdrojComboBox.sho
            }
        }

        private void HranaDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DialogResult == DialogResult.Cancel) return;
            e.Cancel = true;
            if (string.IsNullOrEmpty(nazevTextBox.Text))
            {
                MessageBox.Show("Název činnosti není vyplněn.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (string.IsNullOrEmpty(zkratkaTextBox.Text))
            {
                MessageBox.Show("Zkratka činnosti není vyplněna.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (zkratkaTextBox.Text.Length>3)
            {
                MessageBox.Show("Zkratka činnosti musí být kratší než 3 znaky.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (pocatekComboBox.SelectedItem == null)
            {
                MessageBox.Show("Počáteční vrchol není vyplněn.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (konecComboBox.SelectedItem == null)
            {
                MessageBox.Show("Koncový vrchol není vyplněn.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            double pomOhodnoceni;
            if (!double.TryParse(ohodnoceniTextBox.Text, out pomOhodnoceni))
            {
                MessageBox.Show("Není vyplněno korektně ohodnocení hrany", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (fiktivniCheckBox.Checked==true && pomOhodnoceni!=0)
            {
                MessageBox.Show("Pokud je fiktivní hrana, musí být ohodnocení 0.", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!double.TryParse(optimistickyTextBox.Text, out pomOhodnoceni))
            {
                MessageBox.Show("Není vyplněn korektně optimisticky odhad", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!double.TryParse(modusTextBox.Text, out pomOhodnoceni))
            {
                MessageBox.Show("Není vyplněn korektně nejpravděpodobnější odhad", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!double.TryParse(pesimistickyTextBox.Text, out pomOhodnoceni))
            {
                MessageBox.Show("Není vyplněn korektně pesimistický odhad", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (zmena == true)
            {
                graf.OdeberHranu(hrana);
                hrana.Vrchol1.SetKlic(pocatekComboBox.SelectedItem.ToString());
                hrana.Vrchol2.SetKlic(konecComboBox.SelectedItem.ToString());
                int chyba = graf.VlozHranu(hrana);
                if (chyba == 3)
                {
                    MessageBox.Show("Nelze vkládat smyčky", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    hrana.Vrchol1.SetKlic(staryKlic1.GetKlic());
                    hrana.Vrchol2.SetKlic(staryKlic2.GetKlic());
                    graf.VlozHranu(hrana);
                    pocatekComboBox.Text = hrana.Vrchol1.GetKlic();
                    konecComboBox.Text = hrana.Vrchol2.GetKlic();
                    return;
                } else
                    if (chyba == 2)
                {
                    MessageBox.Show("Nelze vkládat násobné hrany", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    hrana.Vrchol1.SetKlic(staryKlic1.GetKlic());
                    hrana.Vrchol2.SetKlic(staryKlic2.GetKlic());
                    graf.VlozHranu(hrana);
                    pocatekComboBox.Text = hrana.Vrchol1.GetKlic();
                    konecComboBox.Text = hrana.Vrchol2.GetKlic();
                    return;
                }
            }
            e.Cancel = false;
        }

        private void HranaDialog_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (DialogResult == DialogResult.Cancel) return;
            if (fiktivniCheckBox.Checked == true) hrana.Data.Typ = 2; else hrana.Data.Typ = 0;
            hrana.Data.Nazev = nazevTextBox.Text;
            hrana.Data.Zkratka = zkratkaTextBox.Text;
            hrana.Data.Info = informaceRichTextBox.Text;
            if (graf.TypOhodnoceniHran == "CPM")
            {
                hrana.Data.Ohodnoceni = double.Parse(ohodnoceniTextBox.Text);
            }
            else
            {
                hrana.Data.A = double.Parse(optimistickyTextBox.Text);
                hrana.Data.M = double.Parse(modusTextBox.Text);
                hrana.Data.B = double.Parse(pesimistickyTextBox.Text);
            }                
        }

        private void HranaDialog_Load(object sender, EventArgs e)
        {
            if (graf.TypOhodnoceniHran == "CPM")
            {
                label7.Visible = false;
                label8.Visible = false;
                label9.Visible = false;
                optimistickyTextBox.Visible = false;
                modusTextBox.Visible = false;
                pesimistickyTextBox.Visible = false;
            }
            else
            {
                ohodnoceniTextBox.Visible = false;
            }
            nazevTextBox.Text = hrana.Data.Nazev;
            switch (hrana.Data.Typ)
            {
                case 0: fiktivniCheckBox.Checked = false;
                    break;
                case 2: fiktivniCheckBox.Checked = true;
                    break;
            }
            if (string.IsNullOrEmpty(hrana.Vrchol1.GetKlic()))
            {
                pocatekComboBox.SelectedItem = null;
                konecComboBox.SelectedItem = null;
                ohodnoceniTextBox.Text = "0";
                optimistickyTextBox.Text = "0";
                modusTextBox.Text = "0";
                pesimistickyTextBox.Text = "0";
                zkratkaTextBox.Text = "";
            }
            else
            {
                ohodnoceniTextBox.Text = hrana.Data.Ohodnoceni.ToString();
                optimistickyTextBox.Text = hrana.Data.A.ToString();
                modusTextBox.Text = hrana.Data.M.ToString();
                pesimistickyTextBox.Text = hrana.Data.B.ToString();
                zkratkaTextBox.Text = hrana.Data.Zkratka;
                informaceRichTextBox.Text = hrana.Data.Info;
                int index = 0;
                foreach (String polozka in pocatekComboBox.Items)
                {
                    if (polozka == hrana.Vrchol1.GetKlic())
                    {
                        pocatekComboBox.SelectedIndex = index;
                        break;
                    }
                    index++;
                }
                index = 0;
                foreach (String polozka in konecComboBox.Items)
                {
                    if (polozka == hrana.Vrchol2.GetKlic())
                    {
                        konecComboBox.SelectedIndex = index;
                        break;
                    }
                    index++;
                }
                VypisZdrojuHrany();
            }

 
        }

        private void zdrojComboBox_MouseHover(object sender, EventArgs e)
        {
         }

        private void zdrojComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (zdrojComboBox.SelectedItem == null) return;
            if (string.IsNullOrEmpty(zdrojComboBox.SelectedItem.ToString())) return;
            infoZdrojListBox.Items.Clear();
            infoZdrojListBox.BeginUpdate();
            Zdroj pomZdroj = new Zdroj();
            pomZdroj.Nazev = zdrojComboBox.SelectedItem.ToString();
            int pozice = graf.zdroje.Najdi(pomZdroj, 0);
            infoZdrojListBox.Items.Add("Zdroj: " + ((Zdroj)(graf.zdroje[pozice])).Nazev);
            infoZdrojListBox.Items.Add("Aktuální kapacita: " + ((Zdroj)(graf.zdroje[pozice])).AktualniKapacita.ToString());
            infoZdrojListBox.Items.Add("Původní kapacita: " + ((Zdroj)(graf.zdroje[pozice])).PuvodniKapacita.ToString());
            if (((Zdroj)(graf.zdroje[pozice])).Obnovitelnost==true)infoZdrojListBox.Items.Add("Obnovitelný");
            else infoZdrojListBox.Items.Add("Neobnovitelný");
            infoZdrojListBox.EndUpdate();
        }

        private void pridatButton_Click(object sender, EventArgs e)
        {
            if (zdrojComboBox.SelectedItem == null) return;
            if (string.IsNullOrEmpty(zdrojComboBox.SelectedItem.ToString())) return;
            ZdrojHrany pomZdroj = new ZdrojHrany();
            pomZdroj.Nazev = zdrojComboBox.SelectedItem.ToString();
            double spotrebaHrany=0;
            if (!double.TryParse(spotrebaTextBox.Text, out spotrebaHrany))
            {
                MessageBox.Show("Není vyplněna korekektně spotřeba zdroje.", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            pomZdroj.Spotreba = spotrebaHrany;
            if (hrana.PridejZdroj(pomZdroj) == 1)
                MessageBox.Show("Hrana tento zdroj již využívá. Změnte jeho spotřebu.", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            VypisZdrojuHrany();
        }

        private void VypisZdrojuHrany()
        {
            zdrojeListView.BeginUpdate();
            try
            {
                zdrojeListView.Items.Clear();
                foreach (ZdrojHrany polozka in Hrana.Data.zdrojeHrany)
                {
                    ListViewItem item = new ListViewItem(polozka.Nazev);
                    item.SubItems.Add(polozka.Spotreba.ToString());
                    zdrojeListView.Items.Add(item);
                }
            }
            finally
            {
                zdrojeListView.EndUpdate();
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (zdrojeListView.SelectedIndices.Count != 1)
            {
                MessageBox.Show("Není vybrána pouze jedna položka", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            ZdrojHrany zdroj = new ZdrojHrany();
            zdroj.Nazev=zdrojeListView.SelectedItems[0].Text;
            if (hrana.OdeberZdroj(zdroj) == 1)
                MessageBox.Show("Zdroj nebyla smazán", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else VypisZdrojuHrany();
        }

        private void zkratkaTextBox_TextChanged(object sender, EventArgs e)
        {

        }

        private void pocatekComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (zmena == true) return;
            zmena = true;
            staryKlic1 = new KlicVrchol(hrana.Vrchol1.GetKlic());
            staryKlic2 = new KlicVrchol(hrana.Vrchol2.GetKlic());
        }

        private void konecComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (zmena == true) return;
            zmena = true;
            staryKlic1 = new KlicVrchol(hrana.Vrchol1.GetKlic());
            staryKlic2 = new KlicVrchol(hrana.Vrchol2.GetKlic());
        }
    }
}
