﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace DiplomovaPrace
{
        public class ImplicitComparer : IComparer
        {
            public int Compare(object x, object y)
            {
                return ((IComparable)x).CompareTo(y);
            }
        }
        public class SortedArrayList : ArrayList
        {
            private IComparer comparer;
            private bool sorted = false;

            //pristup k vlastnosti, zda je pole utrideno
            public bool Sorted
            {
                get { return sorted; }
                set { sorted = value; }
            }
            public IComparer Comparer
            {
                get { return comparer; }
                set
                {
                    if (comparer != value)
                    {
                        comparer = value;
                        sorted = false;
                    }
                }
            }
            public override int Add(object value)
            {
                sorted = false; //metoda neni utridena
                return base.Add(value);
            }
            public SortedArrayList()
            {
                comparer = new ImplicitComparer();
            }
            public override void Sort()
            {
                if (sorted) return; //je utrideno, neni treba tridit znovu
                base.Sort(comparer);
                sorted = true; //setrideno podle comparer
            }
            public override void Sort(IComparer comparer)
            {
                base.Sort(comparer);
                this.comparer = comparer;
                sorted = true;
            }
            public override void RemoveAt(int index)
            {
                base.RemoveAt(index);
            }
            public int IndexOf(object value, IComparer comparer)
            {
                if (comparer != null)
                {
                    return base.BinarySearch(value, comparer);
                }
                return Sekvencne(value, comparer);
            }
            protected int Sekvencne(object value, IComparer comparer)
            {
                for (int i = 0; i < Count; i++)
                {
                    if ((comparer.Compare(value, base[i])) == 0) return i;
                }
                return -1;
            }
            public override void Insert(int index, object value)
            {
                base.Insert(index, value);
                sorted = false;
            }
            public int Insert(object value)
            {
                if (!sorted) Sort(comparer);
                sorted = true;
                int pozice = NajdiPozici(value, comparer);
                base.Insert(pozice, value);
                return pozice;
            }
            private int NajdiPozici(object value, IComparer comparer)
            {
                if (Count == 0) return 0;
                int r = Count;
                int l = 0;
                int aktualni;
                l = 0;
                r = Count - 1;
                if (comparer.Compare(this[l], value) >= 1) return l;
                if (comparer.Compare(this[r], value) <= 0) return (r + 1);
                while (l <= r)
                {
                    aktualni = (l + r) / 2;
                    if (comparer.Compare(this[aktualni], value) == 0) return aktualni;
                    if (comparer.Compare(this[aktualni], value) == 1)
                        r = aktualni - 1;
                    else l = aktualni + 1;
                }
                return l;
            }
        }
    }