﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace DiplomovaPrace
{
    public partial class TabulkaDialog : Form
    {
        private Graf graf;
        private int posledniKlic;

        public int PosledniKlic
        {
            get { return posledniKlic; }
            set { posledniKlic = value; }
        }
        //PoleVrcholu vrcholy;
        public TabulkaDialog(Graf graf, int posledniKlic)
        {
            
            this.graf = graf;
            InitializeComponent();
        }

        private void VypisVypis()
        {
            VypisVrcholu();
            VypisHran();
            pocetVrcholuTextBox.Text = graf.PocetVrcholu.ToString();
        }

        private void VypisVrcholu()
        {
            vypisVrcholuListView.BeginUpdate();
            try
            {
                vypisVrcholuListView.Items.Clear();
                foreach (Vrchol polozka in graf.vrcholy)
                {
                    ListViewItem item = new ListViewItem(polozka.Klic.GetKlic());
                    item.SubItems.Add(polozka.Nazev);
                    item.SubItems.Add(polozka.MaximalniCas.ToString());
                    item.SubItems.Add(polozka.PocetPredchudcu.ToString());
                    item.SubItems.Add(polozka.IndexPrvnihoPredchudce.ToString());
                    item.SubItems.Add(polozka.PocetNasledniku.ToString());
                    item.SubItems.Add(polozka.IndexPrvnihoNaslednika.ToString());
                    vypisVrcholuListView.Items.Add(item);
                }
            }
            finally
            {
                vypisVrcholuListView.EndUpdate();
            }
        }

        private void VypisHran()
        {
            vypisHranListView.BeginUpdate();
            try
            {
                vypisHranListView.Items.Clear();
                foreach (Hrana polozka in graf.naslednici)
                {
                    ListViewItem item = new ListViewItem(polozka.Vrchol1.GetKlic());
                    item.SubItems.Add(polozka.Vrchol2.GetKlic());
                    item.SubItems.Add(polozka.Data.Nazev);
                    item.SubItems.Add(polozka.Data.Ohodnoceni.ToString());
                    string retezec = "";
                    foreach (ZdrojHrany itemZdroj in polozka.Data.zdrojeHrany)
                    {
                        retezec += itemZdroj.Nazev + ",";
                    }
                    if (retezec.Length>0) retezec = retezec.Remove(retezec.Length - 1);
                    item.SubItems.Add(retezec);
                    vypisHranListView.Items.Add(item);
                }
            }
            finally
            {
                vypisHranListView.EndUpdate();
            }

        }

        private void TabulkaDialog_Load(object sender, EventArgs e)
        {
              VypisVypis();
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            using (VrcholDialog dialog = new VrcholDialog())
            {
                dialog.Vrchol = new Vrchol();
                String klic = "V" + posledniKlic.ToString("D4");
                dialog.Vrchol.Klic.SetKlic(klic);
                dialog.Vrchol.Nazev = klic;
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    if (graf.VlozVrchol(dialog.Vrchol) == 1) return;
                    posledniKlic++;           
                    ListViewItem item = new ListViewItem(dialog.Vrchol.Klic.GetKlic());
                    item.SubItems.Add(dialog.Vrchol.Nazev);
                    item.SubItems.Add(dialog.Vrchol.MaximalniCas.ToString());
                    vypisVrcholuListView.Items.Add(item);
                }
            }
            pocetVrcholuTextBox.Text = graf.PocetVrcholu.ToString();
        }

        private void upravitButton_Click(object sender, EventArgs e)
        {
            if (vypisVrcholuListView.SelectedIndices.Count != 1)
            {
                MessageBox.Show("Není vybrána pouze jedna položka", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            using (VrcholDialog dialog = new VrcholDialog())
            {
                Vrchol vrchol = new Vrchol();
                vrchol.Klic.SetKlic(vypisVrcholuListView.SelectedItems[0].SubItems[0].Text);
                int pom = graf.vrcholy.Najdi(vrchol, 0);
                if (pom <= -1)
                {
                    MessageBox.Show("Prvek nenalezen", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                dialog.Vrchol = (Vrchol)graf.vrcholy[pom];
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    vypisVrcholuListView.SelectedItems[0].SubItems[0].Text = dialog.Vrchol.Klic.GetKlic();
                    vypisVrcholuListView.SelectedItems[0].SubItems[1].Text = dialog.Vrchol.Nazev;
                    vypisVrcholuListView.SelectedItems[0].SubItems[2].Text = dialog.Vrchol.MaximalniCas.ToString();
                }
            }
        }

        private void setriditButton_Click(object sender, EventArgs e)
        {
            graf.vrcholy.Sort(new CompareKlic(true));
            VypisVypis();
        }

        private void ulozitButton_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                switch (saveFileDialog.FilterIndex)
                {
                    case 1:
                        using (StreamWriter sw = new StreamWriter(File.Create(saveFileDialog.FileName)))
                        {
                            graf.UlozDoSouboru(sw);
                        }
                        break;
                    case 2:
                        using (BinaryWriter bw = new BinaryWriter(File.Create(saveFileDialog.FileName)))
                        {
                            graf.UlozDoSouboru(bw);
                        }
                        break;
                }
            }
        }

        private void nacistButton_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                switch (openFileDialog.FilterIndex)
                {
                    case 1:
                        using (StreamReader sr = new StreamReader(File.OpenRead(openFileDialog.FileName)))
                        {
                            graf.NactiZeSouboru(sr);
                        }
                        break;
                    case 2:
                        using (BinaryReader br = new BinaryReader(File.OpenRead(openFileDialog.FileName)))
                        {
                            graf.NactiZeSouboru(br);
                        }
                        break;
                }
                VypisVypis();
                pocetVrcholuTextBox.Text = graf.PocetVrcholu.ToString();
            }
        }

        private void vlozHranuButton_Click(object sender, EventArgs e)
        {
            using (HranaDialog dialog = new HranaDialog(graf))
            {
                dialog.Hrana = new Hrana();
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    if (graf.VlozHranu(dialog.Hrana) == 1) return;
                    ListViewItem item = new ListViewItem(dialog.Hrana.Vrchol1.GetKlic());
                    item.SubItems.Add(dialog.Hrana.Vrchol2.GetKlic());
                    item.SubItems.Add(dialog.Hrana.Data.Nazev);
                    item.SubItems.Add(dialog.Hrana.Data.Ohodnoceni.ToString());
                    vypisHranListView.Items.Add(item);
                }
            }
            VypisHran();
        }

        private void reloadButton_Click(object sender, EventArgs e)
        {
            VypisVypis();
        }

        private void upravitHranuButton_Click(object sender, EventArgs e)
        {
            if (vypisHranListView.SelectedIndices.Count != 1)
            {
                MessageBox.Show("Není vybrána pouze jedna položka", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            using (HranaDialog dialog = new HranaDialog(graf))
            {
                Hrana hrana = new Hrana();
                hrana.Vrchol1.SetKlic(vypisHranListView.SelectedItems[0].SubItems[0].Text);
                hrana.Vrchol2.SetKlic(vypisHranListView.SelectedItems[0].SubItems[1].Text);
                int pom = graf.naslednici.Najdi(hrana, 0);
                if (pom <= -1)
                {
                    MessageBox.Show("Hrana nenalezena", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                dialog.Hrana = (Hrana)graf.naslednici[pom];
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    vypisHranListView.SelectedItems[0].SubItems[0].Text = dialog.Hrana.Vrchol1.GetKlic();
                    vypisHranListView.SelectedItems[0].SubItems[1].Text = dialog.Hrana.Vrchol2.GetKlic();
                    vypisHranListView.SelectedItems[0].SubItems[2].Text = dialog.Hrana.Data.Nazev;
                    vypisHranListView.SelectedItems[0].SubItems[3].Text = dialog.Hrana.Data.Ohodnoceni.ToString();                  
                }
            }
        }

        private void infoVrcholButton_Click(object sender, EventArgs e)
        {
            if (vypisVrcholuListView.SelectedIndices.Count != 1)
            {
                MessageBox.Show("Není vybrána pouze jedna položka", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            using (VypisIncidenciDialog dialog = new VypisIncidenciDialog(graf))
            {
                Vrchol vrchol = new Vrchol();
                vrchol.Klic.SetKlic(vypisVrcholuListView.SelectedItems[0].SubItems[0].Text);
                dialog.Pozice = graf.vrcholy.Najdi(vrchol, 0);
                if (dialog.Pozice <= -1)
                {
                    MessageBox.Show("Prvek nenalezen", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                dialog.Vrchol = (Vrchol)graf.vrcholy[dialog.Pozice];
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                }
            }
        }

        private void vypisVrcholuListView_DoubleClick(object sender, EventArgs e)
        {
            upravitButton_Click(sender, e);
        }

        private void vypisHranListView_DoubleClick(object sender, EventArgs e)
        {
            upravitHranuButton_Click(sender, e);
        }

        private void odstranitVrcholButton_Click(object sender, EventArgs e)
        {
            if (vypisVrcholuListView.SelectedIndices.Count != 1)
            {
                MessageBox.Show("Není vybrána pouze jedna položka", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            Vrchol vrchol = new Vrchol();
            vrchol.Klic.SetKlic(vypisVrcholuListView.SelectedItems[0].Text);
            if (graf.OdeberVrchol(vrchol) == 1)
                MessageBox.Show("Vrchol nebyl smazán", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else VypisVypis();
        }

        private void odstranitHranuButton_Click(object sender, EventArgs e)
        {
            if (vypisHranListView.SelectedIndices.Count != 1)
            {
                MessageBox.Show("Není vybrána pouze jedna položka", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
                Hrana hrana = new Hrana();
                hrana.Vrchol1.SetKlic(vypisHranListView.SelectedItems[0].SubItems[0].Text);
                hrana.Vrchol2.SetKlic(vypisHranListView.SelectedItems[0].SubItems[1].Text);
                if (graf.OdeberHranu(hrana) == 1)
                    MessageBox.Show("Hrana nebyla smazána", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                else VypisVypis();
            
            
                /*int pom = graf.naslednici.Najdi(hrana, 0);
                if (pom <= -1)
                {
                    MessageBox.Show("Hrana nenalezena", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }*/

                
            
        }

        private void nasledniciPredchudciButton_Click(object sender, EventArgs e)
        {
            predchudciListBox.BeginUpdate();
            nasledniciListBox.BeginUpdate();
            predchudciListBox.Items.Clear();
            nasledniciListBox.Items.Clear();
            foreach (Hrana polozka in graf.predchudci)
            {
                predchudciListBox.Items.Add(polozka.Vrchol1.GetKlic());
            }
            foreach (Hrana polozka in graf.naslednici)
            {
                nasledniciListBox.Items.Add(polozka.Vrchol2.GetKlic());
             }




            predchudciListBox.EndUpdate();
            nasledniciListBox.EndUpdate();
        }


       
    }
}
