﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace DiplomovaPrace
{
    public class Vrchol : ADTVrchol
    {
        private int souradniceX = 0;  
        private int souradniceY = 0;
        private String nazev;
        private bool kreslit = false;

        private double minimalniCas = 0.0;
        private double maximalniCas = 0.0;

        private double sigmaMin = 0.0;
        private double sigmaMax = 0.0;
      
        
        #region Vlastnosti
        public double SigmaMax
        {
            get { return sigmaMax; }
            set { sigmaMax = value; }
        }
        public double SigmaMin
        {
            get { return sigmaMin; }
            set { sigmaMin = value; }
        }
        public double MinimalniCas
        {
            get { return minimalniCas; }
            set { minimalniCas = value; }
        }
        public bool Kreslit
        {
            get { return kreslit; }
            set { kreslit = value; }
        }
        public int SouradniceX
        {
            get { return souradniceX; }
            set { souradniceX = value; }
        }
        public int SouradniceY
        {
            get { return souradniceY; }
            set { souradniceY = value; }
        }
        public String Nazev
        {
            get { return nazev; }
            set { nazev = value; }
        }
        public double MaximalniCas
        {
            get { return maximalniCas; }
            set { maximalniCas = value; }
        }
        #endregion
        public Vrchol() : base("")
        {
        }
        public Vrchol(String klic, String nazev, double maximalniCas, double minimalniCas) : base("")
        {
            this.Klic.SetKlic(klic);          
            this.nazev = nazev;
            this.maximalniCas = maximalniCas;
            this.minimalniCas = minimalniCas;
        }
        public Vrchol(String klic)
            : base("")
        {
            this.Klic.SetKlic(klic);
        }

        /// <summary>
        /// Zkopíruje všechny proměnné do nově alokované instance třídy Vrchol.
        /// </summary>
        /// <param name="kopieVrcholu"></param>
        public void Kopie(Vrchol kopieVrcholu)
        {
            kopieVrcholu.Klic.SetKlic(this.Klic.GetKlic());
            kopieVrcholu.Nazev = this.Nazev;
            kopieVrcholu.MaximalniCas = this.MaximalniCas;
            kopieVrcholu.MinimalniCas = this.MinimalniCas;
            kopieVrcholu.PocetNasledniku = this.PocetNasledniku;
            kopieVrcholu.PocetPredchudcu = this.PocetPredchudcu;
            kopieVrcholu.IndexPrvnihoNaslednika = this.IndexPrvnihoNaslednika;
            kopieVrcholu.IndexPrvnihoPredchudce = this.IndexPrvnihoPredchudce;
            kopieVrcholu.souradniceX = this.souradniceX;
            kopieVrcholu.souradniceY = this.SouradniceY;
            kopieVrcholu.Kreslit = this.Kreslit;
        }

         public override string ToString()
        {            
            return "Klíč: " + Klic + " Název: " + nazev + " Ohodnocení: " + maximalniCas.ToString();
        }

        #region praceSeSoubory
        public void Uloz(StreamWriter sw)
        {
            Klic.Uloz(sw);
            sw.WriteLine(nazev);
            sw.WriteLine(maximalniCas);
            sw.WriteLine(souradniceX);
            sw.WriteLine(souradniceY);
        }
        public void Nacti(StreamReader sr)
        {
            Klic.Nacti(sr);
            nazev = sr.ReadLine();
            string pomRet = sr.ReadLine();
            if (double.TryParse(pomRet, out maximalniCas) == false)
                throw new Exception("Chyba pří načítání ohodnocení vrcholu ze souboru");
            pomRet = sr.ReadLine();
            if (Int32.TryParse(pomRet, out souradniceX) == false)
                throw new Exception("Chyba pří načítání souřadnice X vrcholu ze souboru");
            pomRet = sr.ReadLine();
            if (Int32.TryParse(pomRet, out souradniceY) == false)
                throw new Exception("Chyba pří načítání souřadnice Y vrcholu ze souboru");                
        }
        public void Uloz(BinaryWriter bw)
        {
            Klic.Uloz(bw);
            bw.Write(nazev);
            bw.Write(maximalniCas);
            bw.Write(souradniceX);
            bw.Write(souradniceY);
        }
        public void Nacti(BinaryReader br)
        {
            Klic.Nacti(br);
            nazev = br.ReadString();
            maximalniCas = br.ReadDouble();
            souradniceX = br.ReadInt32();
            SouradniceY = br.ReadInt32();
        }
        #endregion

        #region grafikaUzlu
        /// <summary>
        /// Vykreslí uzel grafu.
        /// </summary>
        /// <param name="g">Grafika.</param>
        /// <param name="c1">Barva uzlu.</param>
        /// <param name="c2">Barva uzlu.</param>
        /// <param name="velikost">Velikost uzlu</param>
        /// <param name="popis">Typ popisu uzlu.</param>
        /// <param name="pismoFont">Font písma uzlu.</param>
        /// <param name="tvar">Tvar uzlu.</param>
        /// <param name="typOhodnoceni">Ohodnocení uzlu.</param>
        /// <param name="nastaveni">Nastavení aplikace.</param>
        public void VykresliUzel(Graphics g, Color c1, Color c2, int velikost, bool popis, Font pismoFont, int tvar, string typOhodnoceni, NastaveniVzhled nastaveni)
        {
            SolidBrush barvaPisma = nastaveni.BarvaPismaVrchol;
            if (velikost <= 0) return;
            velikost *= 2;
            int delka = 0;
            switch (tvar)
            {
                case 0: Rectangle rectang = new Rectangle(souradniceX - velikost, SouradniceY - velikost, velikost * 2, velikost * 2);
                    LinearGradientBrush brushUzel = new LinearGradientBrush(rectang, c1, c2,
                                 LinearGradientMode.ForwardDiagonal); 
                    g.FillEllipse(brushUzel, new Rectangle(souradniceX - velikost, SouradniceY - velikost, velikost * 2, velikost * 2));
                    break;
                case 1: rectang = new Rectangle(souradniceX - velikost*3, SouradniceY - velikost, velikost * 6, velikost * 2);
                        brushUzel = new LinearGradientBrush(rectang, c1, c2, LinearGradientMode.ForwardDiagonal); 
                        g.FillEllipse(brushUzel, new Rectangle(souradniceX - velikost*3, SouradniceY - velikost, velikost * 6, velikost * 2));
                    break;
                case 2: rectang = new Rectangle(souradniceX - velikost * 3, SouradniceY - velikost, velikost * 6, velikost * 2);
                        brushUzel = new LinearGradientBrush(rectang, c1, c2, LinearGradientMode.ForwardDiagonal);  
                        g.FillRectangle(brushUzel, new Rectangle(souradniceX - velikost*3, SouradniceY - velikost, velikost * 6, velikost * 2));
                    break;
            }
            //g.FillEllipse(b, new Rectangle(souradniceX - velikost, SouradniceY - velikost, velikost *2, velikost *2));
            delka =(int)(nazev.Length/2);
            if ((nastaveni.ZobrazeniInformaci == 0 || nastaveni.ZobrazeniInformaci == 1) && nastaveni.ZobrazitUzelNazev==true)
            {
                g.DrawString(nazev, pismoFont, barvaPisma, new PointF(souradniceX - delka * pismoFont.Size, souradniceY - pismoFont.Size - 5));
            }
            if (nastaveni.ZobrazeniInformaci == 0 && nastaveni.ZobrazitUzelCasy==true)
            {
                string retezec = String.Format("{0:F2}", minimalniCas);
                delka = retezec.Length;
                g.DrawString(retezec, pismoFont, barvaPisma, new PointF(souradniceX - pismoFont.Size * delka - 5, souradniceY));
                retezec = String.Format("{0:F2}", maximalniCas);
                g.DrawString(retezec, pismoFont, barvaPisma, new PointF(souradniceX + 5, souradniceY));
            }
            if (typOhodnoceni == "PERT" && nastaveni.ZobrazeniInformaci == 0 && nastaveni.ZobrazitUzelCasy == true)
            {
                string retezec1 = String.Format("{0:F2}", sigmaMin);
                delka = retezec1.Length;
                g.DrawString(retezec1, pismoFont, barvaPisma, new PointF(souradniceX - pismoFont.Size * delka - 5, souradniceY + pismoFont.Size + 5));
                retezec1 = String.Format("{0:F2}", SigmaMax);
                g.DrawString(retezec1, pismoFont, barvaPisma, new PointF(souradniceX + 5, souradniceY + pismoFont.Size + 5));
            }
        }
        public void KresliXorUzel(Graphics g, SolidBrush s, int velikost)
        {
            // kresli XORem
            IntPtr HDC = g.GetHdc();
            IntPtr aktPen = ApiGdi.CreatePen((int)PenStyles.PS_DASH, 2, ApiGdi.RGB(0, 0, 255));
            IntPtr oldPen = ApiGdi.SelectObject(HDC, aktPen);
            IntPtr aktBrush = ApiGdi.CreateSolidBrush(ApiGdi.RGB(140, 0, 0));
            IntPtr oldBrush = ApiGdi.SelectObject(HDC, aktBrush);

            int oldMode = ApiGdi.SetROP2(HDC, (int)RasterOps.R2_XORPEN);

            // kresli vyplnenou kruznici
            ApiGdi.Ellipse(HDC, souradniceX - velikost, souradniceY - velikost, souradniceX + velikost, souradniceY + velikost);
            ApiGdi.SelectObject(HDC, oldPen);
            ApiGdi.DeleteObject(aktPen);
            ApiGdi.SelectObject(HDC, oldBrush);
            ApiGdi.DeleteObject(aktBrush);
            ApiGdi.SetROP2(HDC, oldMode);
            g.ReleaseHdc(HDC);
        }

        #endregion
    }
}
