﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DiplomovaPrace
{
    public partial class ZdrojDialog : Form
    {
        private Zdroj zdroj;

        public Zdroj Zdroj
        {
            get { return zdroj; }
            set { zdroj = value; }
        }

        public ZdrojDialog()
        {
            InitializeComponent();
        }

        private void ZdrojDialog_Load(object sender, EventArgs e)
        {
            nazevTextBox.Text = zdroj.Nazev;
            if (string.IsNullOrEmpty(zdroj.Nazev))
            {
                pocKapacitaTextBox.Text = "0";
                aktKapacitaTextBox.Text="0";
                cenaTextBox.Text = "0";
            }
            else
            {
                nazevTextBox.ReadOnly = true;
                pocKapacitaTextBox.Text = zdroj.PuvodniKapacita.ToString();
                aktKapacitaTextBox.Text = zdroj.AktualniKapacita.ToString();
                cenaTextBox.Text = zdroj.Cena.ToString();
                if (zdroj.Obnovitelnost) obnovitelnostCheckBox.Checked = true;
                if (obnovitelnostCheckBox.Checked == false)
                {
                    pocKapacitaTextBox.Enabled = false;
                    aktKapacitaTextBox.Enabled = false;
                }
                else
                {
                    pocKapacitaTextBox.Enabled = true;
                    aktKapacitaTextBox.Enabled = true;
                }
            }
        }

        private void ZdrojDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DialogResult == DialogResult.Cancel) return;
            e.Cancel = true;
            #region chyby v zadávání údajů
            if (string.IsNullOrEmpty(nazevTextBox.Text))
            {
                MessageBox.Show("Není vyplněn název zdroje", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            double pomOhodnoceni;
            if (!double.TryParse(pocKapacitaTextBox.Text, out pomOhodnoceni))
            {
                MessageBox.Show("Není vyplněno korektně hodnota počáteční hodnota zdroje", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (!double.TryParse(cenaTextBox.Text, out pomOhodnoceni))
            {
                MessageBox.Show("Není vyplněno korektně ocenění zdroje", "Chyba v zadávání", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            #endregion
            e.Cancel = false;
        }

        private void ZdrojDialog_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (DialogResult == DialogResult.Cancel) return;
            zdroj.Nazev = nazevTextBox.Text;
            zdroj.PuvodniKapacita = double.Parse(pocKapacitaTextBox.Text);
            zdroj.AktualniKapacita = double.Parse(aktKapacitaTextBox.Text);
            zdroj.Cena = double.Parse(cenaTextBox.Text);
            if (obnovitelnostCheckBox.Checked == true) zdroj.Obnovitelnost = true;
            else zdroj.Obnovitelnost = false;

        }

        private void pocKapacitaTextBox_TextChanged(object sender, EventArgs e)
        {
            aktKapacitaTextBox.Text = pocKapacitaTextBox.Text;
        }

        private void obnovitelnostCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (obnovitelnostCheckBox.Checked == false)
            {
                pocKapacitaTextBox.Enabled = false;
                aktKapacitaTextBox.Enabled = false;
            }
            else
            {
                pocKapacitaTextBox.Enabled = true;
                aktKapacitaTextBox.Enabled = true;
            }
        }
    }
}
